<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;

set_time_limit(60 * 10);

//Use file functions for uncompressed files, and gz functions for compressed files
//If the user has not tampered with a backup it should still be compressed though.
if ($plainSql) {
	$open = 'fopen';
	$read = 'fread';
	$close = 'fclose';
} else {
	$open = 'gzopen';
	$read = 'gzread';
	$close = 'gzclose';
}

//Start out handling statements as SQL
//Eventually we will move on to (re)creating the documents folder, as which point
//this variable will be set to false
$runningSQL = true;
$chunks = '';
$statementNo = 0;
$reading = true;
$state = NEXTPLEASE;

//$docpath = CMS_DOCSTORE_DIR. '/'. SITENAME;
//if (!is_readable($docpath) || !is_writeable($docpath)) {
	//$docpath = false;
//}

//Open the file
$g = $open($filename, 'rb');
$f;

if (!$g = $open($filename, 'rb')) {
	return false;
}

//Read from the file, a little at a time.
//Loop until we've read the file
while($reading) {
	//Grab a little bit from the file, and add it to whatever we already had.
	$reading = ($chunk = $read($g, READ_BACKUP_CHUNK_SIZE)) !== '';
	$chunks .= $chunk;
	
	//Split what we have into individual SQL statements
	$statements = explode(";\n", str_replace("\r", '', $chunks));
		//There should be no carriage returns in the backup; unless the user has tampered with the
		//file on a windows machine. But then if that's the case, because I'm feeling nice, I'll strip them
		//out for the user, rather than have their restore fail.
	$count = count($statements)-1;
	
	//If we have any complete statements, execute them
	for ($i = 0; $i < $count; ++$i) {
		
		//Stop running SQL statements when we see a statement called 'END OF SQL'
		if ($runningSQL && $statements[$i] == 'END OF SQL') {
			$runningSQL = false;

			//Check for any failures
			if (count($failures)) {
				//If we failed at any point, don't write over the actual tables with the temp values!
				//We'll need to delete the temporary tables though.
				foreach(lookupImportedTables(DB_NAME_PREFIX) as $importTable) {
					@mysql_query('DROP TABLE IF EXISTS `'. $importTable. '`');
				}
				foreach(lookupImportedTables(DB_NAME_PREFIX_GLOBAL) as $importTable) {
					@mysql_query('DROP TABLE IF EXISTS `'. $importTable. '`');
				}
				
				return false;
			}
		}
		
		//If we are still running SQL statements then handle SQL statements
		//If we have moved on, work differently
		if ($runningSQL) {
			//If the length is smaller than 20 chars, this is probably just some whitespace and can be ignored
			if (strlen($statements[$i]) < 20) {
				continue;
			}
			
			//Keep count of the statements for debugging purposes
			++$statementNo;
			
			
			//Check the SQL statement, looking for the table prefix it is using.
			
			//DB_NAME_PREFIX is for tables in the local database; i.e. most tables. They are always restored.
			if (strpos($statements[$i], "[['DB_NAME_PREFIX']]") !== false) {
				$prefixInFile = "[['DB_NAME_PREFIX']]";
				$actualPrefix = DB_NAME_PREFIX;
			
			//DB_NAME_PREFIX_GLOBAL is for tables that would be in the global database in multisite, but are actually
			//stored in the same database for singlesite, and need to be in the singlesite backups.
			//They are not included in backups from multisite sites...
			} elseif (strpos($statements[$i], "[['DB_NAME_PREFIX_GLOBAL']]") !== false) {
				
				//...and even if there, should not be restored to multisite sites.
				if (checkModuleReady('multisite.inc.php')) {
					continue;
				}
				
				$prefixInFile = "[['DB_NAME_PREFIX_GLOBAL']]";
				$actualPrefix = DB_NAME_PREFIX_GLOBAL;
			
			//GLOBAL_NOT_NORMALLY_RESTORED is used for the public_urls and site_registry tables
				//There would be no point in restoring the public_urls or site_registry tables; if they
				//weren't set up correctly then you wouldn't be able to get to this page and run the backup
				//in the first place, and if the restore broke them then you could break your site!
				//They are only in the backup file to help phpMyAdmin users
			} elseif (strpos($statements[$i], "[['GLOBAL_NOT_NORMALLY_RESTORED']]") !== false) {
				continue;
			
			//If we didn't see any of those prefixes, report it
			} else {
				$failures[] = 'Statement '. $statementNo. ":\n". $statements[$i]. "\n\nNo Prefix or Prefix not in the correct format\n\n\n";
				continue;
			}

			
			//Attempt to run the SQL. Note there are no further checks done on it, so of course it may not be valid.
			//I'll also check whether it failed or succeeded
			$success = @mysql_query(str_replace($prefixInFile, $actualPrefix. 'i_m_p_', $statements[$i]));
						
			//Even if it failed, we'll keep running the SQL statements. But we'll log the failures and report at the end.
			if (!$success) {
				$failures[] = 'Statement '. $statementNo. ":\n". $statements[$i]. "\n\nDatabase query error ". mysql_errno(). ": ". mysql_error()."\n\n\n";
			}
			
		//If we have moved on from SQL statements, move on to the DocStore creation statements
		//} elseif ($docpath) {
			//Watch out for DIR and FILE labels, that tell us if the next statement is the path for a directory or a file
			//if ($state == NEXTPLEASE) {
				//if ($statements[$i] == 'DIR') {
					//$state = CREATEDIR;
				//} elseif ($statements[$i] == 'FILE') {
					//$state = FILENAME;
				//}
				
			//Create the specified directory, if it doesn't already exist
			//} elseif ($state == CREATEDIR) {
				//if (!file_exists(docstoreDirectoryPath($statements[$i]))) {
					//mkdir(docstoreDirectoryPath($statements[$i]));
				//}
				//The next statement should be the start of a new directory or file
				//$state = NEXTPLEASE;
				
			//Open the specified file for writing
			//} elseif ($state == FILENAME) {
				//$f = fopen(docstoreDirectoryPath($statements[$i]), 'wb');
				
				//The next statement should be the contents of the file
				//$state = FILECONTENTS;
				
			//Write the contents of the file, then close it.
			//} elseif ($state == FILECONTENTS) {
				//Write the (remaining) data to the file
				//fwrite($f, hex2bin($statements[$i]));
				
				//Close the file
				//fclose($f);
				
				//We're done; the next statement should be the start of a new directory or file
				//$state = NEXTPLEASE;
			//}
		}
	}

	//Remember any incomplete statements for the next read
	if ($state != FILECONTENTS) {
		$chunks = $statements[$count];
	} else {
		//Have an exception for writing files - we can stream the data straight into the file
		//without needing to hold it back.
		if (strlen($statements[$i]) % 2 == 0) {
			fwrite($f, hex2bin($statements[$i]));
			$chunks = '';
		} else {
			//The hexadecimal values need to be in pairs; if there is an odd number then remove the
			//last digit and save it for next time.
			fwrite($f, hex2bin(substr($statements[$i], 0, -1)));
			$chunks = substr($statements[$i], -1);
		}
	}

}
$close($g);
//Close the last file, if it was left open
if ($state == FILECONTENTS) {
	//Close the file
	fclose($f);
}

//Finally, we'll need to remove the existing tables and copy over 
foreach(lookupImportedTables(DB_NAME_PREFIX) as $importTable) {
	
	//Work out the name of the actual table by removing the 'i_m_p_' prefix
	$actualTable = str_replace(DB_NAME_PREFIX. 'i_m_p_', DB_NAME_PREFIX, $importTable);
	
	//Remove the currently existing table
	my_mysql_query('DROP TABLE IF EXISTS `'. $actualTable. '`');
	
	//Copy over the actual table
	my_mysql_query('RENAME TABLE `'. $importTable. '` TO `'. $actualTable. '`');
}

if (!checkModuleReady('multisite.inc.php')) {
	foreach(lookupImportedTables(DB_NAME_PREFIX_GLOBAL) as $importTable) {
		
		//Work out the name of the actual table by removing the 'i_m_p_' prefix
		$actualTable = str_replace(DB_NAME_PREFIX_GLOBAL. 'i_m_p_', DB_NAME_PREFIX_GLOBAL, $importTable);
		
		//Remove the currently existing table
		my_mysql_query('DROP TABLE IF EXISTS `'. $actualTable. '`');
		
		//Copy over the actual table
		my_mysql_query('RENAME TABLE `'. $importTable. '` TO `'. $actualTable. '`');
	}
}

return empty($failures);
?>