<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

mb_internal_encoding('UTF-8');
error_reporting(E_ALL ^ E_NOTICE);

if (!defined('NOT_ACCESSED_DIRECTLY')) exit;

//Attempt to set the working directory to the CMS Root, if it is not already
$isFilelimit = 4;
while (!is_file('visitorheader.inc.php')) {
	if (!--$isFilelimit) {
		echo 'Incorrect working directory set.';
		exit;
	} else {
		chdir('../');
	}
}
unset($isFilelimit);


//Attempt to calculate the CMS_ROOT and SUBDIRECTORY, if they're not set already
if (!function_exists('windowsServer')) {
	function windowsServer() {
		return (stristr($php_uname = php_uname('s'), 'win')) && $php_uname != 'Darwin';
	}
}

if (!defined('CMS_ROOT')) {
	if (windowsServer()) {
		define('CMS_ROOT', substr(__FILE__, 0, -strlen('cmsincludes/cms.inc.php')));
	} else {
		define('CMS_ROOT', substr(realpath(__FILE__), 0, -strlen('cmsincludes/cms.inc.php')));
	}
}

if (!is_file(CMS_ROOT. 'visitorheader.inc.php')) {
	echo 'Your CMS_ROOT value is not correctly set. \''. CMS_ROOT. '\' is not the correct path to the CMS.';
	exit;
}

if (!defined('SUBDIRECTORY')) {
	//Get the original included filepath
	$file = false;
	foreach (debug_backtrace() as $backtrace) {
		$file = $backtrace['file'];
	}
	$file = substr($file, strlen(CMS_ROOT));
	
	//Get the included location
	$self = $_SERVER['PHP_SELF'];
	
	//If the two don't match up, try chopping the filenames off the ends of the path
	if (substr($self, -strlen($file)) != $file) {
		$pos = max(strrpos($file, '/'), strrpos($file, '\\'), -1);
		$file = substr($file, 0, $pos? $pos + 1 : 0);
		
		$pos = max(strrpos($self, '/'), strrpos($self, '\\'), -1);
		$self = substr($self, 0, $pos? $pos + 1 : 0);
		
		unset($pos);
	}
	
	//Trim the included location by the filepath to get the current SUBDIRECTORY
	if (strlen($file)) {
		$subdir = substr($self, 0, -strlen($file));
	} elseif ($self) {
		$subdir = $self;
	} else {
		$subdir = '/';
	}
	
	define('SUBDIRECTORY', $subdir);
	
	unset($backtrace);
	unset($file);
	unset($self);
	unset($subdir);
}


//Include any module functions in Plugins
foreach (scandir('plugins/') as $pluginDir) {
	if (substr($pluginDir, 0, 1) != '.'
	 && is_dir($pluginDir = 'plugins/'. $pluginDir. '/modules/functions/')) {
		foreach (scandir($pluginDir) as $pluginModule) {
			if (substr($pluginModule, 0, 1) != '.' && substr($pluginModule, -8) == '.inc.php'
			 && is_file($pluginDir. $pluginModule)) {
				require_once $pluginDir. $pluginModule;
			}
		}
	}
}
unset($pluginDir);
unset($pluginModule);


//Add includes from the CMS
require_once 'documentation_for_developers/helper_functions.inc.php';
require_once 'documentation_for_developers/core_functions.inc.php';
require_once 'cmsincludes/cache.inc.php';
require_once 'cmsincludes/mysubs.inc.php';


//Old module functionality from 5.0 - These are depricated; Plugins should be used instead!
function checkModuleReady($filename) {
	return false;
}

function checkModuleReadyAndInclude($filename) {
	return false;
}





//Get the latest revision number from latest_revision_no.inc.php
function getLatestRevisionNumber() {
	return LATEST_REVISION_NO;
}

//Get the CMS version number from latest_revision_no.inc.php
function getTribiqCmsVersionNumber() {
	
	return TRIBIQ_CMS_VERSION. (
		is_dir(CMS_ROOT. '.svn')? (
			@is_dir(CMS_ROOT. '../../ondemand')?
				'_ondemand'
			  : '_dev')
			  : '');
}




//Write the URLBasePath, and other related JavaScript variables, to the page
function CMSWritePageHead($prefix, $slotContents = array(), $siteConfig = false, $skinId = false, $welcome = false) {
	require 'cmsincludes/visitor/CMSWritePageHead.fun.php';
}

function CMSWritePageBody($cID = false, $cType = false, $status = false) {
	require 'cmsincludes/visitor/CMSWritePageBody.fun.php';
}

//Write the URLBasePath, and other related JavaScript variables, to the page
function CMSWritePageFoot($prefix, $slotContents = array(), $siteConfig = false, $welcome = false) {
	require 'cmsincludes/visitor/CMSWritePageFoot.fun.php';
}



//Set up the floating box functionality
function setupFloatingBoxes($showInfoBox = false, $welcome = false) {
	
	echo '
<div id="plgslt_Fancybox"></div>
<div class="floating_box">
	<div id="tribiq_fb1"></div>
	<div id="tribiq_fb2"></div>
	<div id="tribiq_fb3"></div>
	<div id="tribiq_fb4"></div>
	<div id="tribiq_fb5"></div>
</div>';
	
	if($welcome || checkPriv()) {
		setupAdminFloatingBoxes($showInfoBox);
	}
}


//Check if we should display a plugin slot, and return the plugin in the slot if so. 
function slot($slotName) {
	
	//Replacing anything non-alphanumeric with an underscore
	$slotName = HTMLId($slotName);
	
	//Get global variables defined in index.php and the Template head
	//Normaly it's our policy to pass these in as inputs, and reference them using the global syntax,
	//but to make life easier for Template developers calling this function we'll reference them using globals here
	global $cID, $cType, $cVersion, $slotContents;
	
	
	//Start the plugin if it is there, then return it to the Template
	if (issetArrayKey($slotContents, $slotName) && issetArrayKey($slotContents[$slotName], 'class')) {
		$slotContents[$slotName]['found'] = true;
		
		$slotContents[$slotName]['class']->start($slotContents[$slotName]['init']);
		
		return $slotContents[$slotName]['class'];
	
	//If we didn't find a plugin, but we're in admin mode, 
	//return the basic Template plugin so that the controls are still displayed to the admin
	} elseif (checkPriv()) {
		if (!issetArrayKey($slotContents, $slotName)) {
			$slotContents[$slotName] = array('new_slot' => true, 'found' => true);
		} else {
			$slotContents[$slotName]['found'] = true;
		}

		//Return false if the variables needed have not been set up correctly.
		//(E.g. might happen on an error page).
		if (!$cID || !$cType || !$cVersion) {
			return false;
		}
	
		$emptySlot = new plugin_base_class;
		$emptySlot->setInstance(
			array($cID, $cType, $cVersion, false, $slotName, false, false, false, false, false, false, false, false, false, false, false),
			arrayKey($slotContents[$slotName], 'error'));
		$emptySlot->start(false);
		
		return $emptySlot;
	
	} else {
		return false;
	}
}

//Did we use all of our slots..?
function checkSlotsWereUsed($cID, $cType, $cVersion, $templateName, $familyName) {
	
	//Only run this in admin mode
	if (!checkPriv()) {
		return;
	}
	
	global $slotContents;
	
	//Look up the slots on the page
	$sql = "
		SELECT slot_name
		FROM ". DB_NAME_PREFIX. "template_slot_link
		WHERE template_family_name = '". mysql_real_escape_string($familyName). "'
		  AND template_filename = '". mysql_real_escape_string($templateName). "'";
	$result = my_mysql_query($sql);
	
	$newSlots = array();
	$missingSlots = array();
	$existingSlots = array();
	while($row = mysql_fetch_assoc($result)) {
		$existingSlots[$row['slot_name']] = true;
	}
	
	//Look through $slotContents, looking for newly found slots or missing slots
	foreach ($slotContents as $slotName => &$details) {
		if (issetArrayKey($details, 'new_slot') && !issetArrayKey($existingSlots, $slotName)) {
			$newSlots[] = $slotName;
		} elseif (!issetArrayKey($details, 'found') && issetArrayKey($existingSlots, $slotName)) {
			$missingSlots[] = $slotName;
		}
	}
	
	//Remove any missing slots from the database
	foreach($missingSlots as $slotName) {
		$sql = "
			DELETE FROM ". DB_NAME_PREFIX. "template_slot_link
			WHERE slot_name = '". mysql_real_escape_string($slotName). "'
			  AND template_family_name = '". mysql_real_escape_string($familyName). "'
			  AND template_filename = '". mysql_real_escape_string($templateName). "'";
		my_mysql_query($sql);
	}
	
	//Add any new slots to the database
	foreach($newSlots as $slotName) {
		$sql = "
			REPLACE INTO ". DB_NAME_PREFIX. "template_slot_link SET
				slot_name = '". mysql_real_escape_string($slotName). "',
				template_family_name = '". mysql_real_escape_string($familyName). "',
				template_filename = '". mysql_real_escape_string($templateName). "'";
		my_mysql_query($sql);
	}
	
	//Refresh the page to show the new slots, if needed
	if (count($newSlots)) {
		echo '
			<script type="text/javascript">
				document.location.href = "', addslashes(linkToItem($cID, $cType, false, true, '', false)), '";
			</script>';
	}
}






//Return the a name of a CSS class to use, depending on whether a field is set or not
function mandatory($filled) {
	return $filled? 'class="mandatory_correct"' : 'class="mandatory"';
}

//There is no way to target another frame with a header('location: www.blah.com'); call.
//This function is as close as we're going to get, alas
function topHeaderLocation($url, $prefix = 'top') {
	echo 
'<html>
	<body>
		<script language="JavaScript">
			', $prefix, '.location.href = "', $url, '";
		</script>
		<a href="', $url, '" target="_main">Please click here to continue to the correct location</a>
	</body>
</html>';
	exit;
}


/*
 *   function fillOptions
 *	 returns the rows of a table
 *   as a string ready for use
 *   inside a select tag
 */
function fillOptions( $table, $value, $text, $default = '', $refiner = '', $useGetPhrase = true ) {
	$options = '';
	if ( ! empty( $refiner ) ) {
		$refiner = 'where ' . $refiner;
	}
	$sql = '
		select ' . $value . ', ' . $text . '
		from ' . $table . '
		' . $refiner . ' 
		order by ' . $text;
	$result = my_mysql_query( $sql );
	while ( list( $id, $text ) = mysql_fetch_row( $result ) ) {
		if ( $id == $default ) {
			$selected = ' selected = "selected"';
		} else {
			$selected = '';
		}
		if ( $useGetPhrase ) {
			$options .= '<option value="' . $id . '"' . $selected.'>' . getPhrase( $text ) . '</option>';
		} else {
			$options .= '<option value="' . $id . '"' . $selected.'>' .  $text . '</option>';
		}
	}
	return $options;
}
/*
 *   function enumToOption
 *	 returns the options of enum field
 *   as a string ready for use
 *   inside a select tag
 */

function enumToOption( $table, $field, $defaultValue = '') {
	$result = my_mysql_query( 'describe ' . $table . ' ' . $field );
	$row =  mysql_fetch_array( $result );
	#removes starting 'enum(' and the closing ')'
	$enumValues = substr ( $row['Type'] , 5, -1 );
	#removes any single quotes from the string
	$enumValues = str_replace( '\'', '', $enumValues );
	#makes an array for processing options
	$enumValues = explode ( ',', $enumValues );
	$options = '';
	foreach ( $enumValues as $value ) {
		if ( $value == $defaultValue ) {
			$options .= '<option value="' . $value .'" selected="selected">' . getPhrase( '_' . $value ) . '</option>';
		} else { 
			$options .= '<option value="' . $value .'">' . getPhrase( '_' . $value ) . '</option>';
		}
	}
	return $options;
}


function maskString($str,$char_mask="*") {
	$mask = "";
	for ($i=0; $i < strlen($str); $i++) {
		$mask .= $char_mask;
	}
	return $mask;
}


//Handle error pages

//Attempt to work out a Template and a skin for an error page
function ensureTemplateExists(&$templateId, &$templateFamily, $siteConfig, $requestedContentType = 'html') {
	
	//Look in the site configs for a default Template or a default Template Family, if they have been set
	if (!$templateFamily) {
		$templateFamily = $siteConfig["default_template_family"];
	}
	
	if (!$templateId) {
		foreach (getContentTypes($requestedContentType) as $contentType) {
			$templateId = $contentType['default_template_id'];
			break;	
		}
	}
	
	//Given what we know, find a Template and a Template Family as best we can.
	//Give priority to matching Template ids, matching family names, active Templates,
	//and then html type Templates in that order
	$sql = "
		SELECT family_name, template_id, filename
		FROM ". DB_NAME_PREFIX. "templates
		ORDER BY
			content_type = '". mysql_real_escape_string($requestedContentType). "' DESC,
			template_id = ". (int) $templateId. " DESC,
			family_name = '". mysql_real_escape_string($templateFamily). "' DESC,
			template_id";
	
	$result = my_mysql_query($sql);
	$row = mysql_fetch_assoc($result);
	
	$templateId = $row['template_id'];
	$templateFamily = $row['family_name'];
	
	return $row['filename'];
}


/* Special case for an Admin/Author creating a new site */
function showStartSitePageIfNeeded($cID, $cType, &$siteConfig) {
	$onDefaultHomepage = 
		$cID == ifNull(arrayKey($siteConfig, 'default_homepage_id'), 1)
	 &&	$cType == 'html';
	
	if (checkPriv() && (!checkRowExists('content_html', array()))) {
		echo '
			<html>
				<head>
					<title>Welcome to Tribiq CMS</title>
				</head>
				<body width="100%" height="100%">';
		
		if (checkPriv('_PRIV_CREATE_FIRST_DRAFT')) {
			echo '
					<div style="padding:auto; margin:auto; text-align: center; position: absolute; top: 50%; left: 50%;">
						<a href="admin/storekeeper.php#tribiq__languages" title="', getPhrase('Start your site'), '" target="_blank">
							<img style="position: relative; bottom: 77px; right: 141px;" width="283" height="154" border="0"
								 title="', getPhrase('Start your site'), '" alt="', getPhrase('Start your site'), '"
								 src="admin/images/startSitePage_full_graphic.gif"/>
						</a>
					</div>';
		} else {
			echo '
					<h1>'. getPhrase('_ADMINISTRATOR') .'</h1>
					<p>'. getPhrase('_CMSERROR_NEW_SITE_INFO_NOAUTHOR') .'</p>';
		}
		
		echo '
				</body>
			</html>';
	
		exit;
	
	} elseif (!checkPriv() && ($onDefaultHomepage || !checkRowExists('content_html', array('status' => 'published')))) {
		echo '
			<html>
				<head>
					<title>Welcome to Tribiq CMS</title>
				</head>
				<body width="100%" height="100%">
					<div style="padding:auto; margin:auto; text-align: center; position: absolute; top: 50%; left: 50%;">
						<img style="position: relative; bottom: 90px; right: 179px;" width="359" height="180" border="0" usemap="#Map"
							 title="', getPhrase('A Tribiq CMS site is being built at this location.'), '"
							 alt="', getPhrase('A Tribiq CMS site is being built at this location.'), '"
							 src="admin/images/tribiq_in_build.gif"/>
						<map name="Map" id="Map">
							<area href="admin/" shape="rect" coords="250,125,315,145"/>
						</map>
					</div>
				</body>
			</html>';
	
		exit;
	}
}


//Display the error message itself
function displayErrorPage() {
	return require 'cmsincludes/visitor/displayErrorPage.fun.php';
}




function loadSiteConfig() {

	/* Load the full site config (everything except DB settings is held in the database) */
	$siteConfig = array();
	$sql = "SELECT name, value
			FROM ".DB_NAME_PREFIX."site_settings";
	$result = my_mysql_query ($sql);
	while ($row = mysql_fetch_array ($result)) {
		$siteConfig[$row['name']] = $row['value'];
	}

	// Returns an array $siteConfig which contains all the site-wide settings
	return $siteConfig;
}

function setting($settingName) {

	//Look up the value of a specific setting
	$sql = "SELECT value
			FROM ".DB_NAME_PREFIX."site_settings
			WHERE name = '" . mysql_real_escape_string($settingName) . "'";
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_assoc($result)) {
		return $row['value'];
	} else {
		return false;
	}
}

function getSiteConfig($settingName) {
	return setting($settingName);
}

function getSiteRegistry () {
	connectGlobalDB();

	$sql = "SELECT sr.sitename,
				sr.goto_admin_url,
				sr.force_public_url_refresh,
				COUNT(pu.sitename) AS public_urls
			FROM " . DB_NAME_PREFIX_GLOBAL . "site_registry AS sr
			LEFT JOIN " . DB_NAME_PREFIX_GLOBAL . "public_urls AS pu
				ON sr.sitename = pu.sitename
			GROUP BY sr.sitename,
				sr.goto_admin_url,
				sr.force_public_url_refresh";
	$result = my_mysql_query($sql);

	connectLocalDB();

	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			$site_registry[] = $row;
		}

		return $site_registry;
	} else {
		return false;
	}
}



//Permission functions

//Check to see if an Admin has a certain privilege
function checkPriv($action = false, $welcomePage = false) {
	
	//If the Admin is not logged in to this site, then they shouldn't have Admin rights here
	if ($_SESSION['admin_logged_into_site'] != SUBDIRECTORY. SITENAME) {
		return false;
	}
	
	//If the Admin hasn't passed the welcome page, then they shouldn't be able to use their Admin rights anywhere except the welcome page
	if (!$welcomePage && !session('admin_logged_in')) {
		return false;
	}
	
	//No action specified? Just checking if the admin is logged in will do.
	if ($action === false) {
		return true;
	
	//The update database permission is a special case.
	} elseif ($action == '_PRIV_UPDATE_DATABASE') {
		//We had a previous permissions system before our current one.
		//Mostly this should not be used, but we will still need to keep the old logic for
		//checking if someone has database update rights, otherwise old sites would never be
		//able to be upgraded!
		return issetArrayKey($_SESSION, 'privs', '_PRIV_UPDATE_DATABASE') || session('old_perm_sysadmin');
	
	//Otherwise check if the Admin has this specific privilege
	} else {
		return issetArrayKey($_SESSION, 'privs', $action);
	}
}

function checkAdminPriv ($adminId, $adminAuthType, $action) {
	if ($adminAuthType=="local") {
		$sql = "SELECT admin_id
				FROM " . DB_NAME_PREFIX . "action_admin_link
				WHERE action_name = '" . mysql_real_escape_string($action) . "'
					AND admin_id = " . (int) $adminId;
					
		$result = my_mysql_query($sql);
		
		if (mysql_num_rows($result)>0) {
			return true;
		} else {
			return false;
		}
	} elseif ($adminAuthType=="super") {
		connectGlobalDB();
		$sql = "SELECT admin_id
				FROM " . DB_NAME_PREFIX_GLOBAL . "action_superadmin_link
				WHERE action_name = '" . mysql_real_escape_string($action) . "'
					AND admin_id = " . (int) $adminId;
					
		$result = my_mysql_query($sql);
		connectLocalDB();
		
		if (mysql_num_rows($result)>0) {
			return true;
		} else {
			return false;
		}		
	}
}

//A more agressive version of check priv that stops all execution if an admin does not have the requested privilege
function exitIfNotCheckPriv($action = false) {
	if (!checkPriv($action)) {
		exit;
	} else {
		return true;
	}
}






function custom_function_exists($function_name) {
	if (function_exists($function_name)) {
		return true;
	} else {
		return false;
	}
}

//Declare a function to check if something exists in the database
function checkRowExists($table, $ids, $prefix = DB_NAME_PREFIX, $cols = false, $notZero = false, $multiple = false, $delete = false, $orderBy = array()) {
	
	if (!is_array($ids)) {
		$ids = array('id' => $ids);
	}
	
	if ($delete) {
		$sql = "
			DELETE";
	} elseif (is_array($cols)) {
		$sql = "";
		foreach ($cols as $col) {
			$sql .= ($sql? "," : "SELECT "). mysql_real_escape_string($col). " AS `". mysql_real_escape_string($col). "`";
		}
	} elseif ($cols !== false) {
		$sql = "
			SELECT ". mysql_real_escape_string($cols);
	} else {
		$sql = "
			SELECT 1";
	}
	
	$sql .= "
			FROM ". mysql_real_escape_string($prefix. $table);
	
	$first = true;
	foreach($ids as $col => $val) {
		if ($first) {
			$sql .= "
			WHERE ";
		} else {
			$sql .= "
			  AND ";
		}
		$first = false;
		
		$sql .= mysql_real_escape_string($col). " = ";
		
		if (is_numeric($val)) {
			$sql .= (int) $val;
		} else {
			$sql .= "'". mysql_real_escape_string($val). "'";
		}
	}
	
	if ($notZero && !is_array($cols) && $cols !== false) {
		if ($first) {
			$sql .= "
			WHERE ";
		} else {
			$sql .= "
			  AND ";
		}
		$sql .= mysql_real_escape_string($cols). " != 0";
	}
	
	if ($orderBy) {
		if (!is_array($orderBy)) {
			$orderBy = array($orderBy);
		}
		$first = true;
		foreach ($orderBy as $col) {
			if ($first) {
				$sql .= "
				ORDER BY ". mysql_real_escape_string($col);
			} else {
				$sql .= ",
					". mysql_real_escape_string($col);
			}
			$first = false;
		}
	}
	
	if (!$multiple) {
		$sql .= "
			LIMIT 1";
	}
	
	$result = my_mysql_query($sql);
	if ($delete) {
		return mysql_affected_rows();
	} elseif ($multiple) {
		return $result;
	} elseif (!$row = mysql_fetch_assoc($result)) {
		return false;
	} elseif (is_array($cols)) {
		return $row;
	} elseif ($cols !== false) {
		return $row[$cols];
	} else {
		return true;
	}
}

function setRow($table, $values, $ids = array(), $prefix = DB_NAME_PREFIX, $insertIfNotPresent = true) {
	
	if (!is_array($ids)) {
		$ids = array('id' => $ids);
	}
	
	$sql = "";
	foreach ($values as $col => $val) {
		$sql .= ($sql? "," : ""). "
			`". mysql_real_escape_string($col). "` = ";
		if (is_numeric($val)) {
			$sql .= (int) $val;
		} else {
			$sql .= "'". mysql_real_escape_string($val). "'";
		}
	}
	
	if (!empty($ids) && checkRowExists($table, $ids, $prefix)) {
		$returnValue = false;
		
		if (!empty($values)) {
			$sql = "
				UPDATE ". mysql_real_escape_string($prefix. $table). " SET ".
				$sql;
			
			$first = true;
			foreach($ids as $col => $val) {
				if ($first) {
					$returnValue = $val;
					$first = false;
					$sql .= " 
					  WHERE ";
				} else {
					$sql .= " 
						AND ";
				}
				if (is_numeric($val)) {
					$sql .= mysql_real_escape_string($col). " = ". (int) $val;
				} else {
					$sql .= mysql_real_escape_string($col). " = '". mysql_real_escape_string($val). "'";
				}
			}
			
			my_mysql_query($sql);
		}
		
		if ($insertIfNotPresent) {
			return $returnValue;
		} else {
			return true;
		}
	
	} elseif ($insertIfNotPresent) {
		foreach ($ids as $col => $val) {
			$sql .= ($sql? "," : ""). "
				`". mysql_real_escape_string($col). "` = ";
			if (is_numeric($val)) {
				$sql .= (int) $val;
			} else {
				$sql .= "'". mysql_real_escape_string($val). "'";
			}
		}
		
		$sql = "
			INSERT INTO ". mysql_real_escape_string($prefix. $table). " SET ". $sql;
		
		my_mysql_query($sql);
		return mysql_insert_id();
	
	} else {
		return false;
	}
}


/*
	Functions to help make dynamic forms
*/


//Get a description of a table
function getFields($prefix, $tableName, $addPasswordConfirm = false) {
	$fields = array();
	$sql = "DESC ". mysql_real_escape_string($prefix. $tableName);
	$result = my_mysql_query($sql);

	while($field = mysql_fetch_assoc($result)) {
		$field['Table'] = $tableName;
		
		$field['Date'] = strpos($field['Type'], 'date') !== false;
		
		$field['Numeric'] = strpos($field['Type'], 'enum') === false
						&& (strpos($field['Type'], 'int') !== false
						 || strpos($field['Type'], 'double') !== false
						 || strpos($field['Type'], 'float') !== false);
		
		$fields[$field['Field']] = $field;
		
		if ($field['Field'] == 'password' && $addPasswordConfirm) {
			$fields[$field['Field']]['Type'] = 'password';
			$fields['password_reconfirm'] = $fields[$field['Field']];
			$fields['password_reconfirm']['Table'] = '';
			$fields['password_reconfirm']['Type'] = 'password_reconfirm';
		}
	}
	
	return $fields;
}

	
//A function to help with saving the returns from these fields
function addFieldToSQL(&$sql, $table, $field, $values, $editing, $details = array()) {
	
	if ($sql) {
		$sql .= ",";
	} elseif ($editing) {
		$sql = "
		UPDATE ". $table. " SET";
	} else {
		$sql = "
		REPLACE INTO ". $table. " SET";
	}
	
	$sql .= "
			". $field. " = ";
	
	//Attempt to save empty dates correctly in strict mode
	if ($details['Date'] && strlen((string) $values[$field] < 8)) {
		if ($details['Null'] == 'Yes') {
			$values[$field] = '';
		} else {
			$values[$field] = '0000-00-00';
		}
	}
	
	//Convert empty strings to NULLs if possible
	if ($values[$field] === '' && $details['Null'] == 'Yes') {
		$sql .= "NULL";
	
	//Otherwise convert empty strings to 0s for non-string fields
	} elseif (!$values[$field] && $details['Numeric']) {
		$sql .= "0";
	
	//Make sure Numeric values are actually numeric
	} elseif ($details['Numeric']) {
		$sql .= (int) $values[$field];
	
	//Otherwise use mysql_real_escape_string
	} else {
		$sql .= "'". mysql_real_escape_string($values[$field]). "'";
	}
}


//Take a string, and add any defined constants in using the [[CONSTANT_NAME]] format
function addConstantsToString($sql, $replaceUnmatchedConstants = true) {
	//Get a list of defined constants
	$constants = get_defined_constants(true);
	
	//Ignore the system constants, and split them up into keys and values
	$constantValues = array_values($constants['user']);
	$constants = array_keys($constants['user']);
	
	//Add our standard substitution pattern to the keys
	array_walk($constants, create_function('&$value, $key', '$value = "[[". $value. "]]";'));
	
	if ($replaceUnmatchedConstants) {
		return preg_replace('/\[\[\w+\]\]/', '0', str_replace($constants, $constantValues, $sql));
	} else {
		return str_replace($constants, $constantValues, $sql);
	}
}



//Include the function definitions for a Content Type. Returns false if there was a problem
function includeContentType($cType) {
	$sql = "
		SELECT plugin_id
		FROM ". DB_NAME_PREFIX. "contenttypes
		WHERE content_type_id = '". mysql_real_escape_string($cType). "'";
	$result = my_mysql_query ($sql);
	
	if (($row = mysql_fetch_assoc($result))
	 && ($plugin = canActivatePlugin($row['plugin_id'], $fetchBy = 'id', $activate = true))) {
	 	require_once CMS_ROOT. 'cmsincludes/common_contenttype.inc.php';
		return $plugin->includeContentType($cType);
	} else {
		return false;
	}
}

//Depriated; use cms_core::includeContentType() instead
function checkContentTypeExists($cType) {
	if (includeContentType($cType)) {
		return array($cType, false);
	} else {
		return false;
	}
}


function getAllContentTypes() {
	$sql = "SELECT content_type_id
			FROM ".DB_NAME_PREFIX."contenttypes
			ORDER BY content_type_id";
	$result = my_mysql_query ($sql);

	while ($row = mysql_fetch_array ($result)) {
		$types[] = $row['content_type_id'];
	}
	return $types;
}

function getContentTypes($contentType=0) {
	$sql = "SELECT content_type_id,
				content_type_name_en,
				tag_prefix,
				default_template_id
		FROM ".DB_NAME_PREFIX."contenttypes";
	
	if ($contentType) {
		$sql .= " WHERE content_type_id = '" . mysql_real_escape_string($contentType) . "'";
	}
	
	$sql .= " ORDER BY content_type_id";
	
	$result = my_mysql_query ($sql);
	$contentTypes = array();
	while ($row = mysql_fetch_array ($result)) {
		array_push( $contentTypes, $row ) ;
	}
	return $contentTypes;
}

function getContentTypeName ($cType) {
	$sql = "SELECT content_type_name_en
			FROM " . DB_NAME_PREFIX . "contenttypes
			WHERE content_type_id = '" . mysql_real_escape_string($cType) . "'";

	if ($result = my_mysql_query($sql)) {
		$row = mysql_fetch_array($result);
		return $row[0];
	} else {
		return false;
	}
}

function checkContentTypeHasTemplate ($cType) {
	$sql = "SELECT template_id
			FROM " . DB_NAME_PREFIX . "templates
			WHERE content_type = '" . mysql_real_escape_string($cType) . "'";

	$result = my_mysql_query($sql);

	if (mysql_num_rows($result)>0) {
		return true;
	} else {
		return false;
	}
}

function getContentTypeSettings($contentTypeId, &$fields, &$dbDetails) {
	$fields = array();
	$dbDetails = array();
	
	$sql = "
		SELECT `name`, `type`, `value`
		FROM ". DB_NAME_PREFIX. "contenttype_settings
		WHERE content_type_id = '". mysql_real_escape_string($contentTypeId). "'
		ORDER BY ordinal";
	
	$result = my_mysql_query($sql);
	while ($row = mysql_fetch_assoc($result)) {
		$fields[$row['name']] = array (
				'Field' => $row['name'],
				'Type' => $row['type'],
				'Table' => 'contenttype_settings'
		);
		
		$dbDetails[$row['name']] = $row['value'];
	}
}


function loadContentTypeSettings ($cType) {
	/* Load the content type settings */
	$contentTypeSettings = array();
	$sql = "SELECT name, 
				value
			FROM ".DB_NAME_PREFIX."contenttype_settings
			WHERE content_type_id = '" . mysql_real_escape_string($cType) . "'";
	$result = my_mysql_query ($sql);
	while ($row = mysql_fetch_array ($result)) {
		$contentTypeSettings[$row['name']] = $row['value'];
	}

	// Returns an array $contentTypeSettings which contains all the content type settings
	return $contentTypeSettings;
}

function getContentImageMeta ($cID, $cType, $cVersion) {
	$sql = "SELECT id,
				content_item_id,
				content_item_version,
				content_item_type,
				created_datetime,
				filename,
				mime_type,
				width,
				height,
				size,
				checksum,
				checksum_inc_name,
				sticky_flag
			FROM " . DB_NAME_PREFIX . "images
			WHERE content_item_id = " . (int) $cID . "
				AND content_item_version = " . (int) $cVersion . "
				AND content_item_type = '" . mysql_real_escape_string($cType) . "'";
			 
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		$images = array();
		
		while ($row = mysql_fetch_array($result)) {
			$images[] = $row;
		}
		
		return $images;
	} else {
		return false;
	}
}

function getTemplateFamily ($cID="", $cType="", $cVersion=0) {
	
	$sql = "SELECT t.family_name,
				t.filename,
				t.template_id,
				t.custom_bodytag
			FROM ".DB_NAME_PREFIX."content_" . mysql_real_escape_string($cType) . " AS c
			JOIN " . DB_NAME_PREFIX . "templates AS t
				ON c.template_id = t.template_id
			WHERE c.id = " . (int) $cID . "
				AND c.version = " . (int) $cVersion;

	$result = my_mysql_query ($sql);
	if ($row = mysql_fetch_array ($result)) {
		return $row;
	} else {
		return false;
	}
}

function getTemplateFamilies () {
	$sql = "SELECT family_name
			FROM " . DB_NAME_PREFIX . "template_family";
			
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)) {
		while ($row = mysql_fetch_array($result)) {
			$templateFamilies[] = $row['family_name'];
		}
		
		return $templateFamilies;
	} else {
		return false;
	}
}

function getItemTemplate ($cID, $cType, $cVersion) {

	$sql = "SELECT t.template_id,
				t.family_name,
				t.filename,
				t.name,
				t.content_type,
				t.editor_width_bodymain,
				t.allow_rss
			FROM " . DB_NAME_PREFIX . "templates AS t
			JOIN " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . " AS c
				ON t.template_id = c.template_id
			WHERE c.id = " . (int) $cID . "
				AND c.version = " . (int) $cVersion;

	$result = my_mysql_query($sql);
	return mysql_fetch_array($result);
}


function getTemplateDetails($templateId) {
	$sql = "
		SELECT
			family_name,
			filename,
			image_filename,
			name,
			content_type,
			status,
			skin_id
		FROM ". DB_NAME_PREFIX. "templates
		WHERE template_id = ". (int) $templateId;
	$result = my_mysql_query($sql);
	return mysql_fetch_assoc($result);
}

function getTemplateId ($familyName,$filename) {
	$sql = "SELECT template_id
			FROM " . DB_NAME_PREFIX . "templates
			WHERE family_name = '" . mysql_real_escape_string($familyName) . "'
				AND filename = '" . mysql_real_escape_string($filename) . "'";
				
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
		
		return $row['template_id'];
	} else {
		return false;
	}
}

function getTemplateIdFromContentItem($cID, $cType = 'html', $cVersion = false) {
	
	if (!$cVersion) {
		$cVersion = getLatestVersion($cID, $cType);
	}
	
	$sql = "SELECT template_id
			FROM ".DB_NAME_PREFIX."content_". mysql_real_escape_string($cType). "
			WHERE id = ". (int) $cID . "
			  AND version = ". (int) $cVersion;

	$result = my_mysql_query ($sql);
	if ($row = mysql_fetch_assoc ($result)) {
		return $row['template_id'];
	} else {
		return false;
	}
}

function getTemplateFamilyTemplates ($template_family) {
	$sql = "SELECT template_id,
				family_name,
				filename,
				name,
				usage_flag,
				content_type,
				editor_width_bodymain,
				custom_bodytag,
				enable_content_head,
				enable_content_bodytop,
				enable_content_bodymain,
				enable_content_bodyfoot,
				allow_rss,
				skin_id
			FROM " . DB_NAME_PREFIX . "templates
			WHERE family_name = '" . mysql_real_escape_string($template_family) . "'
			ORDER BY family_name, filename";

	$result = my_mysql_query($sql);

	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			$templates[] = $row;
		}
		
		return $templates;
	} else {
		return false;
	}
}


/*	Content Functions	*/

//A function to save a section of a content item
function saveItemContent($cID, $cVersion, $contentPart, $cType, $content, $admin_user_id, $admin_authtype, $cLanguage = false) {
	
	$sql = "UPDATE ". DB_NAME_PREFIX. "content_". mysql_real_escape_string($cType). "
			SET ". mysql_real_escape_string($contentPart). " = '". mysql_real_escape_string($content). "',
				last_author_id = ". (int) $admin_user_id. ",
				last_author_authtype = '". mysql_real_escape_string($admin_authtype). "'
			WHERE id = ". (int) $cID."
				AND version = ". (int) $cVersion;
	$result = my_mysql_query($sql);
	
	//If this version of the CMS has workflow, we may need to check the effects of this save on a Task
	if (checkModuleReadyAndInclude('workflow.inc.php')) {
		if ($cLanguage === false) {
			$cLanguage = getContentLang($cID, $cType, $cVersion);
		}
		
		checkActionsEffectOnTasks('save', $cID, $cLanguage);
	}
}



function getContentLang($cID, $cType = false, $cVersion = false) {

	if (!$cType) {
		$cType = 'html';
	} elseif (!checkRowExists('contenttypes', array('content_type_id' => $cType))) {
		return false;
	}
	
	if (!$cVersion) {
		$cVersion = getLatestVersion($cID, $cType);
	}

	if ($cID) {
		$sql = "SELECT language_id
				FROM ".DB_NAME_PREFIX."content_". mysql_real_escape_string($cType). "
				WHERE id = ". (int) $cID. "
					AND version = ". (int) $cVersion;

		$result = my_mysql_query ($sql);
		
		if ($row = mysql_fetch_array ($result)) {
			return $row[0];
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function getContentLanguages ($cType) {
	$sql = "SELECT language_id, 
				language_local_name, 
				COUNT(sub.id) as item_count 
			FROM (SELECT language_id, 
						id 
					FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . " AS c 
					INNER JOIN (SELECT id, 
									MAX(version) AS version 
								FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . " 
								WHERE status <> 'archived' 
								GROUP BY id) AS c2 
					USING (id,version)
			) AS sub 
			INNER JOIN " . DB_NAME_PREFIX . "languages AS l 
				ON sub.language_id = l.id 
			GROUP BY language_id";

	$result = my_mysql_query($sql);

	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			$languages[] = $row;
		}

		return $languages;
	} else {
		return false;
	}
}

function getTrashedContentLanguages ($cType) {
	$languageChoices = getLanguages();

	foreach ($languageChoices as $languageChoice) {
		$sql = "SELECT l.id AS language_id,
					language_local_name,
					COUNT(c.id) as item_count
				FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . " AS c
				INNER JOIN (
					SELECT id, max(version) as version 
					FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
					WHERE language_id = '" . mysql_real_escape_string($languageChoice['id']) . "'
					GROUP BY id
					) as sub
					ON c.id = sub.id
						AND c.version = sub.version
				INNER JOIN " . DB_NAME_PREFIX . "languages AS l
					ON c.language_id = l.id
				WHERE language_id = '" . mysql_real_escape_string($languageChoice['id']) . "' 
					AND status = 'archived'
				GROUP BY language_local_name";

		$result = my_mysql_query($sql);

		if (mysql_num_rows($result)>0) {
			while ($row = mysql_fetch_array($result)) {
				$languages[] = $row;
			}
		}
	}

	if ($languages) {
		return $languages;
	} else {
		return false;
	}
}


function checkPerm($cID, $cType, $cVersion = false) {
	return getShowableVersion($cID, $cType, $cVersion, false, $_SESSION['extranetUserID'], $dontShowVersion = true);
}

function getShowableVersion ($cID=0, $cType="", $requestVersion=0, $adminUsername="", $extranetUserID="", $dontShowVersion=false,$special=false,$return_permissions_error=false) {
	return require 'cmsincludes/visitor/getShowableVersion.fun.php';
}

function getLatestContentID ($cType) {
	$sql = "SELECT MAX(id) AS newid
			FROM " . DB_NAME_PREFIX . "content_" . $cType;

	$result = my_mysql_query($sql);

	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
		return $row[0];
	} else {
		return false;
	}
}

function getIdAndVersionFromAlias($alias) {
	$sql = "
		SELECT id, version
		FROM " . DB_NAME_PREFIX . "content_html
		WHERE alias = '" . mysql_real_escape_string($alias) . "'
		ORDER BY version DESC
		LIMIT 1";
	$result = my_mysql_query($sql);
	if (mysql_num_rows($result)>0) {
		return mysql_fetch_array($result);
	} else {
		return false;
	}
}

function getPublishedVersion($cID, $cType) {

	$sql = "SELECT version
			FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
			WHERE id = " . (int) $cID . "
				AND status = 'published'";
	
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
		
		return $row[0];
	} else {
		return false;
	}

}

function getLatestVersion ($cID, $cType) {
	if ($cType) {
		if (is_numeric($cID)) {
			$sql = "SELECT max(version)
					FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
					WHERE id = " . (int) $cID;
	
			$result = my_mysql_query($sql);
	
			if ($row = mysql_fetch_array($result)) {
				return $row[0];
			} else {
				return false;
			}
		} else {
			$sql = "SELECT max(version)
					FROM " . DB_NAME_PREFIX . "content_html
					WHERE alias = '" . mysql_real_escape_string($cID) . "'";
	
			$result = my_mysql_query($sql);
	
			if ($row = mysql_fetch_array($result)) {
				return $row[0];
			} else {
				return false;
			}
		}
	} else {
		return false;
	}
}

function getLatestVersionStatus($cID, $cType, $useRevisionStatus = false) {

	$sql = "SELECT version, status
			FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
			WHERE id = '". (int) $cID. "'
			ORDER BY version DESC
			LIMIT 1";
	$result = my_mysql_query ($sql);

	if ($row = mysql_fetch_assoc($result)) {
		if ($useRevisionStatus && $row['status'] == 'private_draft' && $row['version'] > 1) {
			return 'revision';
		} else {
			return $row['status'];
		}
	} else {
		return false;
	}
}

function getContentItemCount ($cType) {
	$sql = "SELECT count(*)
			FROM (SELECT id
					FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
					GROUP BY id) AS sub";
					
	$result = my_mysql_query($sql);
	
	$row = mysql_fetch_array($result);
	
	return $row[0];
}

function checkContentExists ($cID, $cType, $cVersion) {
	$cID = (int) $cID;
	$cVersion = (int) $cVersion;

	$sql = "SELECT id
			FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
			WHERE id = " . $cID . "
				AND version = " . $cVersion;
				
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		return true;
	} else {
		return false;
	}
}

function allowEdit ($cID="", $cType="", $cVersion=0, $adminUsername="", $returnTextFor) {
	// Returns something if the administrator is allowed to edit this Item/version at this time
	// Returns false if he can't
	return checkPriv("_PRIV_PUBLISH_ITEM");
}

function allowPublish ($cID="", $cType="", $cVersion=0, $adminUsername="", $returnTextFor) {
	// Returns something if the administrator is allowed to publish this Item/version at this time
	// Returns false if he can't

	if ($cID!=""&&$cID>0) {
		if (getContentStatus($cID, $cType, $cVersion)=="private_draft") {
			return true;
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function allowDelete ($cID="", $cType="", $cVersion=0, $adminUsername="") {
	return getContentStatus($cID, $cType, $cVersion) == "private_draft" && ($cVersion > 1 || !checkIfAliasIsSystemAlias($cID, $cType));
}

function allowTrash ($cID="", $cType="", $cVersion=0, $adminUsername="") {
	$status = getContentStatus($cID, $cType, $cVersion);
	
	return checkPriv("_PRIV_PUBLISH_ITEM") && ($status == "published" || $status == "hidden") &&
		!checkIfAliasIsSystemAlias($cID, $cType) && (getLatestVersionStatus($cID,$cType)!="private_draft");
}

function allowHide ($cID="", $cType="", $cVersion=0, $adminUsername="") {
	return 
		checkPriv("_PRIV_PUBLISH_ITEM") &&
		getContentStatus($cID, $cType, $cVersion) == "published" &&
		!checkIfAliasIsSystemAlias($cID, $cType);
}

function getItemTitle($cID, $cType, $cVersion = false) {
	
	if (!$cVersion) {
		$cVersion = getLatestVersion($cID, $cType);
	}

	$sql = "SELECT title
			FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
			WHERE id = " . (int) $cID . "
				AND version = " . (int) $cVersion;

	$result = my_mysql_query($sql);

	$row = mysql_fetch_row($result);
	return $row[0];
}

function getItemDescription($cID, $cType, $cVersion) {

	$sql = "SELECT description
			FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . "
			WHERE id = " . (int) $cID . "
				AND version = " . (int) $cVersion;

	$result = my_mysql_query($sql);

	$row = mysql_fetch_row($result);
	return $row[0];
}

function getItemCreatorAuthor($cID, $cType, $cVersion, &$creatingAuthorId, &$creatingAuthorAuthType) {
	$sql = "
			SELECT creating_author_id,
				creating_author_authtype
			FROM ". DB_NAME_PREFIX. "content_". mysql_real_escape_string($cType). "
			WHERE id = ". (int) $cID. "
				AND version = ". (int) $cVersion;
				
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) { 
		$row = mysql_fetch_assoc($result);
		
		$creatingAuthorId = $row['creating_author_id'];
		$creatingAuthorAuthType = $row['creating_author_authtype'];
	} else {
		return false;
	}
}	

function getTagWithFlag($cID, $cType, $cVersion = false, $content_tag_zerofill=2, $max_title_length=20) {
	//Shortcut to getTag()
	return getTag($cID, $cType, $cVersion, $content_tag_zerofill, $max_title_length, true, true);
}

function getTag($cID, $cType, $cVersion = false, $content_tag_zerofill=2, $max_title_length=20, $escapeTitle = false, $displayFlag = false, $showAlias=true) {

	if (!$cType) {
		$cType = 'html';
	}
	
	if (!$cVersion) {
		$cVersion = getLatestVersion($cID, $cType);
	}

	$cID = (int) $cID;
	$cVersion = (int) $cVersion;
	
	if (($cID>0) && ($cType) && ($cVersion>0)) {
		if ($cType=='html') {
			$sql = "
				SELECT
					IF (LENGTH(title) <= " . (int) $max_title_length . ",
						title,
						CONCAT(LEFT(title, " . floor($max_title_length/2) . "),
								'...',
								RIGHT (title, " . floor($max_title_length/2) . ")
						)
					) AS title,
					alias,
					language_id
				 FROM ".DB_NAME_PREFIX."content_" . mysql_real_escape_string($cType) . "
				 WHERE id = " . (int) $cID . "
					AND version = " . (int) $cVersion;
		} else {
			$sql = "
				SELECT
					IF (LENGTH(title) <= " . (int) $max_title_length . ",
						title,
						CONCAT(LEFT(title, " . floor($max_title_length/2) . "),
								'...',
								RIGHT (title, " . floor($max_title_length/2) . ")
						)
					) AS title,
					language_id
				 FROM ".DB_NAME_PREFIX."content_" . mysql_real_escape_string($cType) . "
				 WHERE id = " . (int) $cID . "
					AND version = " . (int) $cVersion;
		}
		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {

			$title = $row['title'];
			$alias = $row['alias'];
			$flag = '';
			
			if ($displayFlag) {
					$flag = '
						<img src="'. SUBDIRECTORY. 'admin/images/flags/'. getLanguageFlag($row['language_id']).'.gif"
							alt="'. $row['language_id'].'" title="'. $row['language_id'].'" /> ';
			}

			$sql = "SELECT tag_prefix
					FROM " . DB_NAME_PREFIX . "contenttypes
					WHERE content_type_id = '" . mysql_real_escape_string($cType) . "'";

			$result = my_mysql_query($sql);
			
			
			

			if ($row = mysql_fetch_array($result)) {
				$tag_prefix = $row['tag_prefix'];
				$r = $flag. $tag_prefix . str_pad ($cID, $content_tag_zerofill, "0", STR_PAD_LEFT);
				if ($alias && $showAlias) {
					$r .= "/$alias";
				}
				
				if ($escapeTitle) {
					$r .= " ". htmlspecialchars($title);
				} else {
					$r .= " ".$title;
				}
				
				return $r;
			} else {
				return false;
			}
		} else return false;
	} else return false;
}


//As above, however return a text error message instead of false
//Note I am assuming getPhrase is defined.
function getTagIfExists($cID, $cType, $cVersion, $content_tag_zerofill=2, $max_title_length=20) {
	
	if ($tag = getTag($cID, $cType, $cVersion, $content_tag_zerofill, $max_title_length)) {
		return $tag;
	} else {
		return getPhrase('_CONTENT_ITEM_NOT_FOUND');
	}
	
}

//A copy of getTag, for if you already have the title
function formatTag($cID, $cType, $title, $alias = false, $content_tag_zerofill=2, $max_title_length=20, $showAlias=true) {
	
	if (strlen($title) > $max_title_length) {
		$title = mb_substr($title, 0, floor($max_title_length/2)). '...'. mb_substr($title, -floor($max_title_length/2));
	}
	
	$sql = "SELECT tag_prefix
			FROM " . DB_NAME_PREFIX . "contenttypes
			WHERE content_type_id = '" . mysql_real_escape_string($cType) . "'";

	$result = my_mysql_query($sql);

	if ($row = mysql_fetch_assoc($result)) {
		$tag_prefix = $row['tag_prefix'];
		$r = $tag_prefix . str_pad ($cID, $content_tag_zerofill, "0", STR_PAD_LEFT);
		if ($alias && $showAlias) $r .= "/$alias";
		$r .= "\n".$title;
		return $r;
	} else {
		return false;
	}
}

//Given a tag generated by one of the two above functions, strip it apart and get the
//cID and cType
function getCIdAndCTypeFromTag($tag, &$cID, &$cType) {

	//The format of the tag is PREFIXCIDBLAHBLAHBLAH...
	//The prefix is made of only letters, and the id is numeric, so we can use a 
	//regular expression to strip those out
	if (!preg_match('#([A-Z]+)([0-9]+)(.*)#', $tag, $matches)) {
		return false;
	}
	
	list($dummy, $cPrefix, $cID) = $matches;
	
	//Convert the prefix to a content id
	$sql = "SELECT content_type_id
			FROM ".DB_NAME_PREFIX."contenttypes
			WHERE tag_prefix = '". $cPrefix. "'";
	$result = my_mysql_query($sql);
	list($cType) = mysql_fetch_row($result);
	
	return true;
}

//Read a line from admin_phrase_codes/en.txt
//Either read the next full line after a given position, or if $pos is not specified, the next line from the last position
$__tribiq_getAdminPhraseCode_foundCodes__ = array();
function adminPhraseLine($lang, &$f, &$code, &$text, $pos = false) {
	$code = $text = '';
	
	if ($pos !== false) {
		fseek($f, $pos);
		fgets($f);
	}
	
	if ($line = fgets($f)) {
		$line = explode('|', $line, 2);
		$code = $line[0];
		$text = trim($line[1], "\t\n\r\0\x0B");
		
		//Cache every code we find in memory
		global $__tribiq_getAdminPhraseCode_foundCodes__;
		$__tribiq_getAdminPhraseCode_foundCodes__[$lang][$code] = $text;
		
		return true;
	} else {
		return false;
	}
}

//Find a specific phrase code from admin_phrase_codes/en.txt.
//This uses binoimal search, so relies 
function getAdminPhraseCode($target, $lang) {
	
	if (!is_array($__tribiq_getAdminPhraseCode_foundCodes__[$lang])) {
		$__tribiq_getAdminPhraseCode_foundCodes__[$lang] = array();
	}
	if (isset($__tribiq_getAdminPhraseCode_foundCodes__[$lang][$target])) {
		return $__tribiq_getAdminPhraseCode_foundCodes__[$lang][$target];
	}
	
	$size = filesize('cmsincludes/admin_phrase_codes/'. $lang. '.txt');
	$f = fopen('cmsincludes/admin_phrase_codes/en.txt', 'r');
	$step = $pos = (int) ($size / 2);
	$text = $code = $lastCode = '';
	
	//Use binomial search until we get to under half a KB
	while ($step > 512) {
		$lastCode = $code;
		if (!adminPhraseLine($lang, $f, $code, $text, $pos)) {
			return 'error';
		} elseif (($strcmp = strcasecmp($code, $target)) == 0) {
			return $text;
		} elseif ($step <= 512) {
			
		} elseif ($strcmp < 0) {
			$pos += ($step = (int) ($step / 2));
		} else {
			$pos -= ($step = (int) ($step / 2));
		}
	}
	
	//Move the cursor back a bit.
	$pos -= $step*2;
	$pos = max($pos, 0);
	adminPhraseLine($lang, $f, $code, $text, $pos);
	
	//Sweep linearly through the code, looking for the phrase
	while((strcasecmp($code, $target) < 0) && adminPhraseLine($lang, $f, $code, $text));
	
	if ($code == $target) {
		$textOut = $text;
		
		//If we find our target, sweep past it for a while just in case there are releated codes just after it
		//that we might be called to find later
		for ($i = 0; $i < 10; ++$i) {
			adminPhraseLine($lang, $f, $code, $text);
		}
		
		return $textOut;
	} else {
		return 'Missing '. $lang. ' Admin Phrase: '. $target;
	}
}


function getPhrase($code, $replace = false) {
	
	//No support for a multilingual admin interface yet.
	$lang = 'en';
	
	if ($replace === false) {
		$replace = array();
	}
	
	//Some phrases are now being simply hard-coded. However we still wish to call getPhrase on these
	//hardcoded phrases for tracking purposes
	//Ignore anything that does not start with an underscore
	if ($lang == 'en' && substr($code, 0, 1) != '_') {
		$phrase = $code;
	} else {
		$phrase = getAdminPhraseCode($code, $lang);
	}
	
	if (empty($replace)) {
		return $phrase;
	} else {
		foreach ($replace as $key => $replacement) {
			$phrase = str_replace('[['. $key. ']]', $replacement, $phrase);
		}
		return $phrase;
	}
}

function phrase($code, $replace = array(), $pluginClass = '') {
	 return getVLPPhrase($code, $replace, $langOverride = false, $returnFalseOnFailure = false, $pluginClass);
}

function getVLPPhrase($code, $replace = array(), $langOverride = false, $returnFalseOnFailure = false, $pluginClass = '') {
	
	//Ignore anything that does not start with an underscore
	if (substr($code, 0, 1) != '_') {
		return $code;
	}
	
	
	/* Try to get language from session */
	if ($langOverride !== false) {
		$languageId = $langOverride;
	} elseif ( isset( $_SESSION['user_lang'] ) && ! empty( $_SESSION['user_lang'] ) ) {
		$languageId = $_SESSION['user_lang'];
	} else {		
		$languageId = DEFAULT_LANGUAGE;
	}
	
	/* $languageId and phrase code cannot contain spaces, so let's just make sure it's tidy */
	$languageId = str_replace(" ", "", $languageId);

	$sql = "
		SELECT local_text
		FROM ". DB_NAME_PREFIX. "visitor_phrases
		WHERE code = '". mysql_real_escape_string(str_replace(" ", "", $code)). "' 
		  AND language_id = '". mysql_real_escape_string($languageId). "'
		  AND plugin_class = '". mysql_real_escape_string($pluginClass). "'
		LIMIT 1";
			
	$result = my_mysql_query($sql);
	
	
	if ($row = mysql_fetch_array ($result)) {
		if (!is_array($replace) || !count($replace)) {
			return $row['local_text'];
		} else {
			$merged = str_replace ( '"', '', $row['local_text']);
			$placeHolders = array_keys ($replace);
			foreach ( $placeHolders as $key ) {
				$merged = str_replace ( '[[' . $key . ']]', $replace[$key], $merged );
			}
			return $merged;
		}
	}
	
	//Handle the cases where we couldn't find a translation in the database
	if ($returnFalseOnFailure) {
		//Return false straight away if we were asked to report a failure
		return false;
	}
	
	//Otherwise, try to come up with as best we can with just the code and the replace string
	$replaceIsSet = is_array($replace) && count($replace);
	
	//Does our input look like a phrase code, or a normal piece of language?
	if (!$replaceIsSet && substr($code, 0, 1) != '_') {
		//This may not be a code - output it normally. (But complain in admin mode!)
		if (checkPriv()) {
			return getPhrase('Untranslated [[lang]] visitor phrase: [[phrase]]', array('lang' => DEFAULT_LANGUAGE, 'phrase' => $code));
		} else {
			return $code;
		}
	}
	
	//If it looks like a code, it's probably a Missing VLP in the languages
	if (!checkPriv())  {
		$code = getPhraseAsBestWeCanWithoutDatabaseAccess($code);
	}
	
	if ($replaceIsSet) {
		$code .= ': ' . implode( ', ', $replace );
	}
	
	if (checkPriv())  {
		return getPhrase('Missing [[lang]] visitor phrase: [[phrase]]', array('lang' => $languageId, 'phrase' => $code));
	} else {
		return $code;
	}
	
}

//In the installer, we won't be able to use getPhrase or getVLPPhrase, as the database won't
//be connected or populated yet!
//But we may wish to have code shared between the installer and the rest of the CMS, and some of
//that code might work of phrases and codes.
//This function takes a code, replaces the _s with spaces and initcaps it, so the codes look legiable.
function getPhraseAsBestWeCanWithoutDatabaseAccess($code) {
	return ucwords(strtolower(trim(str_replace('_', ' ', $code))));
}

function recurse ($id=0, $table="", $separatorText="", $pathSoFar="") {
	// Generic recursive routine to get the paths of things
	// e.g. Top Folder -> Middle Folder -> This Folder
	$id = (int) $id;
	$table = str_replace(" ", "", $table);

	if ($table) {
		$sql = "SELECT parent_id,
					name
				FROM ".DB_NAME_PREFIX."$table
				WHERE id = $id";

		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {
			if ($row['parent_id'] == 0) {
				// Stop recursing
				$pathSoFar = $row['name'].$separatorText.$pathSoFar;
				return array (false, $pathSoFar);
			} else {
				$pathSoFar = $row['name'].$separatorText.$pathSoFar;
				$r = recurse ($row['parent_id'], $table, $separatorText, $pathSoFar);
				return array ($row['parent_id'], $r[1]);
			}
		}
	} else {
		return false;
	}
}

function recurse_breadcrumb ($id=0, $table="", $separatorText="", $pathSoFar="") {
	// Get paths for Breadcrumb with links
	$id = (int) $id;
	$table = str_replace(" ", "", $table);

	if ($table) {
		$sql = "SELECT parent_id,
					name, content_id
				FROM ".DB_NAME_PREFIX."$table
				WHERE id = $id";

		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {
			$alias = getHTMLContentAlias($row["content_id"]);
			if($alias) { $cID = $alias; } else { $cID = $row["content_id"]; }
			if ($row['parent_id'] == 0) {
				// Stop recursing
				$pathSoFar = "<a href=\"". indexDotPHP(true). "?cID=".$cID."\">".$row['name']."</a>".$separatorText.$pathSoFar;
				return array (false, $pathSoFar);
			} else {

				$pathSoFar = "<a href=\"". indexDotPHP(true). "?cID=".$cID."\">".$row['name']."</a>".$separatorText.$pathSoFar;
				$r = recurse_breadcrumb ($row['parent_id'], $table, $separatorText, $pathSoFar);
				return array ($row['parent_id'], $r[1]);
			}
		}
	} else {
		return false;
	}
}

function getHTMLContentAlias($mID) {
	$mID = (int) $mID;
	if ($mID > 0) {
		$sql = "SELECT html_content_alias
				FROM " .DB_NAME_PREFIX . "menuitems
				WHERE content_id = " . $mID . "";
		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {
			return $row['html_content_alias'];
		} else {
			return false;
		}
	}
	else {
	}
}

function getContentAlias($cID, $cVersion = false, $cType = 'html') {
	
	if ($cType != 'html') {
		return false;
	}
	
	$sql = "
		SELECT alias
		FROM " . DB_NAME_PREFIX . "content_html
		WHERE id = " . (int) $cID;
	
	if ($cVersion) {
		$sql .= "
		  AND version = " . (int) $cVersion;
	} else {
		
		if (!checkPriv()) {
			$sql .= "
		  AND status = 'published'";
		}
		
		$sql .= "
		ORDER BY version DESC
		LIMIT 1";
	}
	
		

	$result = my_mysql_query($sql);

	If ($row = mysql_fetch_array($result)) {
		return $row[0];
	} else {
		return false;
	}
}

function getIDfromAlias($alias) {
	$sql = "SELECT id
			FROM " . DB_NAME_PREFIX . "content_html
			WHERE alias = '" . mysql_real_escape_string($alias) . "'";
	
	if (!checkPriv()) {
		$sql .= "
			  AND status = 'published'";
	}
	
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
		return $row[0];
	} else {
		return false;
	}
}

function getContentFromMenu ($mID) {
	// Returns list ($cID, $cType) on the basis of the menu item ID
	$mID = (int) $mID;

	if ($mID > 0) {
		$sql = "SELECT content_id, content_type
				FROM " . DB_NAME_PREFIX . "menuitems
				WHERE id = " . $mID . "
					AND content_id<>0";

		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {
			return $row;
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function getMenuItemFromContent($cID=0, $cType="", $fetchSecondaries = false) {
	
	$cType = str_replace(" ", "", $cType);

	if ($cID && $cType) {
		$sql = "SELECT id,
					section_id,
					language_id,
					name,
					redundancy, 
					mode, 
					parent_id,
					ext_url, 
					ordinal,
					hide_private_item,
					invisible,
					rel_tag
				FROM ".DB_NAME_PREFIX."menuitems
				WHERE content_id = ". (int) $cID. "
				  AND content_type = '" . mysql_real_escape_string($cType) . "'
				  AND target_loc = 'int'
				ORDER BY redundancy = 'primary' DESC";
		
		if (!$fetchSecondaries) {
			$sql .= "
				LIMIT 1";
		}
		
		$result = my_mysql_query ($sql);
		
		if ($fetchSecondaries) {
			$rows = array();
			while($row = mysql_fetch_array($result)) {
				$rows[] = $row;
			}
			return $rows;
		
		} elseif ($row = mysql_fetch_array ($result)) {
			return $row;
			
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function getSectionMenuItemFromContent($cID, $cType, $section) {

	$sql = "
		SELECT id
		FROM ".DB_NAME_PREFIX."menuitems
		WHERE content_id = ". (int) $cID. "
		  AND content_type = '" . mysql_real_escape_string($cType) . "'
		  AND section_id = '" . mysql_real_escape_string($section) . "'
		  AND target_loc = 'int'
		ORDER BY redundancy = 'primary' DESC
		LIMIT 1";
	
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_assoc($result)) {
		return $row['id'];
	} else {
		return false;
	}
}

function getMenuItemFromMenuID($mID=0) {

	if ($mID) {
		$sql = "SELECT
					name,
					section_id,
					language_id,
					redundancy,
					accesskey,
					rel_tag,
					target_loc,
					open_in_new_window,
					content_id,
					html_content_alias,
					content_type,
					ext_url,
					active,
					invisible,
					hide_private_item,
					parent_id,
					ordinal
				FROM ".DB_NAME_PREFIX."menuitems
				WHERE id = ". (int) $mID;
		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {
			return $row;
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function getMenuName($mID) {
	if ($details = getMenuItemFromMenuID($mID)) {
		return $details['name'];
	} else {
		return false;
	}
}

function getNumMenuChildren($mID=0) {
	$sql = "SELECT COUNT(*)
			FROM ".DB_NAME_PREFIX."menuitems
			WHERE parent_id = ". (int) $mID;
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_row($result)) {
		return $row[0];
	}
	return 0;
}

function getNumMenuChildren_recursive ($parent_id=0) {

	$sql = "SELECT id FROM " . DB_NAME_PREFIX . "menuitems WHERE parent_id = " . (int) $parent_id;
	$result = my_mysql_query($sql);

	$newmenuitems_count = mysql_num_rows($result);

	while ($row = mysql_fetch_array($result)) {
		$newmenuitems_count += getNumMenuChildren_recursive ($row['id']);
	}

	return $newmenuitems_count;
}

function getMenuParent ($mID) {
	
	if (!$mID = (int) $mID) {
		return false;
	}

	$sql = "SELECT parent_id FROM " . DB_NAME_PREFIX . "menuitems WHERE id = " . $mID;

	$result = my_mysql_query($sql);

	if ($row = mysql_fetch_array($result)) {
		return $row[0];
	} else {
		return false;
	}
}

function getMenuParentName ($mID) {
	$mID = (int) $mID;

	$sql = "SELECT name FROM " . DB_NAME_PREFIX . "menuitems WHERE id = " . $mID;

	$result = my_mysql_query($sql);

	if ($row = mysql_fetch_array($result)) {
		return $row[0];
	} else {
		return false;
	}
}

function getMenuItemLevel($mID, &$ancestor, &$ancestorButOne) {
	$levelCount = 1;

	$ancestor = $ancestorButOne = $mID;
	$recurseLimit = 100;
	while ((--$recurseLimit) && ($newMID = getMenuParent($ancestor))) 	{
		$ancestorButOne = $ancestor;
		$ancestor = $newMID;
		$levelCount++;
	}
	
	return $levelCount;
}
//Given a menu item, check whether it has any children
function hasChild($mID, $sectionId, $adminMode) {
	
	//Run a SQL statement to check for children
	$sql = "
		SELECT content_id, content_type, hide_private_item
		FROM ". DB_NAME_PREFIX. "menuitems AS mi 
		WHERE section_id='". mysql_real_escape_string($sectionId). "'
		  AND parent_id = " . (int) $mID;
	
	//Count inactive menu items as well in admin mode
	if (!$adminMode) {
		$sql .= "
		  AND active = 1";
	}

	$result = my_mysql_query($sql);
	
	//Loop through the results, looking for valid children
	while ($row = mysql_fetch_assoc($result)) {
		//Return true as soon as we find a visible child
		if ($adminMode || $row['hide_private_item'] == 0) {
			return true;
		} elseif ($row['hide_private_item'] == 3) {
			return issetArrayKey($_SESSION, 'extranetUserID');
		} elseif ($row['hide_private_item'] == 2) {
			return !issetArrayKey($_SESSION, 'extranetUserID');
		} elseif (getShowableVersion($row['content_id'], $row['content_type'], false, false, arrayKey($_SESSION, 'extranetUserID'))) {
			return true;
		}
	}
	
	return false;
}

function isMenuItemAncestor ($currentMenuItem, $menuItemToCheck, $recurse_count=0) {
	
	if (++$recurse_count > 16) {
		echo "Function isMenuItemAncestor has recursed too far and must exit.";
		exit;
	}

	$sql = "SELECT parent_id
			FROM " . DB_NAME_PREFIX . "menuitems
			WHERE id = " . (int) $currentMenuItem . "
			ORDER BY ordinal";

	$result = my_mysql_query($sql);

	if ($row = mysql_fetch_assoc($result)) {
		if ($row['parent_id'] == (int) $menuItemToCheck) {
			return true;
		} else {
			return isMenuItemAncestor($row['parent_id'], $menuItemToCheck, $recurse_count);
		}
	} else {
		return false;
	}

}

//Build a link to a content item
function linkToItem(
			$cIDIn, $cType = 'html', $siteConfig = false, $fullPath = '', $request = '',
			$autoLookupAlias = true, $autoAddImportantRequests = true, $forceLookupAliasInAdminMode = false,
			$httpHost = false) {
	
	if (!$cType) {
		$cType = 'html';
	}
	
	if (!is_array($siteConfig)) {
		$siteConfig = false;
	}
	
	if ($fullPath) {
		$fullPath = httpOrHttps(). ifNull($httpHost, httpHost()). SUBDIRECTORY;
	} else {
		$fullPath = '';
	}
	
	//Attempt to look up the alias of a html content item
	if ($autoLookupAlias && $cType == 'html' && ($forceLookupAliasInAdminMode || !checkPriv()) && is_numeric($cIDIn)) {
		$cID = ifNull(getContentAlias($cIDIn), $cIDIn);
	} elseif (checkPriv() && !is_numeric($cIDIn)) {
		$cID = ifNull(getIDfromAlias($cIDIn), $cIDIn);
	} else {
		$cID = $cIDIn;
	}
	
	//Add important requests to the URL, if the content item being linked to is the current content item
	if ($autoAddImportantRequests
	 && ($cID == arrayKey($_REQUEST, 'cID') || $cIDIn == arrayKey($_REQUEST, 'cID'))
	 && ($cType == arrayKey($_REQUEST, 'cType') || (!arrayKey($_REQUEST, 'cType') && $cType == 'html'))
	 && is_array(cms_core::$importantGetRequests)) {
		foreach(cms_core::$importantGetRequests as $getRequest => $defaultValue) {
			if (isset($_GET[$getRequest]) && $_GET[$getRequest] != $defaultValue) {
				$request .= '&'. urlencode($getRequest). '='. urlencode($_GET[$getRequest]);
			}
		}
	}
	
	
	if ($cType != 'html') {
		return $fullPath. indexDotPHP(!$fullPath). "?cID=". $cID. '&cType='. $cType. addAmp($request);
	
	} else {
		$config = ifNull($siteConfig, array());
		
		//Check whether we should use a slash on links to the homepage
		if (!defined('USE_SLASH_ON_LINK_TO_HOMEPAGE')) {
			if (!issetArrayKey($config, 'use_slash_on_link_to_homepage')) {
				$config['use_slash_on_link_to_homepage'] = getSiteConfig('use_slash_on_link_to_homepage');
			}
			
			define('USE_SLASH_ON_LINK_TO_HOMEPAGE', $config['use_slash_on_link_to_homepage']);
		}
		
		$onHomepage = false;
		if (USE_SLASH_ON_LINK_TO_HOMEPAGE) {
			//Look up the default homepage id/alias, and cache them using constants
			if (!defined('DEFAULT_HOMEPAGE_ID')) {
				if (!issetArrayKey($config, 'default_homepage_id')) {
					$config['default_homepage_id'] = getSiteConfig('default_homepage_id');
				}
				
				if (class_exists('cms_core')) {
					define('DEFAULT_HOMEPAGE_ID', cms_core::directUserToHomepage($config));
				} else {
					define('DEFAULT_HOMEPAGE_ID', $config['default_homepage_id']);
				}
				
				define('DEFAULT_HOMEPAGE_ALIAS', getContentAlias(DEFAULT_HOMEPAGE_ID));
			}
			
			$onHomepage = $cID == DEFAULT_HOMEPAGE_ID || $cID == DEFAULT_HOMEPAGE_ALIAS;
		}
		
		
		if ($request) {
			return $fullPath. indexDotPHP(!$fullPath). "?cID=". $cID. addAmp($request);
		
		} elseif ($onHomepage) {
			return ifNull($fullPath, SUBDIRECTORY, '/');
		
		} elseif (is_numeric($cID)) {
			return $fullPath. indexDotPHP(!$fullPath). "?cID=". $cID. addAmp($request);
		
		} else {
			return $fullPath. applyModRewriteToAlias($cID, $siteConfig, $fullPath);
		}
	}
}

function applyModRewriteToAlias($alias, $siteConfig = false, $fullPath = false) {
	if (!$siteConfig) {
		$siteConfig = array();
		$siteConfig['mod_rewrite_enabled'] = getSiteConfig('mod_rewrite_enabled');
		$siteConfig['mod_rewrite_suffix'] = getSiteConfig('mod_rewrite_suffix');
	}
	
	if ($siteConfig['mod_rewrite_enabled']) {
		return $alias. $siteConfig['mod_rewrite_suffix'];
	} else {
		return indexDotPHP(!$fullPath). "?cID=". $alias;
	}
}



function getAccessibilityList ($cLanguage_id=DEFAULT_LANGUAGE) {
	$sql = "SELECT name,
				accesskey
			FROM " . DB_NAME_PREFIX . "menuitems
			WHERE accesskey <> ''";

	if (!checkPriv()) {
		$sql .= " AND active>0";
	}

	$sql .= " ORDER BY accesskey";

	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)) {
		$textout .= "<ul>\n";
	
		while ($row = mysql_fetch_array($result)) {
			$textout .= "\t<li>" . $row['name'] . " ( " . $row['accesskey'] . " )</li>\n";
		}
	
		$textout .= "</ul>\n";
		
		return $textout;
	} else {
		return false;
	}
}


function showAdminTitle($titleAdmin, $info="") {
	$text = "<tr style=\"background-color:#CCCCCC;\"><td class=\"panel_title\" style=\"text-align: left;\"><h5 style=\"text-align:left;\">";
	$text .= $titleAdmin;
	$text .="</h5>" . $info . "</td></tr>";
	return $text;
}

function getLanguages( $offset = -1, $quantity = 0, $langId = false ) {
	$sql = "
		SELECT id,
			homepage_id,
			search_page_id,
			extranet_login_page_id,
			extranet_registration_page_id,
			extranet_password_reminder_page_id,
			extranet_change_password_page_id,
			extranet_logout_page_id,
			language_local_name, 
			IF( strcmp( flag_file_name,'' ), flag_file_name, 'white' ) as flag 
		FROM " . DB_NAME_PREFIX . "languages";
	
	if ($langId !== false) {
		$sql .= "
		WHERE id = '". mysql_real_escape_string($langId). "'";
	}
	
	$sql .= "
		ORDER BY id";
	
	if ( $offset >= 0 ) {
		$sql .= "
			LIMIT " . $offset;
		if ( $quantity ) {
			$sql .= ', ' . $quantity;
		}
	}
	
	$result = my_mysql_query ($sql);
	$langs = array();
	while ($row = mysql_fetch_array ($result)) {
		$langs[] = $row;
	}
	return $langs;
}

function getLanguage($langId = false) {
	
	if ($langId === false) {
		$langId = $_SESSION["user_lang"];
	}
	
	$langs = getLanguages($offset = -1, $quantity = 0, $langId);
	return $langs[0];
}

function getLanguageDefaultHomepage($langId = false) {
	$lang = getLanguage($langId);
	return arrayKey($lang, 'homepage_id');
}

function getLanguageDefaultSearchPage($langId = false) {
	$lang = getLanguage($langId);
	return arrayKey($lang, 'search_page_id');
}

function getLanguageDefaultExtranetPage($langId = false) {
	$lang = getLanguage($langId);
	return arrayKey($lang, 'extranet_login_page_id');
}

function getCurrentAlias() {
	global $cID, $cType, $cVersion;
	
	if (!$cID || $cType != 'html' || !$cVersion) {
		return false;
	}
	
	$sql = "
		SELECT alias
		FROM ". DB_NAME_PREFIX. "content_html
		WHERE id = ". (int) $cID;
	
	if (checkPriv()) {
		$sql .= "
		  AND status IN ('private_draft', 'published')
		ORDER BY version DESC
		LIMIT 1";
	} else {
		$sql .= "
		  AND status = 'published'";
	}
	
	$result = my_mysql_query($sql);
	
	list($alias) = mysql_fetch_row($result);
	
	return $alias;
}

function getLanguageName($languageId = false, $addIdInBracketsToEnd = true, $returnIdOnFailure = true) {

	$name = getVLPPhrase('__LANGUAGE_LOCAL_NAME__', $replace = array(), $langOverride = $languageId, $returnFalseOnFailure = true);
	
	if ($name !== false) {
		if ($addIdInBracketsToEnd) {
			return $name. ' ('. $languageId. ')';
		} else {
			return $name;
		}
	} elseif ($returnIdOnFailure) {
		return $languageId;
	} else {
		return false;
	}

}

function getLanguageFlag($langId = false) {
	if ($lang = getLanguage($langId)) {
		return $lang['flag'];
	} else {
		return 'white';
	}
}

function getAdminLanguages () {
	connectGlobalDB();

	$sql = "SELECT language_id,
				local_name
			FROM ".DB_NAME_PREFIX_GLOBAL."admin_languages
			ORDER BY local_name ASC";

	$result = my_mysql_query ($sql);

	connectLocalDB();

	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			$languages[] = $row;
		}

		return $languages;
	} else {
		return false;
	}
}

function getAdminLanguageName($langId) {
	$name = "";
	connectGlobalDB();

	$sql = "
		SELECT local_name
		FROM ".DB_NAME_PREFIX_GLOBAL."admin_languages
		WHERE language_id = '" . mysql_real_escape_string($langId) . "'
		ORDER BY local_name ASC";
	$result = my_mysql_query ($sql);
	connectLocalDB();
	if ($row = mysql_fetch_array ($result)) {
		$name = $row["local_name"];
	}
	return $name;
}


function isAdminLangSupported ($langId="") {
	connectGlobalDB();

	$sql = "SELECT language_id
			FROM ".DB_NAME_PREFIX_GLOBAL."admin_languages
			WHERE language_id = '" . mysql_real_escape_string($langId) . "'";
	$result = my_mysql_query ($sql);

	connectLocalDB();

	if ($row = mysql_fetch_array ($result)) {
		return $row["language_id"];
	} else {
		return false;
	}
}
function isEmtpyVisitorLanguagePack( $languageId ) {
	$sql = "
		SELECT COUNT( code )
		FROM " . DB_NAME_PREFIX . "visitor_phrases
		WHERE language_id = '" . mysql_real_escape_string($languageId) . "'";
	$result = my_mysql_query ($sql);
	list( $quantity ) = mysql_fetch_row($result);
	if ( $quantity > 0 ) { 
		return false;
	} else {
		return true;
	}
}
function showMessage($type, $msg_title, $msg_text) {
	$text = "<div class=\"message_".$type."_admin\">
		<table style=\"width:100%\"  border=\"0\" cellspacing=\"0\" cellpadding=\"0\">
           <tr>
              <td style=\"width:7%\"><img src=\"images/img_".$type."_small.gif\"> <img src=\"images/divider.gif\" width=\"2\" height=\"20\"></td>
              <td style=\"width:93%; text-align:left;\" class=\"msg_color\"><strong>".$msg_title.":</strong> ".$msg_text."</td>
           </tr>
        </table>
		</div>";
	return $text;
}

//USER
// get Select box from Enum field
function getSelectFromEnum($table, $field, $selected) {
	$sql = "SELECT ". $field ."
			FROM ". DB_NAME_PREFIX . $table ."";
	$result = my_mysql_query($sql);
	$selectBox = "<select name=\"status\">";
	$selectBox .= "<option>-- ". getPhrase("_PLEASE_SELECT") ." --</option>";
	while ($row = mysql_fetch_array ($result)) {
		if($row[$field] == $selected) { $selectBox .= "<option value=\"". $row[$field] ."\" selected>". $row[$field] ."</option>"; }
		else { $selectBox .= "<option value=\"". $row[$field] ."\">". $row[$field] ."</option>"; }
	}
	$selectBox .= "</select>";
	return $selectBox;
}

function getSelectCountries($id, $name, $selected="") {
	$sql = "SELECT iso2,
				iso3,
				name_en,
				name_fr,
				name_de,
				name_pt,
				name_es,
				default_language,
				worldzone_en,
				active
			FROM ". DB_NAME_PREFIX . "countries
			ORDER BY name_en";
	$result = my_mysql_query($sql);
	$selectBox = "<select name=\"".$name."\">";
	$selectBox .= "<option value=\"\">-- ". getPhrase("_PLEASE_SELECT") ." --</option>";
	while ($row = mysql_fetch_array ($result)) {
		if($row['iso2'] == $id) { $selectBox .= "<option value=\"". $row['iso2'] ."\" selected>". $row['name_'.DEFAULT_LANGUAGE] ."</option>"; }
		else { $selectBox .= "<option value=\"". $row['iso2'] ."\">". $row['name_'.DEFAULT_LANGUAGE] ."</option>"; }
	}
	$selectBox .= "</select>";
	return $selectBox;
}

// Grab countryDetails based on iso length
// iso		  : Search String
// iso_length : 2 or 3
function getCountryDetails($iso,$iso_length) {	
	$sql = "SELECT iso2,
				iso3,
				name_en,
				name_fr,
				name_de,
				name_pt,
				name_es,
				default_language,
				worldzone_en,
				active
			FROM ". DB_NAME_PREFIX . "countries WHERE 1";
	
	if ($iso_length == "3") {
		$sql .= " AND iso3='" . mysql_real_escape_string($iso) . "'";
	} elseif ($iso_length == "2") {
		$sql .= " AND iso2='". mysql_real_escape_string($iso) . "'";
	}
	$sql .= " LIMIT 1";
	$result = my_mysql_query($sql);
	$row = mysql_fetch_array ($result);
	return $row;
}

function getCountryName($id) {
	$sql = "SELECT name_".DEFAULT_LANGUAGE."
			FROM ". DB_NAME_PREFIX . "countries
			WHERE iso2='" . mysql_real_escape_string($id) . "'";
	$result = my_mysql_query($sql);
	if($row = mysql_fetch_array ($result)) {
		$countryName = $row['name_'.DEFAULT_LANGUAGE];
	}
	return $countryName;
}

function getCountries() {
	$sql = "SELECT iso2,
				iso3,
				name_en,
				name_fr,
				name_de,
				name_pt,
				name_es,
				default_language,
				worldzone_en,
				active
			FROM ". DB_NAME_PREFIX . "countries";
	
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			$countries[] = $row;
		}
		
		return $countries;
	} else {
		return false;
	}
}




function getAdminFullname ($admin_id,$authtype,$separator = ' ',$reverse=false) {
	$admin_id = (int) $admin_id;

	if ($authtype=="local") {
		$sql = "SELECT first_name,
					last_name
				FROM ".DB_NAME_PREFIX."admins
				WHERE id='".mysql_real_escape_string($admin_id)."'";
		$result = my_mysql_query ($sql);
	} else {
		connectGlobalDB();

		$sql = "SELECT first_name,
					last_name
				FROM ".DB_NAME_PREFIX_GLOBAL."superadmins
				WHERE id='".mysql_real_escape_string($admin_id)."'";
		$result = my_mysql_query ($sql);

		connectLocalDB();
	}
	
	if (mysql_num_rows($result)) {
		$row = mysql_fetch_array($result);
		
		if ($reverse) {
			$output = $row['last_name'] . $separator . $row['first_name'];
		} else {
			$output = $row['first_name'] . $separator . $row['last_name'];
		}
		
		return $output;
	} else {
		return false;
	}
}




function getUserGroups($user_id) {
	$user_id = (int) $user_id;
	$sql = "SELECT *
			FROM ". DB_NAME_PREFIX. "group_user_link
			WHERE user_id= " . $user_id;
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		while($row = mysql_fetch_array ($result)) {
			$grps[] = $row['group_id'];
		}
		
		return $grps;
	} else {
		return false;
	}
}

function getAllGroupsAndCheckWhichAUserIsIn($userId) {
	$sql = "
		SELECT
			g.id, g.name,
			gul.group_id IS NOT NULL AS in_group
		FROM ". DB_NAME_PREFIX. "groups AS g
		LEFT JOIN ". DB_NAME_PREFIX. "group_user_link AS gul
		   ON gul.group_id = g.id
		  AND gul.user_id = ". (int) $userId. "
		ORDER BY g.name";
	
	$result = my_mysql_query($sql);
	
	$groups = array();
	while($row = mysql_fetch_assoc($result)) {
		$groups[] = $row;
	}
	return $groups;
}

function checkUserInGroup($groupId, $userId = 'session') {
	
	if ($userId === 'session') {
		$userId = $_SESSION['extranetUserID'];
	}
	
	if (!$userId || !((int) $groupId)) {
		return false;
	}
	
	$sql = "
		SELECT 1
		FROM ". DB_NAME_PREFIX. "group_user_link
		WHERE group_id = ". (int) $groupId. "
		  AND user_id = ". (int) $userId;
	
	$result = my_mysql_query($sql);
	
	return (bool) mysql_fetch_row($result);
}

function getUserGroupsNames( $userId ) {
	$thisUserGroups = array();
	$groupList = '';
	$userGroupsList = getUserGroups( $userId  );
	if ( is_array( $userGroupsList ) ) {
		foreach( $userGroupsList as $userGroup ) {
			$group = getGroupDetails( $userGroup );
			array_push( $thisUserGroups, array( 'name' => $group['name'] ) );
		}
		asort( $thisUserGroups );
	}
	if ( ! sizeof( $thisUserGroups ) ) {
		$groupList = getPhrase('_NO_GROUP_MEMBERSHIPS');
	} else {
		foreach ( $thisUserGroups as $group ){
			$groupList .=  $group['name'] . ', ';
		}
		$groupList = substr( $groupList, 0, -2 );
	}
	return $groupList;
}

function getGroupName($group_id) {
	if ($group_id) {
		$sql = "SELECT name
			FROM ".DB_NAME_PREFIX."groups
			WHERE id = ". (int) $group_id;
		$result = my_mysql_query($sql);
		$row = mysql_fetch_array ($result);		
		return $row['name'];
	} else {
		return getPhrase("_ALL_EXTRANET_USERS");
	}

}

function getGroupPrivateName($group_id) {
	$sql = "
		SELECT name
		FROM ". DB_NAME_PREFIX."groups
		WHERE id = ". (int) $group_id;
	
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_array($result)) {
		return $row['name'];
	}
	return false;
}

//GROUPS


function getContentPermissions ($cID, $cType, $cVersion) {
	$cID = (int) $cID;
	$cVersion = (int) $cVersion;

	if ($cID) {
		$sql = "SELECT ucl.user_id,
					u.first_name,
					u.last_name
				FROM " . DB_NAME_PREFIX . "user_content_link AS ucl
				LEFT JOIN " . DB_NAME_PREFIX . "users AS u
					ON ucl.user_id = u.id
				WHERE ucl.content_id = " . $cID . "
					AND ucl.content_type = '" . mysql_real_escape_string($cType) . "'
					AND ucl.content_version = " . $cVersion;

		$result = my_mysql_query($sql);

		if (mysql_num_rows($result)>0) {
			while ($row = mysql_fetch_array($result)) {
				$users[] = $row;
			}
		} else {
			$users = false;
		}

		$sql = "SELECT gcl.group_id,
					g.name
				FROM " . DB_NAME_PREFIX . "group_content_link AS gcl
				LEFT JOIN " . DB_NAME_PREFIX . "groups AS g
					ON gcl.group_id = g.id
				WHERE gcl.content_id = " . $cID . "
					AND gcl.content_type = '" . mysql_real_escape_string($cType) . "'
					AND gcl.content_version = " . $cVersion;

		$result = my_mysql_query($sql);

		if (mysql_num_rows($result)>0) {
			while ($row = mysql_fetch_array($result)) {
				$groups[] = $row;
			}
		} else {
			$groups = false;
		}

		$sql = "SELECT private
				FROM " . DB_NAME_PREFIX . "content_" . $cType . "
				WHERE id = " . $cID . "
					AND version = " . $cVersion;

		$result = my_mysql_query($sql);

		if ($row = mysql_fetch_array($result)) {
			if ($row[0]==1) {
				$private = true;
			} else {
				$private = false;
			}
		}

		return array("private" => $private,"groups" => $groups, "users" => $users);
	} else {
		return false;
	}
}

function getContentPermissionsString ($private, $groups, $users) {
	if (!$private) {
		return getPhrase("_PUBLICLY_VISIBLE");
	} else {
		if ($groups) {
			$output .= getPhrase("_GROUPS") . ": ";

			foreach ($groups AS $group) {
				$output .= $group['name'] . ", ";
			}

			if (substr($output,strlen($output)-2)==", ") {
				$output = substr($output,0,strlen($output)-2);
			}
		}

		if ($users) {
			if ($output) {
				$output .= "<br />";
			}

			$output .= getPhrase("_USERS:");

			foreach ($users AS $user) {
				$output .= $user['first_name'] . " " . $user['last_name'] . ", ";
			}

			if (substr($output,strlen($output)-2)==", ") {
				$output = substr($output,0,strlen($output)-2);
			}
		}

		if (!$groups && !$users) {
			return getPhrase("_ALL_EXTRANET_USERS");
		} else {
			return $output;
		}
	}
}

# function which returns the number of $type that are linked to the Group object
function getNumberOf($type, $group_id) {
	$group_id = (int) $group_id;
	if($type == "User") {

		$sql = "SELECT DISTINCT user_id
				FROM ".DB_NAME_PREFIX."group_user_link
				JOIN ".DB_NAME_PREFIX."users ON ( user_id = id )
				WHERE group_id = ".$group_id;
				
	} else {

		$sql = "SELECT DISTINCT entity_id
				FROM ".DB_NAME_PREFIX."group_entity_link
				WHERE group_id = ".$group_id."
					AND entity_type = '$type'
				";
	}

	$result = my_mysql_query($sql);
	$number = mysql_num_rows($result);

	return $number;
}

function getTotalNumberOfEntities( $groupId ) {
	$groupId = (int) $groupId;
	$sql = "
		SELECT count( content_id ) as num
			FROM ".DB_NAME_PREFIX."group_content_link
		WHERE group_id = " . $groupId;
	$result = my_mysql_query($sql);
	list( $quantity ) = mysql_fetch_row( $result );
	return $quantity;
}

function getAllLinkedEntities($group_id) {
	$group_id = (int) $group_id;
	$sql = "SELECT DISTINCT content_id,
				content_type,
				content_version
			FROM ".DB_NAME_PREFIX."group_content_link
			WHERE group_id = '" . mysql_real_escape_string($group_id) . "'
			ORDER BY content_type;
			";
	$result = my_mysql_query($sql);

	$count = 0;
	while($row = mysql_fetch_array ($result)) {
		$entities[$count] = array(
			'id' => $row[content_id],
			'type' => $row[content_type],
			'version' => $row[content_version]);
		$count++;
	}

	return $entities;
}



//ENTITIES

/** lookup displayed now for entity types
* @param string option type to lookup
* @return string displayed name
*/
function getDisplayedEntityType($type="") {

	$sql = "SELECT content_name
			FROM ".DB_NAME_PREFIX."contenttypes
			WHERE content_type = '";
	if($type) $sql .= mysql_real_escape_string($type);
	//else $sql .= getEntityType($this);
	$sql .= "'";
	$result = my_mysql_query($sql);
	$i = 0;

	$row = mysql_fetch_array ($result);
	return $row["content_name"];
}

//CONTENT TYPE
function getContentStatus($cIDOrAlias, $cType, $cVersion = false) {
	
	if (!$cVersion) {
		$cVersion = getLatestVersion($cIDOrAlias, $cType);
	}

	if ($cType) {
		if (is_numeric ($cIDOrAlias) or $cType != "html") {
			$sql = "SELECT status
					FROM ".DB_NAME_PREFIX."content_". mysql_real_escape_string($cType). "
					WHERE id = ". (int) $cIDOrAlias. "
						AND version = ". (int) $cVersion;
		} else {
			/* $cID cannot contain spaces, so let's just make sure it's tidy */
			$cID = str_replace(" ", "", $cID);
			$sql = "SELECT status
					FROM ".DB_NAME_PREFIX."content_". mysql_real_escape_string($cType). "
					WHERE alias = '". mysql_real_escape_string($cIDOrAlias). "'
						AND version = ". (int) $cVersion;
		}
		$result = my_mysql_query ($sql);
		if ($row = mysql_fetch_array ($result)) {
			return $row[0];
		} else {
			return false;
		}
	} else {
		return false;
	}
}



function resizeImage($image_width, $image_height, $constraint_width, $constraint_height) {
	/* Resize images upon upload */
	if (($image_width<=$constraint_width) && ($image_height<=$constraint_height)) {
		return array($image_width,$image_height);
	}
	
	if ($image_width>$constraint_width) {
		$width_out = $constraint_width;
		$height_out = $image_height * ($constraint_width/$image_width);
		
		if ($height_out>$constraint_height) {
			$width_out = $width_out * ($constraint_height/$height_out);
			$height_out = $constraint_height;
		}
	} elseif ($image_height>$constraint_height) {
		$height_out = $constraint_height;
		$width_out = $image_width * ($constraint_height/$image_height);
		
		if ($width_out>$constraint_width) {
			$height_out = $height_out * ($constraint_width/$width_out);
			$width_out = $constraint_width;
		}
	}

	return array($width_out,$height_out);
}

function resizeImageString(&$orig_image, $mime_type, &$width, &$height, $maxWidth, $maxHeight) {

	list($newWidth, $newHeight) = resizeImage($width, $height, $maxWidth, $maxHeight);
	
	if ($newWidth != $width || $newHeight != $height) {
		if ($image = imagecreatefromstring($orig_image)) {
			$resized_image = imagecreatetruecolor($newWidth, $newHeight);
									
			imagecopyresampled($resized_image, $image, 0, 0, 0, 0, $newWidth, $newHeight, $width, $height);
			
			$temp_file = tempnam(sys_get_temp_dir(), 'Img');
				if ($mime_type == 'image/gif') imagegif($resized_image, $temp_file);
				if ($mime_type == 'image/png') imagepng($resized_image, $temp_file);
				if ($mime_type == 'image/jpeg' || $mime_type == 'image/pjpeg') imagejpeg($resized_image, $temp_file);
				
				$orig_image = file_get_contents($temp_file);
			unlink($temp_file);
	
			$width = $newWidth;
			$height = $newHeight;
		} else {
			return "Problem with image";
		}
	}
}

function getUserImageSrc($userId = false, $usePlaceholder = true) {
	
	if ($userId === false) {
		$userId = $_SESSION['extranetUserID'];
	}
	
	if ($userId && ($checksum = getRow('user_images', 'checksum', array('user_id' => $userId, 'sticky_flag' => 1)))) {
		return 'cmsincludes/user_image.php?user_id='. (int) $userId. '&c='. $checksum;
	} elseif ($usePlaceholder) {
		return 'cmsincludes/user_image.php';
	} else {
		return;
	}

}



	
function outputStorekeeperImage($data,$width,$height,$mimeType) {
	$storekeeper_width_constraint = 190;
	$storekeeper_height_constraint = 190;
	
	
	if (($width > $storekeeper_width_constraint) || ($height > $storekeeper_height_constraint)) {
		// Resize image
		list($storekeeper_width,$storekeeper_height) = resizeImage($width,$height,$storekeeper_width_constraint,$storekeeper_height_constraint);
			
		// Round it
		$storekeeper_width = (int)$storekeeper_width;
		$storekeeper_height = (int)$storekeeper_height;
	} else {
		// If image width is less than constraint then round the image instead.
		$storekeeper_width = (int)$width;
		$storekeeper_height = (int)$height;
	}
	
	// Manipulate Image
	$imageResource = imagecreatefromstring($data);
	$resized_image = imagecreatetruecolor($storekeeper_width, $storekeeper_height);
	imagecopyresampled($resized_image, $imageResource, 0, 0, 0, 0, $storekeeper_width, $storekeeper_height, $width, $height);
							
	
	header("Content-type: " . $mimeType);
	if ($mimeType == "image/gif") imagegif($resized_image);
	if ($mimeType == "image/png") imagepng($resized_image);
	if ($mimeType == "image/jpeg" || $mimeType == "image/pjpeg") imagejpeg($resized_image);
}

function outputImage($data,$width,$height,$mimeType) {
	$imageResource = imagecreatefromstring($data);
	$resized_image = imagecreatetruecolor($width, $height);
	imagecopyresampled($resized_image, $imageResource, 0, 0, 0, 0, $width, $height, $width, $height);
							
	
	header("Content-type: " . $mimeType);
	if ($mimeType == "image/gif") imagegif($resized_image);
	if ($mimeType == "image/png") imagepng($resized_image);
	if ($mimeType == "image/jpeg" || $mimeType == "image/pjpeg") imagejpeg($resized_image);
	$img_data_thumb = ob_get_contents();
}



/* Content specific functions */


/* Documents */

function getShowableDocuments ($hide_private_items=true) {
	$sql = "SELECT id
			FROM " . DB_NAME_PREFIX . "content_document
			GROUP BY id";

	$result = my_mysql_query($sql);

	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			if (!$hide_private_items && !checkPriv()) {
				$version = getPublishedVersion($row['id'],"document");
			} else {
				$version = getShowableVersion($row['id'],"document",0,$_SESSION['admin_username'],$_SESSION["extranetUserID"]);
			}

			if ($version) {
				$sql = "SELECT title,
							type,
							status,
							published_datetime
						FROM " . DB_NAME_PREFIX . "content_document
						WHERE id = " . $row['id'] . "
							AND version = " . $version;

				$result2 = my_mysql_query($sql);

				$row2 = mysql_fetch_array($result2);

				$documents[] = array("id" => $row['id'], "version" => $version, "title" => $row2['title'], "type" => $row2['type'], "published_datetime" => $row2['published_datetime'], "status" => $row2['status']);
			}
		}

		return $documents;
	} else {
		return false;
	}
}


function documentTypeIsAllowed($type) {
	
	$sql = "
		SELECT 1
		FROM " . DB_NAME_PREFIX . "document_types
		WHERE `type` = '". mysql_real_escape_string($type). "'";
			
	$result = my_mysql_query($sql);
	
	return (bool) mysql_num_rows($result);
}

function getSearchtermParts ($searchString) {
	$searchString = str_replace('+', '', rawurldecode($searchString));
	
	//Break the search string up into tokens.
	//Normally we break by spaces, but you can use a pattern in double quotes to override this
	preg_match_all('/"([^"]+)"|(\S+)/', $searchString, $searchStrings, PREG_SET_ORDER);
	
	$quotesUsed = false;
	$searchWordsAndPhrases = array();
	foreach($searchStrings as $string) {
		if (isset($string[2])) {
			$searchWordsAndPhrases[$string[2]] = 'word';
		} else {
			$searchWordsAndPhrases[$string[1]] = 'phrase';
			$quotesUsed = true;
		}
	}
	
	//Just in case the user doesn't know about the "using quotes to group words together" feature,
	//add the whole phrase in as a search term
	if (!$quotesUsed && count($searchStrings > 1)) {
		$searchWordsAndPhrases[$searchString] = 'whole phrase';
	}
	
	return $searchWordsAndPhrases;
}




/* Categories */

function getCategories($id=0) {
	return cms_core::getCategories($id);
}

function getCategoryName($id) {
	$sql = "SELECT name
			FROM " . DB_NAME_PREFIX . "categories
			WHERE id = ". (int) $id;

	$result = my_mysql_query($sql);
	$row = mysql_fetch_assoc($result);
	return $row['name'];
}

function contentItemCategoryInCategoryList ($cID, $cType, $cVersion, $categoriesList) {
	if ($contentItemCategories = getContentItemCategories($cID,$cType,$cVersion)) {
		foreach ($categoriesList as $category) {
			foreach ($contentItemCategories as $contentItemCategory) {
				if (categoryIsAncestor($contentItemCategory['name'],$category)) {
					return true;
				}
			}
		}
		
		return false;
	} else {
		return false;
	}
}

function categoryIsAncestor ($childCategory, $ancestorCategory, $recurseCount=0) {
	if ($childCategory==$ancestorCategory) {
		return true;
	}

	$sql = "SELECT parent_id
			FROM " . DB_NAME_PREFIX . "categories
			WHERE name = '" . mysql_real_escape_string($childCategory) . "'";
			
	$result = my_mysql_query($sql);
	
	$recurseCount++;
	
	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
	
		if (isset($row['parent_id'])) {
			$sql = "SELECT name 
					FROM " . DB_NAME_PREFIX . "categories
					WHERE id = " . $row['parent_id'];
					
			$result2 = my_mysql_query($sql);
			
			if (mysql_num_rows($result2)>0) {
				$row2 = mysql_fetch_array($result2);
				
				if ($row2['name']==$ancestorCategory) {
					return true;
				} else {
					if ($recurseCount<100 && $row['parent_id']!=0) {
						return categoryIsAncestor ($row2['name'],$ancestorCategory,$recurseCount);
					} else {
						if ($childCategory==$ancestorCategory) {
							return true;
						} else {
							return false;
						}
					}
				}
			} else {
				if ($childCategory==$ancestorCategory) {
					return true;
				} else {
					return false;
				}
			}
		} else {
			if ($childCategory==$ancestorCategory) {
				return true;
			} else {
				return false;
			}
		}
	} else {
		return false;
	}
}

function getContentItemCategories($cID, $cType, $cVersion = false, $publicOnly = false, $langId = false, $sql = false) {

	if (!$cVersion) {
		$cVersion = getLatestVersion($cID, $cType);
	}
	
	if ($sql === false) {
		$sql = "
			SELECT 
				c.id,
				c.parent_id,
				c.name,
				c.public,
				c.landing_page_content_id,
				c.landing_page_content_type
			FROM " . DB_NAME_PREFIX . "categories AS c
			INNER JOIN " . DB_NAME_PREFIX . "category_item_link AS cil
				ON c.id = cil.category_id
			WHERE cil.item_id = " . (int) $cID . "
				AND cil.content_type = '" . mysql_real_escape_string($cType) . "'
				AND cil.version = " . (int) $cVersion;
	}
	
	if ($publicOnly) {
		$sql .= "
			AND c.public = 1";
	}
				
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			if (!$row['public']) {
				$row['public_name'] = false;
				$row['landing_page_content_id'] = false;
				$row['landing_page_content_type'] = false;
			} else {
				$row['public_name'] = getVLPPhrase('_CATEGORY_'. $row['id'], $replace = array(), $langId);
			}
			
			$categories[] = $row;
		}
		
		return $categories;
	} else {
		return false;
	}
}	

function getContentItemCategoriesString($categories) {
	$output = '';
	if ($categories) {
		foreach ($categories AS $category) {
			$output .= $category['name'] . ", ";
		}

		if (substr($output,strlen($output)-2)==", ") {
			$output = substr($output,0,strlen($output)-2);
		}
	}

	if (!$categories) {
		return getPhrase("_NO_CATEGORIES_SET");
	} else {
		return $output;
	}
}

function getCategoryContentItems ($cType,$categoryId) {
	$sql = "SELECT item_id
			FROM " . DB_NAME_PREFIX . "category_item_link
			WHERE content_type = '" . mysql_real_escape_string($cType) . "'
				AND category_id = " . (int) $categoryId . "
			GROUP BY item_id";
				
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			if ($version = getShowableVersion($row['item_id'],$cType,false,$_SESSION['admin_username'],$_SESSION['extranet_userid'])) {
				$contentItems[] = array("id" => $row['item_id'],"version" => $version);
			}
		}
		
		if ($contentItems) {
			return $contentItems;
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function getCategoryIdFromName ($categoryName) {
	$sql = "SELECT id
			FROM " . DB_NAME_PREFIX . "categories
			WHERE name = '" . mysql_real_escape_string($categoryName) . "'";
			
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
		
		return $row['id'];
	} else {
		return false;
	}
}

/* Miscellaneous */

function checkHomepageExists($homepage_alias) {
	$sql = "SELECT id
			FROM " . DB_NAME_PREFIX . "content_html
			WHERE alias = '" . mysql_real_escape_string($homepage_alias) . "'";

	$result = my_mysql_query($sql);

	if (mysql_num_rows($result)>0) {
		return true;
	} else {
		return false;
	}
}

function formatNicely($text,$limit) {
	
	$text = trim(strip_tags($text));
	
	if (!$text || mb_strlen($text) < $limit) {
		return $text;
	} else {
		$new_content = mb_substr($text,0,$limit);
		
		$new_content = mb_ereg_replace('[^\s,]+$','',$new_content);
		$new_content = mb_ereg_replace('[\s\t\n\r]+$','',$new_content);	
		$new_content = mb_ereg_replace(',+$','',$new_content);	
			
		if (preg_match('/[\.]$/',$new_content) || !$text) {
			return $new_content;
		} else {
			return $new_content. '...';
		}
		
		return $new_content;
	}
}

function getDateErrors($date) {
	
	// Do Date Format checks if it exists
	if ($date) {
		// _MSG_DATE_FORMAT
		if (preg_match("/\d{4}-\d{2}-\d{2}/",$date)) {
			$digit_array_check = explode("-",$date);
		
			$date_year_error = false;
			$date_month_error = false;
			$date_day_error = false;
			$date_error_string = "";
			
			// Digit check
			$digit_year = (int)$digit_array_check[0];
			$digit_month = (int)$digit_array_check[1];
			$digit_day = (int)$digit_array_check[2];				
		
			// Date 0000-00-00 should be a null entry in db
			if ((!$digit_year) && (!$digit_month) && (!$digit_day)) {
				$date = ""; 
			} else {
			
				if (!$digit_year) $date_year_error = true;
				if (($digit_month < 1) || ($digit_month > 12)) $date_month_error = true;
				if (($digit_day < 1) || ($digit_day > 31)) $date_day_error = true;	
				
					
				// If no errors in date then, Format for SQL insert
				if ((!$date_year_error) && (!$date_month_error) && (!$date_day_error)) {
					//$date = "'".$digit_array_check[0]."-".$digit_array_check[1]."-".$digit_array_check[2]."'";
					return false;					
				} else {
					if ($date_year_error) $date_error_string .= getPhrase("_MSG_DATE_YEAR")."<br />";
					if ($date_month_error) $date_error_string .= getPhrase("_MSG_DATE_MONTH")."<br />";
					if ($date_day_error) $date_error_string .= getPhrase("_MSG_DATE_DAY")."<br />";
					return getPhrase("_MSG_DATE_FORMAT")."<br />".$date_error_string;
				}
			}
		} else return getPhrase("_MSG_DATE_FORMAT");
	}
}

function checkDateValid ($date) {
	if ($date) {
		if (preg_match("/\d{4}-\d{2}-\d{2}/",$date)) {
			$digit_array_check = explode("-",$date);
		
			// Digit check
			$digit_year = (int)$digit_array_check[0];
			$digit_month = (int)$digit_array_check[1];
			$digit_day = (int)$digit_array_check[2];				
		
			// Date 0000-00-00 should be a null entry in db
			if ((!$digit_year) && (!$digit_month) && (!$digit_day)) {
				return false;
			} else {
				if (!$digit_year) return false;
				if ($digit_month < 1 || $digit_month > 12) return false;
				if ($digit_day < 1 || $digit_day > 31) return false;
				return true;
				
			}
		} else {
			return false;
		}
	} else {
		return false;
	}
}

function formatDateNicely($date, $format_type = false, $lang = false, $time_format = '', $rss = false) {

	
	if ($time_format === true) {
		$time_format = ' %H:%i';
	}
	
	if ($rss) {
		$format_type = '%a, %d %b %Y';
		$time_format = ' %H:%i:%s ';
	
	} elseif (!$format_type || $format_type == 'vis_date_format_long' || $format_type == '_LONG') {
		$format_type = getSiteConfig('vis_date_format_long');
	
	} elseif ($format_type == 'vis_date_format_med' || $format_type == '_MEDIUM') {
		$format_type = getSiteConfig('vis_date_format_med');
	
	} elseif ($format_type == 'vis_date_format_short' || $format_type == '_SHORT') {
		$format_type = getSiteConfig('vis_date_format_short');
	}
	
	
	$sql = "SELECT DATE_FORMAT('" . mysql_real_escape_string($date) . "', '" . mysql_real_escape_string($format_type. $time_format) . "')";
	
	$result = my_mysql_query($sql);
	list($formattedDate) = mysql_fetch_row($result);
	
	$returnDate = '';
	if ($rss) {
		$returnDate = $formattedDate;
		
		if ($time_format) {
			$sql = "SELECT TIME_FORMAT(NOW() - UTC_TIMESTAMP(), '%H%i') ";
			$result = my_mysql_query($sql);
			list($timezone) = mysql_fetch_row($result);
			
			if (substr($timezone, 0, 1) != '-') {
				$timezone = '+'. $timezone;
			}
			
			$returnDate .= $timezone;
		}
		
	} else {
		foreach (preg_split('/\[\[([^\[\]]+)\]\]/', $formattedDate, -1,  PREG_SPLIT_DELIM_CAPTURE) as $part) {
			$returnDate .= getVLPPhrase($part, array(), $lang);
		}
	}
	
	return $returnDate;
}

function formatDateTimeNicely($date, $format_type = false, $lang = false, $rss = false) {
	return formatDateNicely($date, $format_type, $lang, true, $rss);
}

function formatTimeNicely($time,$format_type,$lang) {
	$sql = "SELECT TIME_FORMAT('" . mysql_real_escape_string($time) . "', '" . mysql_real_escape_string($format_type) . "') as new_time";
	$result = my_mysql_query($sql);
	$row = mysql_fetch_array($result);
	if($row) {
		$new_time = $row['new_time'];
	} 
	return $new_time;
}

function formatFilesizeNicely($size, $precision = 0, $adminMode = false, $vlpClass = '') {
	
	if (is_array($size)) {
		$size = $size['size'];
	}
	
	//Use getPhrase is in admin mode, or getVLPPhrase otherwise
	$getPhrase = $adminMode? 'getPhrase' : 'getVLPPhrase';
	
	//Return 0 without formating if the size is 0.
	if ($size <= 0) {
		return '0';
	}
	
	//Define labels to use
	$labels = array('_BYTES', '_KBYTES', '_MBYTES', '_GBYTES', '_TBYTES');
	
	//Work out which of the labels to use, based on how many powers of 1024 go into the size, and
	//how many labels we have
	$order = min(
				floor(
					log($size) / log(1024)
				),
			  count($labels)-1);
	
	return $getPhrase($labels[$order],
		array('size' => 
			round($size / pow(1024, $order), $precision)
		),
		false, false, $vlpClass
	);

}

function formatFileTypeNicely($type, $vlpClass = '') {
	switch($type) {
	case "image/jpeg": 
		$new_type = getVLPPhrase("_JPEG_file", array(), false, false, $vlpClass);
		break;
	case "image/pjpeg": 
		$new_type = getVLPPhrase("_JPEG_file", array(), false, false, $vlpClass);
		break;
	case "image/jpg": 
		$new_type = getVLPPhrase("_JPG_file", array(), false, false, $vlpClass);
		break;
	case "image/gif": 
		$new_type = getVLPPhrase("_GIF_file", array(), false, false, $vlpClass);
		break;
	case "image/png": 
		$new_type = getVLPPhrase("_PNG_file", array(), false, false, $vlpClass);
		break;
	default:
		$new_type = getVLPPhrase("_UNDEFINED", array(), false, false, $vlpClass);
	}
	return $new_type;
}



//Check if the random number generator has been seeded yet, and seed it if not
function seedRandomNumberGeneratorIfNeeded() {
	//Use whether the RANDOM_NUMBER_GENERATOR_IS_SEEDED constant is defined to remember
	//if we've seeded yet
	if (!defined('RANDOM_NUMBER_GENERATOR_IS_SEEDED')) {
		//If we need to seed the random number generator, get the number of microseconds
		//past the current second, and use that to seed
		$x = explode(' ', microtime());
		mt_srand((int) (1000000 * $x[0]));
		
		//Define the RANDOM_NUMBER_GENERATOR_IS_SEEDED constant so that we don't seed again
		define('RANDOM_NUMBER_GENERATOR_IS_SEEDED', true);
	}
}

//Generate a random string of numbers and letter
function randomString($requiredLength = 12) {
	
	seedRandomNumberGeneratorIfNeeded();
	
	//Only use certain characters. (I've stripped out vowels, just in case a swearword
	//is randomly generated. Also 1s look too much like ls and 0s look too much like os
	$lettersToUse = str_split('BCDFGHJKLMNPQRSTVWXYZbcdfghjklmnpqrstvwxyz23456789');
	$stringOut = '';
	
	//Loop while our output string is still too short
	while (strlen($stringOut) < $requiredLength) {
		//Generate a random number, convert it into lots of little numbers between 0 and
		//49 (our string of characters above is 50 long), convert each to one of the characters
		//from our string of characters, then append each to the output string
		foreach(str_split((string) mt_rand(), 2) as $chunk) {
			$stringOut .= $lettersToUse[((int) $chunk) % 50];
		}
	}
	
	//We might have made out string too long, chop it if needed and return it.
	return substr($stringOut, 0, $requiredLength);
}


//Returns an array of password strength names to scores
//Defined as a function so that it is only hardcoded here, rather
//than everywhere it is used.
function passwordStrengthsToValues($strength = false) {
	$a = array('_WEAK' => 0, '_MEDIUM' => 35, '_STRONG' => 70, '_VERY_STRONG' => 100);
	//Either return an array, or if $strength is set, return the score for that strength
	if ($strength) {
		return $a[$strength];
	} else {
		return $a;
	}
}

function passwordValuesToStrengths($value) {
	if ($value < 35) {
		return '_WEAK';
	} elseif ($value < 70) {
		return '_MEDIUM';
	} elseif ($value < 100) {
		return '_STRONG';
	} else {
		return '_VERY_STRONG';
	}
}


//Calculate the bonus for using different sets of characters
	//e.g. lower case, upper case, numeric, non-alphanumeric...
function calculateBonusFromUsage($n) {
	return min(2, $n);
}

//Check if a given password meets the strength requirements.
function checkPasswordStrength($pass, $passwordStrengthRequired = false) {
	
	if (!$passwordStrengthRequired
	  //As above, $passwordStrengthRequired defaults to the site setting if not specified
	  && !($passwordStrengthRequired = getSiteConfig('min_admin_password_strength'))) {
		//Default to '_MEDIUM' if the site setting is not there
		$passwordStrengthRequired = '_MEDIUM';
	}
	
	//Count the number of lower case, upper case, numeric and non-alphanumeric
	//characters to work out the complexity of a password.
	
	//E.g., a password using only lower case letters is not very complex, and will
	//probably be cracked faster than one that mixes upper case, lower case and numbers in.
	
	//Work out a measure of the complexity by checking if it uses one of each different type
	//of character. Bonus points are also given for each type used twice.
	$lower = strlen(preg_replace('#[^a-z]#', '', $pass));
	$lower = calculateBonusFromUsage($lower);
	
	$upper = strlen(preg_replace('#[^A-Z]#', '', $pass));
	$upper = calculateBonusFromUsage($upper);
	
	$numeric = strlen(preg_replace('#[^0-9]#', '', $pass));
	$numeric = calculateBonusFromUsage($numeric);
	
	$nonalpha = strlen(preg_replace('#[a-z0-9]#i', '', $pass));
	$nonalpha = calculateBonusFromUsage($nonalpha);
	
	$weighting = $lower + $upper + $numeric + $nonalpha;
	//Multiply by 1.5.
	//(There's no reason for doing this, other than to make the power bar a bit wide that it
	// would otherwise be.)
	$weighting *= 1.5;
	
	//Score the password by multiplying the length by our estimate of the complexity.
	$score = strlen($pass) * $weighting;
	//E.g. a password of "p_441ngs" is stronger than a password of "puddings"
		//They are the same length, but "p_441ngs" is more complex
	//Or e.g. a password of "ireallyliketoeatpuddings" is stronger than a password of "p_441ngs"
		//"p_441ngs" is more complex, but "ireallyliketoeatpuddings" is a lot longer
	
	if ($passwordStrengthRequired === true) {
		//Return the actual score if $passwordStrengthRequired was set to true
		return $score;
	} else {
		//Otherwise return whether the score was good enough
		return $score >= passwordStrengthsToValues($passwordStrengthRequired);
	}
}





function getEmailTemplate ($name) {
	$sql = "SELECT subject,
				body
			FROM " . DB_NAME_PREFIX . "email_templates
			WHERE template_name = '" . mysql_real_escape_string($name) . "'";
	
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		$row = mysql_fetch_array($result);
		return $row;
	} else {
		return false;
	}
}

/* Contenttype specific functions */

function updateUserForumEntries($user_id,$new_user_id) {
	$user_id = (int) $user_id;
	$new_user_id = (int) $new_user_id;
	
	$sql = "UPDATE " . DB_NAME_PREFIX . "forumthread
			SET start_user_id = " . $new_user_id . "
			WHERE start_user_id = " . $user_id;
	$result = my_mysql_query($sql);

	$sql = "UPDATE " . DB_NAME_PREFIX . "forumpost
			SET writer_user_id = " . $new_user_id . "
			WHERE writer_user_id = " . $user_id;
	$result = my_mysql_query($sql);

}



/*  Plugins  */

function getPluginDetails($idOrName, $fetchBy = 'id') {
	$sql = "
		SELECT
			id,
			id AS plugin_id,
			name,
			name AS plugin_name,
			display_name,
			class_name,
			vlp_class,
			status,
			default_framework,
			default_theme,
			uses_instances,
			keep_in_album
		FROM " . DB_NAME_PREFIX . "plugins";
	
	if ($fetchBy == 'class') {
		$sql .= "
			WHERE class_name = '" . mysql_real_escape_string($idOrName) . "'
			  AND status IN ('_ENUM_RUNNING', '_ENUM_INITIALISED')";
	
	} elseif ($fetchBy == 'name') {
		$sql .= "
			WHERE name = '" . mysql_real_escape_string($idOrName) . "'";
	
	} else {
		$sql .= "
			WHERE id = " . (int) $idOrName;
	}
	
	$result = my_mysql_query($sql);
	
	if (!$plugin = mysql_fetch_assoc($result)) {
		return false;
	} else {
		//Work out what the plugin's directory will be
		$plugin['rel_path'] = "plugins/" . $plugin['name'] . "/";
		$plugin['path'] = CMS_ROOT . $plugin['rel_path'];

		return $plugin;
	}
}

function getPluginName($id) {
	$plugin = getPluginDetails($id);
	return $plugin['name'];
}

function getPluginDisplayName($id) {
	$plugin = getPluginDetails($id);
	return $plugin['display_name'];
}

function getPluginDisplayNameByClassName($name) {
	$plugin = getPluginDetails($name, 'class');
	return ifNull($plugin['display_name'], $name);
}

function getPluginClass($id) {
	$plugin = getPluginDetails($id);
	return $plugin['class_name'];
}

function getPluginId($name) {
	$plugin = getPluginDetails($name, 'name');
	return $plugin['id'];
}

function getPluginStatus($id) {
	$plugin = getPluginDetails($id);
	return $plugin['status'];
}

function getPluginStatusByName($name) {
	$plugin = getPluginDetails($name, 'name');
	return $plugin? $plugin['status'] : false;
}

function getPluginStatusByClassName($name) {
	$plugin = getPluginDetails($name, 'class');
	return $plugin? $plugin['status'] : false;
}

function getPluginIdByClassName($name) {
	$plugin = getPluginDetails($name, 'class');
	return $plugin['id'];
}

function getPluginClassNameByName($name) {
	$plugin = getPluginDetails($name, 'name');
	return $plugin['class_name'];
}


//Get all existing plugins
function getPlugins($onlyGetRunningPlugins = false, $ignoreUninstalledPlugins = false, $dbUpdateSafemode = false, $orderBy = false) {
	
	$result = false;
	
	//For certain parts of the CMS (e.g. the database updater), this query may cause errors, yet we'd still like to continue smoothly
	//(albeit without a list of plugins)
	if ($dbUpdateSafemode) {
		$sql = "
			SELECT id, name, class_name
			FROM " . DB_NAME_PREFIX . "plugins
				WHERE status != '_ENUM_INSTALLED'";
		$result = @mysql_query($sql);

	} else {
		$sql = "
			SELECT
				id,
				id AS plugin_id,
				name,
				name AS plugin_name,
				display_name,
				class_name,
				vlp_class,
				status,
				default_framework,
				default_theme,
				uses_instances
			FROM " . DB_NAME_PREFIX . "plugins";
		
		//Add in certain conditions, depending on where this is being called
		if ($onlyGetRunningPlugins) {
			$sql .= "
				WHERE status = '_ENUM_RUNNING'";
		} elseif ($ignoreUninstalledPlugins) {
			$sql .= "
				WHERE status != '_ENUM_INSTALLED'";
		}
		$sql .= "
				ORDER BY ". ifNull($orderBy, "status = '_ENUM_RUNNING' DESC, status, name");
		
		$result = my_mysql_query($sql);
	}
	
	//Fetch details on each plugin
	if ($result && mysql_num_rows($result)>0) {
		while ($plugin = mysql_fetch_assoc($result)) {
		
			//Is the plugin's prefix defined? Define it now if not.
			setPluginPrefix($plugin);
			
			//Work out what the plugin's directory will be
			$plugin['rel_path'] = "plugins/" . $plugin['name'] . "/";
			$plugin['path'] = CMS_ROOT . $plugin['rel_path'];

			$plugins[$plugin['id']] = $plugin;
		}
		
		return $plugins;
	} else {
		return false;
	}
}

//Get all of the existing plugins that are running
function getRunningPlugins($orderBy = false) {
	return getPlugins($onlyGetRunningPlugins = true, false, false, $orderBy);
}


function canActivatePlugin($name, $fetchBy = 'name', $activate = false) {
	
	$error = array();
	
	if ($plugin = getPluginDetails($name, $fetchBy)) {
		if ($plugin['status'] != "_ENUM_RUNNING") {
			$error[] = "_PLUGIN_INCORRECT_STATUS" . $status;
		} elseif (!includePluginAndDependencies($plugin['name'], $missingPlugin)) {
			$error[] = "_PLUGIN_DOES_NOT_EXIST";
		}
	} else {
		$error[] = "_PLUGIN_DOES_NOT_EXIST";
	}
	
	
	if ($activate) {
		if (count($error)) {
			return false;
		} else {
			
			setPluginPrefix($plugin);
			$class = new $plugin['class_name'];
			$class->setInstance(array(false, false, false, false, false, false, false, $plugin['name'], $plugin['class_name'], $plugin['vlp_class'], $plugin['id'], false, false, false, false, false));

			return $class;
		}
	} else {
		return $error;
	}
}

function activatePlugin($name) {
	return canActivatePlugin($name, 'name', $activate = true);
}

function activatePluginClass($name) {
	return canActivatePlugin($name, 'class', $activate = true);
}


function inc($pluginClass) {
	$missingPlugin = array();
	if (($plugin = getPluginDetails($pluginClass, 'class'))
	 && (includePluginAndDependencies($plugin['plugin_name'], $missingPlugin))) {
		setPluginPrefix($plugin);
		return true;
	} else {
		return false;
	}
}

function getPluginDependencies($pluginName) {
	$sql = "
		SELECT
			d.dependency_plugin_class_name,
			d.`type`,
			p.name AS plugin_name,
			p.class_name,
			p.id AS plugin_id,
			p.vlp_class
		FROM ". DB_NAME_PREFIX. "plugin_dependencies AS d
		LEFT OUTER JOIN ". DB_NAME_PREFIX. "plugins AS p
		   ON d.dependency_plugin_class_name = p.class_name
		  AND p.status = '_ENUM_RUNNING'
		WHERE d.plugin_name = '". mysql_real_escape_string($pluginName). "'
		  AND `type` IN ('dependency', 'inheritance')";
	
	$depends = array();
	$result = my_mysql_query($sql);
	while($row = mysql_fetch_assoc($result)) {
		$depends[] = $row;
	}
	
	return $depends;
}


//Include all of a Plugin's Dependency files, then include the Plugin
//Note: you need to check to see if a Plugin is running first, before calling this
function includePluginAndDependencies($pluginName, &$missingPlugin, $recurseCount = 9) {
	
	if (!$recurseCount) {
		return false;
	}
	
	//Check that this has not been included already - if so, our job has already been done
	$file = CMS_ROOT. 'plugins/'. $pluginName. '/plugin_code.php';
	if (in_array($file, get_included_files())) {
		return true;
	}

	//Check for dependencies
	foreach (getPluginDependencies($pluginName) as $row) {
		//For each dependency, check if it is running and try to include it
		if ($row['plugin_id'] && includePluginAndDependencies($row['plugin_name'], $missingPlugin, $recurseCount-1)) {
			setPluginPrefix($row);
		} else {
			//Otherwise report a dependancy as missing, then stop
			$missingPlugin = array('plugin' => $row['dependency_plugin_class_name']);
			return false;
		}
	}
	
	$missingPlugin = false;
	if (file_exists($file)) {
		require_once $file;
		return true;
	} else {
		return false;
	}
}

//Define a plugin's database table prefix
function setPluginPrefix(&$plugin) {
	
	$plugin['prefix'] = strtoupper($plugin['class_name']). '_PREFIX';
	
	//Is the plugin's prefix defined? Define it now if not.
	if (!defined($plugin['prefix'])) {
		
		if (issetArrayKey($plugin, 'plugin_id')) {
			define($plugin['prefix'], 'plg'. $plugin['plugin_id']. '_');
		
		} elseif (issetArrayKey($plugin, 'id')) {
			define($plugin['prefix'], 'plg'. $plugin['id']. '_');
		}
	}
}


/* Instances and plugin settings */

function getPluginInheritance($pluginClassName) {
	$sql = "
		SELECT dependency_plugin_class_name
		FROM ".  DB_NAME_PREFIX. "plugin_dependencies
		WHERE plugin_class_name = '". mysql_real_escape_string($pluginClassName). "'
		  AND `type` = 'inheritance'
		LIMIT 1";
	
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_assoc($result)) {
		return $row['dependency_plugin_class_name'];
	} else {
		return false;
	}
}


//Returns the name of the currently running plugin, in upper-case.
//Must be called from code within the plugin's own folder with the __FILE__ Magic Constant
//The main reason for this function is to use in the latest_revision_no.inc.php files, to keep them
//nice and tidy.
function pluginName($file) {
	//Take the current path
		//Match up to and including the plugins directory with .*[/\\]plugins[/\\]
		//The next sequence of chars will be the plugins directory
		//There may be another slash, or to the path - match with [/\\]?.*
	return strtoupper(preg_replace('#.*[/\\\\]plugins[/\\\\]([^/\\\\]*)[/\\\\]?.*#', '\1', dirname($file)));
}


/* Plugin Slots */

//Get a list of every plugin instance currently running on a page
	//Could do with rewriting so that only plugins on a page are run...
function getSlotContents(&$slotContents, $cID, $cType, $cVersion, $templateId, $templateName, $familyName, $skinId, $specificInstanceId = false, $specificSlotName = false, $ajaxReload = false) {
	return require 'cmsincludes/visitor/getSlotContents.fun.php';
}


//Activate and setup a plugin
//Note that the function canActivatePlugin() or equivalent should be called on the plugin's name before calling this.
function setInstance(&$instance, $cID, $cType, $cVersion, $skinId, $slotName, $checkForErrorPages = false, $nest = 0, $tab = 0) {
	
	$missingPlugin = false;
	if (!includePluginAndDependencies($instance['plugin_name'], $missingPlugin)) {
		$instance['class'] = false;
		return false;
	}
	
	$instance['class'] = new $instance['class_name'];
	
	if (!isset($_GET['slotName']) || $_GET['slotName'] == $slotName) {
		$instance['class']->setGETVariables($_GET);
	} else {
		$instance['class']->setGETVariables(array());
	}
	
	if (!isset($_POST['slotName']) || $_POST['slotName'] == $slotName) {
		$instance['class']->setPOSTVariables($_POST);
	} else {
		$instance['class']->setPOSTVariables(array());
	}
	
	$instance['class']->setInstance(array($cID, $cType, $cVersion, $skinId, $slotName,
										  $instance['instance_name'], $instance['instance_id'],
										  $instance['plugin_name'], $instance['class_name'],
										  $instance['vlp_class'],
										  $instance['plugin_id'],
										  $instance['default_framework'], $instance['framework'],
										  $instance['default_theme'], $instance['theme'],
										  $instance['level']), false, $nest, $tab);
	
	//Work out whether we are displaying this Plugin
	if ((
		//If this is an error page, ask the Plugin if it minds still being displayed.
		$checkForErrorPages && !($cID || ($instance['class']->allowPluginToBeShownOnErrorPage()))
	 ) || (
		//Run the plugin's own initalisation routine. If it returns true, then display the plugin.
		//(But note that plugins are always displayed in admin mode.)
		!($instance['init'] = (bool) $instance['class']->init()) && !(checkPriv())
	)) {
		$instance['class'] = false;
		return false;
	} else {
		return true;
	}
}

function getPluginInstanceDetails($instanceIdOrName, $fetchBy = 'id') {
	
	if (!$instanceIdOrName) {
		return false;
	}
	
	$sql = "
		SELECT
			i.id AS instance_id,
			i.name AS instance_name,
			i.keep_in_album,
			IF(i.framework = '', p.default_framework, i.framework) AS framework,
			i.use_custom_framework,
			p.default_framework,
			p.default_theme,
			i.plugin_id,
			p.name AS plugin_name,
			p.class_name,
			p.vlp_class,
			p.status
		FROM ". DB_NAME_PREFIX. "plugin_instances AS i
		INNER JOIN ". DB_NAME_PREFIX. "plugins AS p
		   ON p.id = i.plugin_id";
	
	if ($fetchBy == 'id') {
		$sql .= "
		WHERE i.id = ". (int) $instanceIdOrName;
	
	} elseif ($fetchBy == 'name') {
		$sql .= "
		WHERE i.name = '". mysql_real_escape_string($instanceIdOrName). "'";
	
	} else {
		return false;
	}
	
	$result = my_mysql_query($sql);
	return mysql_fetch_assoc($result);
}

function getPluginInstanceName($instanceId) {
	$instanceDetails = getPluginInstanceDetails($instanceId);
	return $instanceDetails['instance_name'];
}

function getPluginInstanceInItemSlot($slotName, $cID, $cType = 'html', $cVersion = false) {
	
	if (!$cVersion) {
		$cVersion = getLatestVersion($cID, $cType);
	}
	
	$sql = "
		SELECT instance_id
		FROM ". DB_NAME_PREFIX. "plugin_inst_item_link
		WHERE slot_name = '". mysql_real_escape_string($slotName). "'
		  AND content_id = ". (int) $cID. "
		  AND content_type = '". mysql_real_escape_string($cType). "'
		  AND content_version = ". (int) $cVersion;
	
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_assoc($result)) {
		return $row['instance_id'];
	} else {
		return false;
	}
}

function getPluginInstanceInTemplateSlot($slotName, $templateFamily, $templateId = false) {
	
	if ($templateId) {
	
		$sql = "
			SELECT instance_id
			FROM ". DB_NAME_PREFIX. "plugin_inst_temp_link
			WHERE slot_name = '". mysql_real_escape_string($slotName). "'
			  AND family_name = '". mysql_real_escape_string($templateFamily). "'
			  AND template_id = ". (int) $templateId;
	
	} else {
	
		$sql = "
			SELECT instance_id
			FROM ". DB_NAME_PREFIX. "plugin_inst_temp_fam_link
			WHERE slot_name = '". mysql_real_escape_string($slotName). "'
			  AND family_name = '". mysql_real_escape_string($templateFamily). "'";
	}
	
	$result = my_mysql_query($sql);
	if ($row = mysql_fetch_assoc($result)) {
		return $row['instance_id'];
	} else {
		return false;
	}
}

function sendSignal($signalName, $signalParams) {
	
	//Don't try to send a signal if we are in the Admin Login Screen applying Database Updates
	if (!class_exists('cms_core')) {
		return false;
	}
	
	if (issetArrayKey(cms_core::$signalsCurrentlyTriggered, $signalName)) {
		return false;
	}
	
	cms_core::$signalsCurrentlyTriggered[$signalName] = true;
	
		$sql = "
			SELECT plugin_id, plugin_name, plugin_class AS class_name, static_method
			FROM ". DB_NAME_PREFIX. "signals
			WHERE signal_name = '". mysql_real_escape_string($signalName). "'
			  AND plugin_class NOT IN (
			  	SELECT suppresses_plugin_class
			  	FROM ". DB_NAME_PREFIX. "signals AS e
			  	INNER JOIN ". DB_NAME_PREFIX. "plugins AS p
			  	   ON e.plugin_id = p.id
			  	WHERE e.signal_name = '". mysql_real_escape_string($signalName). "'
			  	  AND e.suppresses_plugin_class != ''
			  	  AND p.status = '_ENUM_RUNNING'
			  )
			ORDER BY signal_name, plugin_class";

		$returns = array();
		$result = my_mysql_query($sql);
		while($row = mysql_fetch_assoc($result)) {
			if (inc($row['class_name'])) {
				if ($row['static_method']) {
					$returns[$row['class_name']] = call_user_func_array(array($row['class_name'], $signalName), $signalParams);
				} else {
					$plugin = new $row['class_name'];
					$returns[$row['class_name']] = call_user_func_array(array($plugin, $signalName), $signalParams);
				}
			}
		}
	
	unset(cms_core::$signalsCurrentlyTriggered[$signalName]);
	return $returns;
}



/* Skins */

function getSkin($cID, $cType = 'html', $cVersion, $templateId = false, $array = false) {

	if (!$templateId) {
		$templateId = getTemplateIdFromContentItem($cID, $cType, $cVersion);
	}
	
	if (($skin = checkForActiveContentItemSkin($cID, $cType, $cVersion, $array))
	 || ($skin = checkForActiveTemplateSkin($templateId, $array))
	 || ($skin = checkForActiveTemplateFamilySkin($templateId, $array))) {
		return $skin;
	} else {
		return false;
	}
}

function checkForActiveTemplateFamilySkin ($template_id, $array = false) {
	$template = getTemplateDetails($template_id);
	if ($skinId = getRow('template_family', 'skin_id', array('family_name' => $template['family_name']))) {
		$skin = getSkinFromId($skinId, true);
		return $array? $skin : $skin['name'];
	} else {
		return false;
	}
}

function checkForActiveTemplateSkin ($template_id, $array = false) {
	if ($skinId = getRow('templates', 'skin_id', array('template_id' => $template_id))) {
		$skin = getSkinFromId($skinId, true);
		return $array? $skin : $skin['name'];
	} else {
		return false;
	}
}

function checkForActiveContentItemSkin ($cID, $cType, $cVersion, $array = false) {
	if ($skinId = getRow('content_'. $cType, 'skin_id', array('id' => $cID, 'version' => $cVersion))) {
		$skin = getSkinFromId($skinId, true);
		return $array? $skin : $skin['name'];
	} else {
		return false;
	}
}

function getSkins ($templateFamily, $skinId = false) {
	$sql = "SELECT id,
				family_name,
				name,
				active,
				1 AS uses_in_template_families,
				1 AS uses_in_templates,
				1 AS uses_in_content_items
			FROM " . DB_NAME_PREFIX . "skins
			WHERE ";
	
	if ($skinId) {
		$sql .= "id = " . (int) $skinId;
	} else {
		$sql .= "family_name = '" . mysql_real_escape_string($templateFamily) . "'";
	}
			
	$result = my_mysql_query($sql);
	
	if (mysql_num_rows($result)>0) {
		while ($row = mysql_fetch_array($result)) {
			if ($skinId) {
				return $row;
			} else {
				$skins[] = $row;
			}
		}
		
		return $skins;
	} else {
		return false;
	}
}

function getSkinFromId($skinId, $activeOnly = false) {
	$values = array('id' => $skinId);
	if ($activeOnly) {
		$values['active'] = 1;
	}
	return getRow('skins', array('id', 'family_name', 'name', 'related_plugin_themes'), $values);
}

function getSkinName($skinId) {
	$skin = getSkinFromId($skinId);
	return $skin && $skin['id']? $skin['name'] : getPhrase('_NOT_SET');
}



//Loads a skin description from a skin id
function validateSkinFromXMLDescription($skinId, &$skin, &$desc) {
	
	if (!$skin = getSkinFromId($skinId)) {
		return false;
	}
	
	return validateSkinFromXMLDescriptionByName($skin['family_name'], $skin['name'], $desc);
}

//Loads a skin description from a skin name
function validateSkinFromXMLDescriptionByName($familyName, $skinName, &$desc) {
	
	//Attempt to open and read the XML description
	try {
		if (!$desc = SimpleXML(CMS_ROOT. 'templates/'. SITENAME. '/'. $familyName. '/skins/'. $skinName. '/skin.xml')) {
			return false;
		}
	} catch (Exception $e) {
		return false;
	}
	
	return true;
}



function stripBadCharsForXMLParser ($input) {
	return str_replace(chr(12),"",$input);
}




//Open a file and attempt to turn it into a SimpleXMLElement
function SimpleXML($file) {
	
	if (!(is_file($file)) || (!$string = file_get_contents($file))) {
		return false;
	}
	
	$xml = new SimpleXMLElement($string);
	
	return $xml;
}
	


function assocArrayToXML ($data) {
	$xml = new XmlWriter();
	$xml->openMemory();
	$xml->startDocument('1.0', 'UTF-8');
	$xml->startElement('root');
	
	function write(XMLWriter $xml, $data, $recurseCounter){
		$recurseCounter++;
		
		foreach($data as $key => $value){
			if ($recurseCounter==1) {
				$key = "row";
			}
		
			if (!is_numeric($key)) {
				if(is_array($value)){
					$xml->startElement($key);
					write($xml, $value, $recurseCounter);
					$xml->endElement();
					continue;
				}
				
				$xml->writeElement($key, $value);
			}
		}
	}
	
	write($xml, $data,$recurseCounter);
	$xml->endElement();
	return $xml->outputMemory(true);	
}


require_once 'cmsincludes/extranet-cms.inc.php';


?>