<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


define('CMS_FRAMEWORK_CONDITION', 0);
define('CMS_FRAMEWORK_HTML', 1);
define('CMS_FRAMEWORK_INCLUDE', 2);
define('CMS_FRAMEWORK_INPUT', 3);
define('CMS_FRAMEWORK_MERGE', 4);
define('CMS_FRAMEWORK_PHRASE', 5);
define('CMS_FRAMEWORK_SECTION', 6);

/**
 * This class contains core plugin functions that the CMS calls in order to display plugins
 * They're not really part of the plugin API;
 * plugin developers don't need to be aware of them and should never call them from within a plugin.
 */

class cms_plugin_api_link extends plugin_placeholder_methods_admin {
	
	

	/**
	 * Store the settings for an instance of a plugin.
	 * 
	 * Will set automatically by the CMS when the plugin is displayed in a slot
	 */
	private $cmsPluginApiLinkSettings;

	/**
	 * Most plugins should be able to be reloaded via AJAX
	 * 
	 * However if there is a good reason why yours can't, if this variable is set to true by the
	 * init() function then the page reload will be forced instead.
	 */
	private $cmsPluginApiLinkForcePageReloadVar = false;
	private $cmsPluginApiLinkHeaderRedirectLocation = false;

	/**
	 * Store a list of JavaScript functions to run
	 */
	private $showInFloatingBox = false;
	private $cmsPluginApiLinkScrollToTop = null;

	/**
	 * Store a list of JavaScript functions to run
	 */
	private $scripts = array();
	private $scriptsBefore = array();

	/**
	 * Marks whether to display HTML tags around the plugin when using the showAlone() method
	 */
	private $notHTMLContent;

	/**
	 * Is this plugin currently in some kind of "edit" mode for an admin?
	 * Set by the markSlotAsBeingEdited function
	 */
	private $slotBeingEdited;

	/**
	 * Stores objects for a Plugin's StoreKeeper screen
	 */
	protected $storekeeperPanel;
	
	
	/**
	 * Stores the theme/framework for this plugin.
	 * 
	 * Will set automatically by the CMS when the plugin is displayed in a slot
	 */
	
	private $framework;
	private $themeStyle;
	
	
	/**
	 * Stores the path to the theme for this plugin.
	 * 
	 * Will set automatically by the CMS when the plugin is displayed in a slot
	 */
	
	private $themePath;
	private $frameworkPath;
	
	private $themePluginName;
	private $themePluginClassName;
	private $frameworkPluginName;
	private $frameworkPluginClassName;
		
	/**
	 * Stores the data on the framework for this plugin.
	 * It can be set using the cmsPluginApiLinkLoadFramework function and displayed using the framework() function
	 */
	private $frameworkData;
	
	
	/**
	 * A theme to use as a fallback
	 */
	
	private $defaultTheme;
	private $defaultFramework;
		
		
	/**
	 * Checks if the framework has been loaded yet
	 */
	private $frameworkLoaded = false;
	private $frameworkOutputted = false;
	
	
	
	private $errorMessage = false;
	
	
	

	/**
	 * Lets the CMS check the state of the forcePageReload option
	 */
	public final function beingEdited($canBe = false) {
		if ($canBe) {
			return $this->slotCanBeEdited;
		} else {
			return $this->slotBeingEdited;
		}
	}

	/**
	 * Lets the CMS check the state of the forcePageReload option
	 */
	public final function checkForcePageReloadVar() {
		return $this->cmsPluginApiLinkForcePageReloadVar;
	}

	public final function checkHeaderRedirectLocation() {
		return $this->cmsPluginApiLinkHeaderRedirectLocation;
	}

	/**
	 * Lets the CMS check the state of the forcePageReload option
	 */
	public final function checkShowInFloatingBoxVar() {
		return $this->showInFloatingBox;
	}
	
	public final function checkScrollToTopVar() {
		return $this->cmsPluginApiLinkScrollToTop;
	}
	
	/** 
	 * Lets a plugin add a JavaScript function to be run when the Plugin has loaded
	 */
	protected final function cmsPluginApiLinkCallScriptWhenLoaded($beforeAJAXReload, &$script) {
		if ($beforeAJAXReload && arrayKey($_REQUEST, 'method_call') == 'refreshPlugin') {
			$this->scriptsBefore[] = $script;
		} else {
			$this->scripts[] = $script;
		}
	}
	
	/** 
	 * Lets a plugin force a page reload, rather than allow their plugin to be reloaded via AJAX
	 */
	protected final function cmsPluginApiLinkForcePageReload($reload) {
		$this->cmsPluginApiLinkForcePageReloadVar = $reload;
	}
	
	protected final function cmsPluginApiLinkHeaderRedirect($location) {
		$this->cmsPluginApiLinkHeaderRedirectLocation = $location;
	}
	
	/** 
	 * A Plugin will be placed in a floating box when it reloads
	 */
	protected final function cmsPluginApiShowInFloatingBox($showInFloatingBox) {
		$this->showInFloatingBox = $showInFloatingBox;
	}

	protected final function cmsPluginApiScrollToTopOfSlot($scrollToTop) {
		$this->cmsPluginApiLinkScrollToTop = $scrollToTop;
	}
	
	/** 
	 * Get the value of a setting from the private $settings array
	 */
	protected final function cmsPluginApiLinkGetSetting($name) {
		return arrayKey($this->cmsPluginApiLinkSettings, $name);
	}
	
	/** 
	 * Get the value of the tabId variable
	 */
	public final function cmsPluginApiLinkGetTabId() {
		return $this->tabId;
	}
	
	/** 
	 * Get the Plugin Theme's directory
	 */
	public final function cmsPluginApiLinkGetThemeDir() {
		if ($this->themeStyle && !is_numeric($this->themeStyle)) {
			return $this->themePath;
		} else {
			return false;
		}
	}
	
	/** 
	 * Get the Plugin Framework's directory
	 */
	public final function cmsPluginApiLinkGetFrameworkDir() {
		if ($this->framework) {
			return $this->frameworkPath;
		} else {
			return false;
		}
	}
	
	/** 
	 * Get the Plugin Framework
	 */
	public final function cmsPluginApiLinkGetFrameworkName() {
		return $this->framework;
	}
	
	/** 
	 * Get the Plugin Theme's prefix, or false if there is no CSS file
	 */
	public final function cmsPluginApiLinkGetThemePrefix() {
		if ($this->themeStyle) {
			return $this->themePluginClassName. '__'. $this->themeStyle;
		} else {
			return false;
		}
	}
	
	/** 
	 * Get the Plugin Theme's info
	 */
	public final function cmsPluginApiLinkGetThemeParams(&$frameworkPluginName, &$frameworkPluginClassName, &$themePluginName) {
		if ($this->framework) {
			$frameworkPluginName = $this->frameworkPluginName;
			$frameworkPluginClassName = $this->frameworkPluginClassName;
			$themePluginName = $this->themePluginName;
			return true;
		} else {
			return false;
		}
	}
	
	//Allow the CMS to query the Plugin Id and/or Class name
	public final function cmsPluginApiLinkGetPluginClassName() {
		return $this->pluginClassName;
	}
	public final function cmsPluginApiLinkGetPluginId() {
		return $this->pluginId;
	}
	
	
	/**
	 * Is this plugin currently in some kind of "edit" mode for an admin?
	 * (For example, content areas switch to edit mode when the editor pops up.)
	 * 
	 * If so, the CMS will turn the usual red dotted line around it into a thick red line,
	 * to indicate that it is currently being edited.
	 * 
	 * Most plugins don't do this, but if yours does, you can set this
	 */
	protected final function cmsPluginApiLinkMarkSlotAsBeingEdited($beingEdited = true, $canBeEdited = true) {
		$this->slotBeingEdited = $beingEdited;
		$this->slotCanBeEdited = $canBeEdited;
	}
	
	/**
	 * Marks whether to display HTML tags around the plugin when using the showAlone() method
	 */
	protected final function cmsPluginApiLinkNotHTMLContent($notHTMLContent = true) {
		$this->notHTMLContent = $notHTMLContent;
	}
	
	private $cmsPluginApiLinkFirst = true;
	
	private final function cmsPluginApiLinkFirst() {
		if ($this->cmsPluginApiLinkFirst) {
			$this->cmsPluginApiLinkFirst = false;
			return 'first';
		} else {
			return '';
		}
	}
	
	private $cmsPluginApiLinkOddOrEven = 'even';
	
	private final function cmsPluginApiLinkOddOrEven($change = true) {
		if ($change) {
			$this->cmsPluginApiLinkOddOrEven = $this->cmsPluginApiLinkOddOrEven == 'odd'? 'even' : 'odd';
		}
		return $this->cmsPluginApiLinkOddOrEven;
	}
	
	protected final function cmsPluginApiLinkResetOddOrEven() {
		$this->cmsPluginApiLinkOddOrEven = 'odd';
	}
	
	/**
	 * Automatically called by the CMS to give plugins the GET requests
	 */
	public final function setGETVariables($input) {
		$this->getVariables = $input;
	}
	
	
	
	
	//Find any inheritances this Plugin uses, and get a few details
	private $cmsPluginApiLinkInheritances = array();
	private final function cmsPluginApiLinkFindInheritances($pluginClassName, $recurseLimit = 9) {
		
		$plugin =
			$pluginClassName == $this->pluginClassName?
				array('id' => $this->pluginId, 'name' => $this->pluginName, 'class_name' => $this->pluginClassName)
			  : getPluginDetails($pluginClassName, 'class');
		
		$this->cmsPluginApiLinkInheritances[$plugin['id']] = $plugin;
		
		if (!$recurseLimit) {
			return;
		}
		
		if ($inheritance = getPluginInheritance($pluginClassName)) {
			$this->cmsPluginApiLinkFindInheritances($inheritance, --$recurseLimit);
		}
		
	}
	
	
	//Attempt to find the path to a Theme or JavaScript file
	public final function cmsPluginApiLinkFindPath(&$pluginNameUsed, &$pluginClassNameUsed, $subdir, $file) {
		
		foreach($this->cmsPluginApiLinkInheritances as $plugin) {
			if (is_file(CMS_ROOT. ($path = 'plugins/'. $plugin['name']. '/'. $subdir. '/'). $file)) {
				$pluginNameUsed = $plugin['name'];
				$pluginClassNameUsed = $plugin['class_name'];
				return $path;
			}
		}
		
		return false;
	}
	
	/** 
	 * setInstance is called by the CMS for a plugin in a slot to tell the plugin where it is
	 */
	public final function setInstanceVariables($locationAndInstanceDetails, $nest = 0, $tab = 0, $errorMessage = false) {
		return require 'cmsincludes/visitor/cmsPluginApiLinkSetInstanceVariables.fun.php';
	}
	
	/** 
	 * setInstance is called by the CMS for a plugin in a slot to tell the plugin where it is
	 */
	public final function setInstance($locationAndInstanceDetails, $errorMessage = false, $nest = 0, $tab = 0) {
		return require 'cmsincludes/visitor/cmsPluginApiLinkSetInstance.fun.php';
	}
	
	//Allows the value of settings to be set or changed
	//public final function cmsPluginApiLinkOverwriteSetting($setting, &$value) {
		//$this->cmsPluginApiLinkSettings[$setting] = &$value;
	//}
	
	
	/**
	 * Automatically called by the CMS to give plugins the POST requests
	 */
	public final function setPOSTVariables($input) {
		$this->postVariables = $input;
	}
	
	/**
	 * This is the function that Templates call to display the plugin
	 * 
	 * Note: If there is no plugin in the slot, an instance of the api
	 * will be created in order to run these functions
	 */
	public final function show($includeAdminControlsIfInAdminMode = true, $showPlaceholderMethod = 'showSlot') {
		
		if ($includeAdminControlsIfInAdminMode) {
			$this->startIncludeAdminControls();
		}
		
		$this->startInner();
		
		//Display the plugin, if it has been set up.
		if ($this->instanceId) {
			$this->$showPlaceholderMethod();
		} else {
			echo ifNull($this->errorMessage, '<span class="tribiq_empty_slot_message">'. getPhrase('[Empty Slot]'). '</span>');
		}
		
		$this->endInner();
	}
	
	/**
	 * Display the admin controls for a slot.
	 * This is run before the template adds its own HTML
	 */
	public final function start($shownInVisitorMode) {
		//Put a div around the slot and the slot controls in admin mode
		//This lets us adjust the look of the slot and the slot controls using CSS
		if (checkPriv()) {
			echo '
				<!--[if lt IE 8]><div><![endif]--><span class="',
					$this->instanceId?
						'tribiq_slotWithContents'.
						($this->beingEdited()?
							' tribiq_slotWithContentsBeingEdited'
						:
							($this->beingEdited($canBe = true)?
								' tribiq_slotWithContentsCanBeEdited' :null
							)
						) . (
							$shownInVisitorMode? null: ' tribiq_slotNotShownInVisitorMode'
					) :
						'tribiq_slotWithNoContents',
				'" id="', $this->containerId, '-wrap">';
		}
	}
	
	/**
	 * Display the admin controls for a slot.
	 * This is run after the template adds its own HTML
	 */
	private final function startIncludeAdminControls() {
		if (checkPriv('_PRIV_VIEW_SLOT')) {
			return require 'cmsincludes/admin/cmsPluginApiLinkAdminControls.fun.php';
		}
	}
	
	//Put a div around the slot, so we can reload the contents
	private final function startInner() {
		echo '
					<div class="tribiq_slot ', $this->themePrefix(), '" id="', $this->containerId, '">';
	}
	
	/**
	 * Close the admin controls for a slot.
	 * This is run before the template closes its own HTML
	 */
	private final function endInner() {
		if (checkPriv()) {
			if ($this->instanceId && !$this->frameworkOutputted) {
				echo '
					<span class="tribiq_slot_padding">&nbsp;</span>';
			}
		}
		
		echo '
					</div>';
	}
	
	/**
	 * Close the admin controls for a slot.
	 * This is run after the template closes its own HTML
	 */
	public final function end() {
		//Display the HTML at the end of a slot when in admin mode
		if (checkPriv()) {
			echo '
				</span><!--[if lt IE 8]></div><![endif]-->';
		}
	}
	
	
	
	
	/**
	 * Add JavaScript function calls after the Plugin is on the page, if the plugin has requested it
	 */
	public final function addRequestedScripts() {
		if (!empty($this->scripts)) {
			echo '
				<script type="text/javascript">';
				
					foreach ($this->scripts as &$script) {
						echo '
					tribiq.callScript("', $this->pluginClassName, '", "', addslashes(json_encode($script)), '");';
					}
					
			echo '
				</script>';
		}
	}
	

	/**
	 * Lets the CMS check the JavaScript functions a Plugin wishes to be run
	 */
	public final function cmsPluginApiLinkCheckRequestedScripts(&$scripts, &$scriptsBefore) {
		$scripts = $this->scripts;
		$scriptsBefore = $this->scriptsBefore;
	}
	

		
	/**
	 * Load a framework from the theme directory, and split it up into sections
	 * 
	 * Note that while it is possible to extend a plugin and use its code, you cannot
	 * use themes from another plugin directory
	 */
	protected final function cmsPluginApiLinkLoadFramework() {
		return require 'cmsincludes/visitor/cmsPluginApiLinkLoadFramework.fun.php';
	}
	
		
	/**
	 * Check to see if a section in a framework exists.
	 * This can be called to check if a section exists in a theme before we attempt to display it
	 * 
	 * This is pretty much the limit of the control themes get over the execution of php code
	 */
	protected final function cmsPluginApiLinkCheckFrameworkSectionExists($section = 'Outer') {
		
		if (!$this->frameworkLoaded) {
			$this->cmsPluginApiLinkLoadFramework();
		}
		
		return !(!issetArrayKey($this->frameworkData, $section) || !is_array($this->frameworkData[$section]));
	}
	
	public final function cmsPluginApiLinkGetSlotLevel() {
		return $this->slotLevel == 1? getPhrase('this Content Item') : ($this->slotLevel == 2? getPhrase('this Template') : getPhrase('this Template Family'));
	}
	
	
	/**
	 * Please see the comments on the framework() method in the plugin base class for a description
	 */
	protected final function cmsPluginApiLinkFramework(
								$section = 'Outer', $mergeFields = array(),
								$allowedSubSections = array(), $subSectionDepthLimit = 5,
								$half = false, $halfwayPoint = true
							 ) {
		return require 'cmsincludes/visitor/cmsPluginApiLinkFramework.fun.php';
	}
	
	private final function cmsPluginApiLinkFrameworkField(&$mergeFieldsRow, $type, $attributes, &$value, $readonly, $i = false, $saveVal = false, $dispVal = false) {
		return require 'cmsincludes/visitor/cmsPluginApiLinkFrameworkField.fun.php';
	}
	
	private final function cmsPluginApiLinkFrameworkLOV($type, &$attributes, &$lov) {
		require 'cmsincludes/visitor/cmsPluginApiLinkFrameworkLOV.fun.php';
	}
	
	protected final function cmsPluginApiLinkListFrameworkFields($section, &$fields, &$allowedSubSections = array(), $limit = 5) {
		
		if (!$this->frameworkLoaded) {
			$this->cmsPluginApiLinkLoadFramework();
		}
		
		if (!--$limit || !$this->checkFrameworkSectionExists($section)) {
			return;
		}
		
		//Get all of the fields in a sub section, checking sub sections too
		foreach ($this->frameworkData[$section] as &$sectionContents) {
			foreach ($sectionContents as $type => &$thing) {
				if ($type == CMS_FRAMEWORK_SECTION) {
					if (issetArrayKey($allowedSubSections, $thing)) {
						$this->cmsPluginApiLinkListFrameworkFields($thing, $fields, $allowedSubSections, $limit);
					}
				}
			}
		}
		
		if (is_array($this->fieldInfo[$section])) {
			foreach ($this->fieldInfo[$section] as $name => &$field) {
				$fields[$name] = $field;
			}
		}
	}
}