<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;

//Keep an eye on whether this method has already run, and don't let it be called twice
if ($this->frameworkLoaded) {
	return true;
}

$errors = array();
$this->fieldInfo = array();
$path = CMS_ROOT. $this->frameworkPath;

//Check to see if the framework html file exists, and start loading it if it does
if (isset($this->cmsPluginApiLinkSettings['%custom_framework%']) || is_file($file = $path. 'framework.html')) {
	
	//Load the framework from the file (note there is a way to override this, and pass in a framework from any source...)
		//Load the framework into memory, and split it up into pieces using special comments
	if (isset($this->cmsPluginApiLinkSettings['%custom_framework%'])) {
		$themeThings = preg_split('@\[(/?)cms:(\w+)(.*?)(/?)\]@', $this->cmsPluginApiLinkSettings['%custom_framework%'], -1,  PREG_SPLIT_DELIM_CAPTURE);
	} else {
		$themeThings = preg_split('@\[(/?)cms:(\w+)(.*?)(/?)\]@', file_get_contents($file), -1,  PREG_SPLIT_DELIM_CAPTURE);
	}
	
	
	//We need to be able to tell the difference between sections (which is opened and closed) and mergeFields that close themselves
	//Make a list of things that get closed, so we can tell the two apart
	$this->frameworkData = array('Outer' => array());
	
	$openThings = array('Outer');
	$count = 0;
	$c = count($themeThings) - 1;
	for ($i=0; $i < $c; $i += 5) {
		//Add any raw HTML directly to the output
		$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_HTML => $themeThings[$i]);
		
		//Is this a close-tag for a section?
		if ($themeThings[$i+1] && $themeThings[$i+2] != 'condition') {
			if ($themeThings[$i+4]) {
				$errors['open-close-tags'] = 'Tags cannot be both a close tag and a self closing tag.';
			
			//Ignore anything closing that isn't a section
			} elseif ($themeThings[$i+2] != 'section') {
				continue;
			
			//Otherwise try to close the current section
			} elseif (--$count < 0) {
				$errors['close-tags'] = 'Sections are unbalanced. More were closed than opened.';
			}
		
		} else {
			$attributes = array();
			$details = preg_split('@\s*(\w*)=([\'"]?)(.*?)\2\s*@', $themeThings[$i+3], -1,  PREG_SPLIT_DELIM_CAPTURE);
			$cd = count($details) - 1;
			for ($j=0; $j < $cd; $j += 4) {
				$attributes[$details[$j+1]] = html_entity_decode($details[$j+3], ENT_COMPAT, 'UTF-8');
			}
			
			//Is this an opening of a section?
			if ($themeThings[$i+2] == 'section' && issetArrayKey($attributes, 'name')) {
				
				if ($themeThings[$i+4]) {
					//Allow self closing sections as placeholders
					$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_SECTION => $attributes['name']);
				
				} elseif (is_array(arrayKey($this->fieldInfo, $attributes['name']))) {
					$errors['section ' + $attributes['name']] = 'Section '. $attributes['name']. ' was defined twice.';
					
				} else {
					$this->fieldInfo[$attributes['name']] = array();
					
					//Note that the section we were currently looking in contains this new section
					$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_SECTION => $attributes['name']);
					
					//Add this section into the list of currently open sections, so everything we parse will fall into it
					$this->frameworkData[$openThings[++$count] = $attributes['name']] = array();
				}
			
			//Is this a phrase?
			} elseif ($themeThings[$i+2] == 'phrase' && issetArrayKey($attributes, 'code')) {
				$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_PHRASE => $attributes['code']);
			
			//Is this is a named mergeField?
			} elseif ($themeThings[$i+2] == 'merge' && issetArrayKey($attributes, 'name')) {
				$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_MERGE => $attributes['name']);
			
			//Is this is an include?
			} elseif ($themeThings[$i+2] == 'include' && issetArrayKey($attributes, 'plugin') && issetArrayKey($attributes, 'method')) {
				$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_INCLUDE => $attributes);
			
			//Is this is a condition?
			} elseif ($themeThings[$i+2] == 'condition') {
				$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_CONDITION => $attributes);
			
			//Is this an input?
			} elseif ($themeThings[$i+2] == 'input' && issetArrayKey($attributes, 'name')) {
				$this->frameworkData[$openThings[$count]][] = array(CMS_FRAMEWORK_INPUT => $attributes['name']);
				$this->fieldInfo[$openThings[$count]][$attributes['name']] = $attributes;
				
				//For date pickers - if the client does not have JavaScript enabled, three select lists are used as a fallback
				if ($attributes['type'] == 'date' && isset($_POST['cID'])) {
					if (issetArrayKey($this->postVariables, $attributes['name']. '__1')
					 && issetArrayKey($this->postVariables, $attributes['name']. '__2')
					 && issetArrayKey($this->postVariables, $attributes['name']. '__3')) {
						$this->postVariables[$attributes['name']] = 
							$this->postVariables[$attributes['name']. '__3']. '-'.
							$this->postVariables[$attributes['name']. '__2']. '-'.
							$this->postVariables[$attributes['name']. '__1'];
						
						unset($this->postVariables[$attributes['name']. '__1']);
						unset($this->postVariables[$attributes['name']. '__2']);
						unset($this->postVariables[$attributes['name']. '__3']);
					}
				
				//For file uploads - if there has been a submission, move it to the tmp_downloads directory and
				//place the path in the postVariables.
				} elseif ($attributes['type'] == 'file' && isset($_POST['cID'])) {
					if ($_FILES[$attributes['name']] && is_uploaded_file($_FILES[$attributes['name']]['tmp_name'])) {
						require_once CMS_ROOT. 'cmsincludes/common_contenttype.inc.php';
						cleanDownloads();
						
						$randomDir = 'tmp_downloads/'. createRandomDir('15', CMS_ROOT. 'tmp_downloads/'). '/';
						$newName = $randomDir. preg_replace('/\.\./', '.', preg_replace('/[^\w\.-]/', '', $_FILES[$attributes['name']]['name']));
						
						//Change the extension
						$newName = $newName. '.upload';
						
						if (move_uploaded_file($_FILES[$attributes['name']]['tmp_name'], CMS_ROOT. $newName)) {
							$this->postVariables[$attributes['name']] = $newName;
						}
					}
					
					//Stop the user trying to trick the CMS into submitting a different file in a different location
					if (strpos($this->postVariables[$attributes['name']], '..') !== false
					 || !preg_match('@^tmp_downloads/\w+/[\w\.-]+\.upload$@', $this->postVariables[$attributes['name']])) {
						unset($this->postVariables[$attributes['name']]);
					}
					
					unset($this->getVariables[$attributes['name']]);
				
				
				//If this is a post submission, check if a toggle type field has been changed
				} elseif ($attributes['type'] == 'toggle' && isset($_POST['cID']) && isset($this->postVariables[$attributes['name']. '__n'])) {
					//Get the total number of toggles, and look at each toggle in turn
					for ($k = 1; $k <= (int) $this->postVariables[$attributes['name']. '__n']; ++$k) {
						//If that toggle was just pressed, then we need to either change the field to that value,
						//or unset the field if it was already that value
						if (isset($this->postVariables[$attributes['name']. '__'. $k])) {
							//Look up the list of possible values
							$lov = false;
							$this->cmsPluginApiLinkFrameworkLOV($attributes['type'], $attributes, $lov);
							
							//Loop through, looking for the value that matches this toggle
							if (is_array($lov)) {
								$l = 0;
								foreach ($lov as $saveVal => &$dispVal) {
									if (++$l == $k) {
										//Toggle the toggle
										if ($this->postVariables[$attributes['name']] == $saveVal) {
											$this->postVariables[$attributes['name']] = '';
										} else {
											$this->postVariables[$attributes['name']] = $saveVal;
										}
										
										unset($this->postVariables[$attributes['name']. '__'. $k]);
										break 2;
									}
								}
							}

						}
					}
					
					unset($this->postVariables[$attributes['name']. '__n']);
				}
			}
			
			unset($attributes);
		}
	}
	
	//Add the last HTML
	$this->frameworkData['Outer'][] = array(CMS_FRAMEWORK_HTML => $themeThings[$c]);
	
	//Check that everything that was opened was closed
	if ($count != 0) {
		$errors[] = 'This template has a mismatch between the number of sections that were opened and closed.';
	}
	
	unset($openThings);
	unset($themeThings);
	
} else {
	$errors[] = 'The file &ldquo;'. htmlspecialchars($path. 'framework.html'). '&rdquo; does not exist.';
}

$this->frameworkLoaded = true;

if (!empty($errors)) {
	$this->frameworkData = false;
	print_r($errors);
	return $errors;
} else {
	return true;
}
?>