<?php

/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */



function quickValidateWelcomePage(&$values, &$rowClasses, &$snippets) {
	if (post('tab') == 4 || post('tab') == 'change_password') {
		$strength = passwordValuesToStrengths(checkPasswordStrength($values['password'], true));
		$snippets['password_strength'] = '<div class="password_'. $strength. '"><span>'. getPhrase($strength). '</span></div>';
	
	} elseif (post('tab') == 5) {
		if (is_file($values['path'])) {
			if (substr($values['path'], -7) != '.sql.gz' && substr($values['path'], -4) != '.sql') {
				$snippets['path_status'] = getPhrase('Please select a backup file.');
			} elseif (is_readable($values['path'])) {
				$snippets['path_status'] = getPhrase('File exists.');
			} else {
				$snippets['path_status'] = getPhrase('File exists but cannot be read.');
			}
		} elseif (is_dir($values['path'])) {
			$snippets['path_status'] = getPhrase('Directory exists, please enter a filename.');
		} else {
			$snippets['path_status'] = getPhrase('No file or directory with that name.');
		}
	
	} elseif (post('tab') == 0) {
		
		if (isset($values['backup_dir'])) {
			$rowClasses['dir_1'] = 'sub_section_invalid';
			if (!$values['backup_dir']) {
				$rowClasses['backup_dir_status'] = 'sub_invalid';
				$snippets['backup_dir_status'] = getPhrase('Please enter a directory.');
			
			} elseif (!is_dir($values['backup_dir'])) {
				$rowClasses['backup_dir_status'] = 'sub_invalid';
				$snippets['backup_dir_status'] = getPhrase('This directory does not exist.');
			
			} elseif (!is_writable($values['backup_dir'])) {
				$rowClasses['backup_dir_status'] = 'sub_invalid';
				$snippets['backup_dir_status'] = getPhrase('This directory is not writable.');
			
			} elseif (realpath($values['backup_dir']) == realpath(CMS_ROOT)) {
				$rowClasses['backup_dir_status'] = 'sub_invalid';
				$snippets['backup_dir_status'] = getPhrase('The CMS is installed in this directory. Please choose a different directory.');
			
			} else {
				$rowClasses['dir_1'] = 'sub_section_valid';
				$rowClasses['backup_dir_status'] = 'sub_valid';
				$snippets['backup_dir_status'] = getPhrase('This directory exists and is writable.');
			}
		}
		
		
		if (isset($values['docstore_dir'])) {
			$rowClasses['dir_2'] = 'sub_section_invalid';
			if (!$values['docstore_dir']) {
				$rowClasses['docstore_dir_status'] = 'sub_invalid';
				$snippets['docstore_dir_status'] = getPhrase('Please enter a directory.');
			
			} elseif (!is_dir($values['docstore_dir'])) {
				$rowClasses['docstore_dir_status'] = 'sub_invalid';
				$snippets['docstore_dir_status'] = getPhrase('This directory does not exist.');
			
			} elseif (!is_writable($values['docstore_dir'])) {
				$rowClasses['docstore_dir_status'] = 'sub_invalid';
				$snippets['docstore_dir_status'] = getPhrase('This directory is not writable.');
			
			} elseif (realpath($values['docstore_dir']) == realpath(CMS_ROOT)) {
				$rowClasses['docstore_dir_status'] = 'sub_invalid';
				$snippets['docstore_dir_status'] = getPhrase('The CMS is installed in this directory. Please choose a different directory.');
			
			} else {
				$rowClasses['dir_2'] = 'sub_section_valid';
				$rowClasses['docstore_dir_status'] = 'sub_valid';
				$snippets['docstore_dir_status'] = getPhrase('This directory exists and is writable.');
			}
		}
		
		
		if (isset($values['dropbox_dir'])) {
			$rowClasses['dir_3'] = 'sub_section_invalid';
			if (!$values['dropbox_dir']) {
				$rowClasses['dropbox_dir_status'] = 'sub_invalid';
				$snippets['dropbox_dir_status'] = getPhrase('Please enter a directory.');
			
			} elseif (!is_dir($values['dropbox_dir'])) {
				$rowClasses['dropbox_dir_status'] = 'sub_invalid';
				$snippets['dropbox_dir_status'] = getPhrase('This directory does not exist.');
			
			} elseif (!is_writable($values['dropbox_dir'])) {
				$rowClasses['dropbox_dir_status'] = 'sub_invalid';
				$snippets['dropbox_dir_status'] = getPhrase('This directory is not writable.');
			
			} elseif (realpath($values['dropbox_dir']) == realpath(CMS_ROOT)) {
				$rowClasses['dropbox_dir_status'] = 'sub_invalid';
				$snippets['dropbox_dir_status'] = getPhrase('The CMS is installed in this directory. Please choose a different directory.');
			
			} else {
				$rowClasses['dir_3'] = 'sub_section_valid';
				$rowClasses['dropbox_dir_status'] = 'sub_valid';
				$snippets['dropbox_dir_status'] = getPhrase('This directory exists and is writable.');
			}
		}
	}

}




//This file includes common functionality for running SQL scripts

function runSQL($prefix = false, $file, &$error, $patterns = false, $replacements = false) {
	$error = false;
	
	//Attempt to work out the location of the installer scripts, if not provided
	if (!$prefix) {
		$prefix = CMS_ROOT. 'admin/db_install/';
	}
	
	if (!file_exists($prefix. $file)) {
		$error = 'SQL Template File '. $file. ' does not exist.';
		return false;
	}
	
	//Build up a list of pattern replacements
	if (!$patterns) {
		//If no patterns have been set, go in with a few default patterns. Note I am assuming that the CMS
		//is running here...
		getLatestRevisionNumber();
		$from = array("\r", '[[DB_NAME_PREFIX]]',	'[[DB_NAME_PREFIX_GLOBAL]]',	'[[LATEST_REVISION_NO]]',	'[[INSTALLER_REVISION_NO]]');
		$to =	array('',	DB_NAME_PREFIX,			DB_NAME_PREFIX_GLOBAL,			LATEST_REVISION_NO,			INSTALLER_REVISION_NO);
	} else {
		$from = array("\r");
		$to = array('');
		foreach($patterns as $pattern => $replacement) {
			
			//Accept $patterns and $replacements in two different arrays with numeric keys
			if (is_array($replacements)) {
				$pattern = $replacement;
				$replacement = $replacements[$pattern];
			}
			
			$from[] = '[['. $pattern. ']]';
			if ($pattern == 'DB_NAME_PREFIX' || $pattern == 'DB_NAME_PREFIX_GLOBAL') {
				$to[] = $replacement;
			} else {
				$to[] = mysql_real_escape_string($replacement);
			}
		}
	}
	
	//Get the contents of the script, do the replacements, and split up into statements
	$sqls = explode(";\n", str_replace($from, $to, file_get_contents($prefix. $file)));


	//Get the number of sql statements, check if the last one is empty and exclude it if so
	$count = count($sqls);
	trim($sqls[$count-1])? null: --$count;
	
	//Loop through and execute each statement
	for ($i = 0; $i < $count; ++$i) {
		$query = $sqls[$i];
		
		if (!$result = mysql_query($query)) {
			$errno = mysql_errno();
			$error = mysql_error();
			$error = '(Error '. $errno. ': '. $error. "). \n\n". $query. "\nFile: ". $file;
			return false;
		}
	}
	
	return true;
}


function readSampleConfigFile($file, $patterns) {
	$searches = $replaces = array();
	foreach ($patterns as $pattern => $value) {
		$searches[] = '[['. $pattern. ']]';
		$replaces[] = $value;
	}
	
	return str_replace($searches, $replaces, file_get_contents(CMS_ROOT. 'siteconfigs/'. $file. '-sample.php'));
}

function checkConfigFileExists($file) {
	$filesize = @filesize(CMS_ROOT. 'siteconfigs/global.php');
	return $filesize && $filesize >= 20;
}

function compareVersionNumber($actual, $required) {
	return ('_'. str_replace('.', '00', (string) $actual)) >= ('_'. str_replace('.', '00', (string) $required));
}


function installerReportError() {
	return "\n". getPhrase('(Error [[errno]]: [[error]])', array('errno' => mysql_errno(), 'error' => mysql_error()));
}


function suggestDir($dir) {
	if (preg_match ('/\//', $_SERVER['DOCUMENT_ROOT'])) {
		/* Forward slashes, Linux */
		$docroot_arr = explode ("/", $_SERVER['DOCUMENT_ROOT']);
		array_pop ($docroot_arr);
		$suggestedDir = implode ("/", $docroot_arr) . "/". $dir;
	} else {
		/* Backslashes, Windows */
		$docroot_arr = explode ("\\", $_SERVER['DOCUMENT_ROOT']);
		array_pop ($docroot_arr);
		$suggestedDir = implode ("\\", $docroot_arr) . "\\". $dir;
	}
	return $suggestedDir;
}


function getDiamondPath() {
	foreach (array('enterprise', 'probusiness', 'pro') as $edition) {
		if (file_exists(CMS_ROOT. ($path = 'plugins/tribiq_cms_core_'. $edition. '_v1/storekeeper/diamond-'. $edition. '.gif'))) {
			return $path;
		}
	}
	return 'plugins/tribiq_cms_core_v1/storekeeper/diamond.gif';
}


function installerAJAX(&$tags, &$box, &$task, $installStatus, &$freshInstall) {
	$merge = array();
	$merge['SUBDIRECTORY'] = SUBDIRECTORY;
	
	//Validation for Step 1: Check if the Admin has accepted the license
	if (!$licenseAccepted = issetArrayKey($box, 'tabs', 1, 'fields', 'i_agree', '_value')) {
		$box['tabs'][1]['fields']['fresh_install']['disabled'] = 'disabled';
		$box['tabs'][1]['fields']['restore']['disabled'] = 'disabled';
	} else {
		unset($box['tabs'][1]['fields']['fresh_install']['disabled']);
		unset($box['tabs'][1]['fields']['restore']['disabled']);
	}
	
	if (issetArrayKey($box, 'tabs', 1, 'fields', 'fresh_install', 'pressed')) {
		$task = 'install';
	} elseif (issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed')) {
		$task = 'restore';
	}
	
	//Validation for Step 2: Check if the server meets the requirements
	if ($box['tab'] > 1 || issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed') || issetArrayKey($box, 'tabs', 1, 'fields', 'fresh_install', 'pressed')) {
		//Get the server phpinfo
		//(Note this section of code may not work properly if compression has been enabled)
		$search = array();
		$replace = array();
		
		$search[] = '~';
		$replace[] = '--';
		
		$search[] = '`';
		$replace[] = '---';
		
		$search[] = '<h2';
		$replace[] = '~<h2';
		
		$search[] = '</h2>';
		$replace[] = '</h2>~';
		
		$search[] = '<td';
		$replace[] = '`<td';
		
		ob_start();
		phpinfo();
		$info = strip_tags(str_replace($search, $replace, ob_get_contents()));
		ob_end_clean();
		
		$phpinfo = array();
		$sectionName = 'PHP';
		foreach (explode('~', $info) as $i => $section) {
			if ($i % 2) {
				$phpinfo[$sectionName = $section] = array();
			} else {
				foreach (explode("\n", $section) as $line) {
					foreach (explode('`', $line) as $j => $cell) {
						if ($j == 1) {
							$setting = trim($cell);
						} else if ($j == 2) {
							$phpinfo[$sectionName][$setting] = trim($cell);
						}
					}
				}
			}
		}
		unset($info);
		unset($search);
		unset($replace);
		
		
		$phpRequirementsMet = false;
		$phpVersion = phpversion();
		$box['tabs'][2]['fields']['php_1']['post_field_html'] =
			getPhrase(' (you have version [[version]])', array('version' => htmlspecialchars($phpVersion)));
		
		if (!compareVersionNumber($phpVersion, '5.2.0')) {
			$box['tabs'][2]['fields']['php_1']['row_class'] = 'invalid';
		
		} else {
			$box['tabs'][2]['fields']['php_1']['row_class'] = 'valid';
			$phpRequirementsMet = true;
		}
		
		
		$mysqlRequirementsMet = false;
		if (!$mysqlVersion = arrayKey($phpinfo, 'mysql', 'Client API version')) {
			$box['tabs'][2]['fields']['mysql_1']['row_class'] = 'invalid';
			$box['tabs'][2]['fields']['mysql_2']['row_class'] = 'invalid';
			$box['tabs'][2]['fields']['mysql_2']['post_field_html'] = '';
		
		} else {
			$box['tabs'][2]['fields']['mysql_1']['row_class'] = 'valid';
			$box['tabs'][2]['fields']['mysql_2']['post_field_html'] =
				getPhrase(' (you have version [[version]])', array('version' => htmlspecialchars($mysqlVersion)));
			
			if (!compareVersionNumber($mysqlVersion, '5.0.0')) {
				$box['tabs'][2]['fields']['mysql_2']['row_class'] = 'invalid';
			} else {
				$box['tabs'][2]['fields']['mysql_2']['row_class'] = 'valid';
				$mysqlRequirementsMet = true;
			}
		}
		
		
		$mbRequirementsMet = false;
		if (!extension_loaded('mbstring')) {
			$box['tabs'][2]['fields']['mb_1']['row_class'] = 'invalid';
		
		} else {
			$box['tabs'][2]['fields']['mb_1']['row_class'] = 'valid';
			$mbRequirementsMet = true;
		}
		
		
		$gdRequirementsMet = true;
		if (arrayKey($phpinfo, 'gd', 'GD Support') != 'enabled') {
			$gdRequirementsMet = false;
			$box['tabs'][2]['fields']['gd_1']['row_class'] = 'invalid';
			$box['tabs'][2]['fields']['gd_2']['row_class'] = 'invalid';
			$box['tabs'][2]['fields']['gd_3']['row_class'] = 'invalid';
			$box['tabs'][2]['fields']['gd_4']['row_class'] = 'invalid';
		
		} else {
			$box['tabs'][2]['fields']['gd_1']['row_class'] = 'valid';
			
			if (arrayKey($phpinfo, 'gd', 'GIF Read Support') != 'enabled') {
				$gdRequirementsMet = false;
				$box['tabs'][2]['fields']['gd_2']['row_class'] = 'invalid';
			} else {
				$box['tabs'][2]['fields']['gd_2']['row_class'] = 'valid';
			}
			
			if (arrayKey($phpinfo, 'gd', 'JPG Support') != 'enabled' && arrayKey($phpinfo, 'gd', 'JPEG Support') != 'enabled') {
				$gdRequirementsMet = false;
				$box['tabs'][2]['fields']['gd_3']['row_class'] = 'invalid';
			} else {
				$box['tabs'][2]['fields']['gd_3']['row_class'] = 'valid';
			}
			
			if (arrayKey($phpinfo, 'gd', 'PNG Support') != 'enabled') {
				$gdRequirementsMet = false;
				$box['tabs'][2]['fields']['gd_4']['row_class'] = 'invalid';
			} else {
				$box['tabs'][2]['fields']['gd_4']['row_class'] = 'valid';
			}
		}
	}
	
	
	//Validation for Step 3: Validate the Database Connection
	if ($box['tab'] > 3 || ($box['tab'] == 3 && issetArrayKey($box, 'tabs', 3, 'fields', 'next', 'pressed'))) {
		$box['tabs'][3]['errors'] = array();
		
		$merge['DBHOST'] = $merge['DBHOST_GLOBAL'] = $box['tabs'][3]['fields']['host']['_value'];
		$merge['DBNAME'] = $merge['DBNAME_GLOBAL'] = $box['tabs'][3]['fields']['name']['_value'];
		$merge['DBUSER'] = $merge['DBUSER_GLOBAL'] = $box['tabs'][3]['fields']['user']['_value'];
		$merge['DBPASS'] = $merge['DBPASS_GLOBAL'] = $box['tabs'][3]['fields']['password']['_value'];
		$merge['DB_NAME_PREFIX'] = $merge['DB_NAME_PREFIX_GLOBAL'] = $box['tabs'][3]['fields']['prefix']['_value'];
		
		if (!$merge['DBHOST']) {
			$box['tabs'][3]['errors'][] = getPhrase('Please enter a Hostname.');
		}
		
		if (!$merge['DBNAME']) {
			$box['tabs'][3]['errors'][] = getPhrase('Please enter a Database Name.');
		} elseif (preg_match('/[^a-zA-Z0-9_-]/', $merge['DBNAME'])) {
			$box['tabs'][3]['errors'][] = getPhrase('The Database Name should only contain [a-z, A-Z, 0-9, _ and -].');
		}
		
		if (!$merge['DBUSER']) {
			$box['tabs'][3]['errors'][] = getPhrase('Please enter a Database Username.');
		} elseif (preg_match('/[^a-zA-Z0-9_-]/', $merge['DBUSER'])) {
			$box['tabs'][3]['errors'][] = getPhrase('The Database Username should only contain [a-z, A-Z, 0-9, _ and -].');
		}
		
		if (preg_match('/[^a-zA-Z0-9_-]/', $merge['DB_NAME_PREFIX'])) {
			$box['tabs'][3]['errors'][] = getPhrase('The Table Prefix should only contain [a-z, A-Z, 0-9, _ and -].');
		}
		
		if (empty($box['tabs'][3]['errors'])) {
			if (!@mysql_connect($merge['DBHOST'], $merge['DBUSER'], $merge['DBPASS'])) {
				$box['tabs'][3]['errors'][] = 
					getPhrase('The Database User and/or password are invalid.');
			
			} elseif (!(@mysql_select_db($merge['DBNAME']))
				   || !($result = @mysql_query("SELECT VERSION()"))
				   || !($version = @mysql_fetch_row($result))
				   || !($result = @mysql_query("SHOW TABLES"))) {
				$box['tabs'][3]['errors'][] = 
					getPhrase('The Database [[database]] does not exist, or you do not have access rights.', array('database' => $merge['DBNAME']));
			
			} elseif (!compareVersionNumber($version[0], '5.0.0')) {
				$box['tabs'][3]['errors'][] = 
					getPhrase('Sorry, your MySQL server is "[[version]]". Version 5.0 or later is required.', array('version' => $version[0]));
			
			} elseif (!(@mysql_query("CREATE TABLE IF NOT EXISTS `tribiq_priv_test` (`id` TINYINT(1) NOT NULL )"))
				   || !(@mysql_query("DROP TABLE `tribiq_priv_test`"))) {
				$box['tabs'][3]['errors'][] = 
					getPhrase('Cannot verify database privileges. Please ensure the MySQL User [[user]] has CREATE TABLE and DROP TABLE privileges. You may need to contact your MySQL administrator to have these privileges enabled.',
						array('user' => $merge['DBUSER'])).
					installerReportError();
			
			} else {
				while ($tables = mysql_fetch_row($result)) {
					if ($merge['DB_NAME_PREFIX'] == '' || substr($tables[0], 0, strlen($merge['DB_NAME_PREFIX'])) == $merge['DB_NAME_PREFIX']) {
						$box['tabs'][3]['errors'][] = getPhrase('There are already Tables in this Database that match your chosen Table Prefix. Please choose a different Table Prefix, or a different Database.');
						break;
					}
				}
			}
		}
	}
	
	
	if (!issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed')) {
		//Validation for Step 4: Validate new Admin's details
		if (($box['tab'] > 4 || ($box['tab'] == 4 && issetArrayKey($box, 'tabs', 4, 'fields', 'next', 'pressed')))) {
			$box['tabs'][4]['errors'] = array();
			
			if (!$merge['admin_first_name'] = $box['tabs'][4]['fields']['first_name']['_value']) {
				$box['tabs'][4]['errors'][] = getPhrase('Please enter your First Name.');
			}
	
			if (!$merge['admin_last_name'] = $box['tabs'][4]['fields']['last_name']['_value']) {
				$box['tabs'][4]['errors'][] = getPhrase('Please enter your Last Name.');
			}
	
			if (!$merge['EMAIL_ADDRESS_GLOBAL_SUPPORT'] = $box['tabs'][4]['fields']['email']['_value']) {
				$box['tabs'][4]['errors'][] = getPhrase('Please enter your Email Address.');
			
			} elseif (!validateEmailAddress($merge['EMAIL_ADDRESS_GLOBAL_SUPPORT'])) {
				$box['tabs'][4]['errors'][] = getPhrase('Please enter a valid Email Address.');
			}
	
			if (!$merge['USERNAME'] = $box['tabs'][4]['fields']['username']['_value']) {
				$box['tabs'][4]['errors'][] = getPhrase('Please choose a Username.');
			}
	
			if (!$merge['PASSWORD'] = $box['tabs'][4]['fields']['password']['_value']) {
				$box['tabs'][4]['errors'][] = getPhrase('Please choose a Password.');
			
			} elseif ($merge['PASSWORD'] != $box['tabs'][4]['fields']['re_password']['_value']) {
				$box['tabs'][4]['errors'][] = getPhrase('The password fields do not match.');
			}
		}
	} else {
		//Validation for Step 5: Check a backup file exists
		if (($box['tab'] > 5 || ($box['tab'] == 5 && issetArrayKey($box, 'tabs', 5, 'fields', 'next', 'pressed')))) {
			$box['tabs'][5]['errors'] = array();
			
			if (!is_file($box['tabs'][5]['fields']['path']['_value'])) {
				$box['tabs'][5]['errors'][] = getPhrase('Please enter a path to a file.');
			
			} elseif (substr($box['tabs'][5]['fields']['path']['_value'], -7) != '.sql.gz'
				   && substr($box['tabs'][5]['fields']['path']['_value'], -4) != '.sql') {
				$box['tabs'][5]['errors'][] = getPhrase('Please select a backup file.');
			
			} elseif (!is_readable($box['tabs'][5]['fields']['path']['_value'])) {
				$box['tabs'][5]['errors'][] = getPhrase('Please enter a path to a file with read-access set.');
			}
		}
	}
	
	
	//Validation for Step 6: Attempt to create (if requested) and then validate the siteconfig files
	if ($box['tab'] == 6 && (issetArrayKey($box, 'tabs', 6, 'fields', 'ive_done_it', 'pressed') || issetArrayKey($box, 'tabs', 6, 'fields', 'do_it_for_me', 'pressed'))) {
		$box['tabs'][6]['errors'] = array();
		
		if (issetArrayKey($box, 'tabs', 6, 'fields', 'do_it_for_me', 'pressed')) {
			$permErrors = false;
			if (!checkConfigFileExists('global')) {
				if (!file_exists(CMS_ROOT. 'siteconfigs/global.php')) {
					$box['tabs'][6]['errors'][] = getPhrase('Please create the file siteconfigs/global.php');
				
				} elseif (!@file_put_contents(CMS_ROOT. 'siteconfigs/global.php', readSampleConfigFile('global', $merge))) {
					$box['tabs'][6]['errors'][] = getPhrase('Could not write to file siteconfigs/global.php');
					$permErrors = true;
				}
			}
			
			if (!checkConfigFileExists('mytribiqsite')) {
				if (!file_exists(CMS_ROOT. 'siteconfigs/mytribiqsite.php')) {
					$box['tabs'][6]['errors'][] = getPhrase('Please create the file siteconfigs/mytribiqsite.php');
				
				} elseif (!@file_put_contents(CMS_ROOT. 'siteconfigs/mytribiqsite.php', readSampleConfigFile('mytribiqsite', $merge))) {
					$box['tabs'][6]['errors'][] = getPhrase('Could not write to file siteconfigs/mytribiqsite.php');
					$permErrors = true;
				}
			}
			
			if ($permErrors && !stristr(php_uname('s'), 'win')) {
				$box['tabs'][6]['errors'][] = getPhrase('To correct the file permissions: chmod 646 siteconfigs/*');
			}
		
		} else {
			if (!checkConfigFileExists('global')) {
				$box['tabs'][6]['errors'][] = getPhrase('Please create the file siteconfigs/global.php');
			}
			
			if (!checkConfigFileExists('mytribiqsite')) {
				$box['tabs'][6]['errors'][] = getPhrase('Please create the file siteconfigs/mytribiqsite.php');
			}
		}
		
		if (checkConfigFileExists('global')) {
			if (!@include_once CMS_ROOT. 'siteconfigs/global.php') {
				$box['tabs'][6]['errors'][] = getPhrase('There is a syntax error in siteconfigs/global.php');
			} else {
				foreach (array('DBHOST_GLOBAL', 'DBNAME_GLOBAL', 'DBUSER_GLOBAL', 'DBPASS_GLOBAL', 'DB_NAME_PREFIX_GLOBAL') as $constant) {
					if (!defined($constant) || constant($constant) !== $merge[$constant]) {
						$box['tabs'][6]['errors'][] = getPhrase('The constants in siteconfigs/global.php are not set as below.');
						break;
					}
				}
			}
		}
		
		if (checkConfigFileExists('mytribiqsite')) {
			if (!@include_once CMS_ROOT. 'siteconfigs/mytribiqsite.php') {
				$box['tabs'][6]['errors'][] = getPhrase('There is a syntax error in siteconfigs/mytribiqsite.php');
			} else {
				foreach (array('DBHOST', 'DBNAME', 'DBUSER', 'DBPASS', 'DB_NAME_PREFIX') as $constant) {
					if (!defined($constant) || constant($constant) !== $merge[$constant]) {
						$box['tabs'][6]['errors'][] = getPhrase('The constants in siteconfigs/mytribiqsite.php are not set as below.');
						break;
					}
				}
			}
		}
	}
	
	
	
	//Handle navigation
	switch ($box['tab']) {
		case 0:
			if (issetArrayKey($box, 'tabs', 0, 'fields', 'next', 'pressed')) {
				$box['tab'] = 1;
			}
			
			break;
		
		
		case 1:
			if (issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed') || issetArrayKey($box, 'tabs', 1, 'fields', 'fresh_install', 'pressed')) {
				$box['tab'] = 2;
			}
			
			break;
		
		
		case 2:
			if (issetArrayKey($box, 'tabs', 2, 'fields', 'previous', 'pressed')) {
				$box['tab'] = 1;
			}
			
			break;
		
		
		case 3:
			if (issetArrayKey($box, 'tabs', 3, 'fields', 'previous', 'pressed')) {
				$box['tab'] = 1;
			} elseif (issetArrayKey($box, 'tabs', 3, 'fields', 'next', 'pressed')) {
				if (issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed')) {
					$box['tab'] = 5;
				} else {
					$box['tab'] = 4;
				}
			}
			
			break;
		
		
		case 4:
			if (issetArrayKey($box, 'tabs', 4, 'fields', 'previous', 'pressed')) {
				$box['tab'] = 3;
			} elseif (issetArrayKey($box, 'tabs', 4, 'fields', 'next', 'pressed')) {
				$box['tab'] = 6;
			}
			
			break;
			
			
		case 5:
			if (issetArrayKey($box, 'tabs', 5, 'fields', 'previous', 'pressed')) {
				$box['tab'] = 3;
			} elseif (issetArrayKey($box, 'tabs', 5, 'fields', 'next', 'pressed')) {
				$box['tab'] = 6;
			}
			
			break;
		
		
		case 6:
			if (issetArrayKey($box, 'tabs', 6, 'fields', 'previous', 'pressed')) {
				if (issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed')) {
					$box['tab'] = 5;
				} else {
					$box['tab'] = 4;
				}
			} elseif (issetArrayKey($box, 'tabs', 6, 'fields', 'do_it_for_me', 'pressed') || issetArrayKey($box, 'tabs', 6, 'fields', 'ive_done_it', 'pressed')) {
				$box['tab'] = 7;
			}
			
			break;
			
			
		case 7:
			if (issetArrayKey($box, 'tabs', 7, 'fields', 'previous', 'pressed')) {
				$box['tab'] = 6;
			}
			
			break;
	}
	
	
	//Don't let the Admin proceed from Step 1 without accepting the license
	if ($box['tab'] > 0 && !$licenseAccepted) {
		$box['tab'] = 1;
	}
	
	if ($box['tab'] > 1) {
		//Did the Server meets the requirements?
		if ($phpRequirementsMet && $mysqlRequirementsMet && $mbRequirementsMet && $gdRequirementsMet) {
			//If so, show Step 3 after Step 1, skipping the Requirements section
			if ($box['tab'] == 2) {
				$box['tab'] = 3;
			}
		} else {
			//If not, don't let the Admin proceed from Step 2
			$box['tab'] = 2;
		}
	}
	
	//Don't let the Admin proceed from Step 3 without a valid database connection
	if ($box['tab'] > 3 && !empty($box['tabs'][3]['errors'])) {
		$box['tab'] = 3;
	}
	
	if (!issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed')) {
		//Don't let the Admin proceed from Step 4 without entering their admin details
		if ($box['tab'] > 4 && !empty($box['tabs'][4]['errors'])) {
			$box['tab'] = 4;
		}
	} else {
		//Don't let the Admin proceed from Step 5 without selecting a backup
		if ($box['tab'] > 5 && !empty($box['tabs'][5]['errors'])) {
			$box['tab'] = 5;
		}
	}
	
	//Don't let the Admin proceed from Step 6 without setting the siteconfig up correctly
	if ($box['tab'] > 6 && !empty($box['tabs'][6]['errors'])) {
		$box['tab'] = 6;
	}
	
	
	//Display the current step
	switch ($box['tab']) {
		case 0:
			
			if ($installStatus == 0) {
				$box['tabs'][0]['fields']['reason']['snippet']['html'] = 
					getPhrase('Config files in the "[[dir]]" directory are empty.',
						array('dir' => '<code>siteconfigs</code>'));
			
			} elseif ($installStatus == 1) {
				$box['tabs'][0]['fields']['reason']['snippet']['html'] = 
					getPhrase('A database connection could not be established.');
			
			} elseif ($installStatus == 2) {
				$box['tabs'][0]['fields']['reason']['snippet']['html'] = 
					getPhrase('The database is empty.');
			}
			
			$box['tabs'][0]['fields']['why']['class'] = (
				$box['tabs'][0]['fields']['reason']['hidden'] =
					!issetArrayKey($box, 'tabs', 0, 'fields', 'why', 'pressed')
				) ?
					'closed'
				:	'opened';
			
			break;
		
		
		case 1:
			if (is_file(CMS_ROOT. 'license.txt')) {
				$box['tabs'][1]['fields']['license']['snippet']['url'] = 'license.txt';
			
			} elseif (!isset($box['tabs'][1]['fields']['license']['snippet']['html'])) {
				$box['tabs'][1]['fields']['license']['snippet']['url'] = 'admin/welcome_ajax.php?getLicense=1';
			}
			
			break;
		
		
		case 2:
			if (!$phpRequirementsMet) {
				$box['tabs'][2]['fields']['show_php']['pressed'] = true;
				$box['tabs'][2]['fields']['php']['row_class'] = 'section_invalid';
			} else {
				$box['tabs'][2]['fields']['php']['row_class'] = 'section_valid';
			}
			
			if (!$mysqlRequirementsMet) {
				$box['tabs'][2]['fields']['show_mysql']['pressed'] = true;
				$box['tabs'][2]['fields']['mysql']['row_class'] = 'section_invalid';
			} else {
				$box['tabs'][2]['fields']['mysql']['row_class'] = 'section_valid';
			}
			
			if (!$mbRequirementsMet) {
				$box['tabs'][2]['fields']['show_mb']['pressed'] = true;
				$box['tabs'][2]['fields']['mb']['row_class'] = 'section_invalid';
			} else {
				$box['tabs'][2]['fields']['mb']['row_class'] = 'section_valid';
			}
			
			if (!$gdRequirementsMet) {
				$box['tabs'][2]['fields']['show_gd']['pressed'] = true;
				$box['tabs'][2]['fields']['gd']['row_class'] = 'section_invalid';
			} else {
				$box['tabs'][2]['fields']['gd']['row_class'] = 'section_valid';
			}
			
			
			$box['tabs'][2]['fields']['show_php']['class'] = (
				$box['tabs'][2]['fields']['php_1']['hidden'] =
					!issetArrayKey($box['tabs'][2]['fields']['show_php'], 'pressed')
				) ?
					'closed'
				:	'opened';
			
			$box['tabs'][2]['fields']['show_mysql']['class'] = (
				$box['tabs'][2]['fields']['mysql_1']['hidden'] =
				$box['tabs'][2]['fields']['mysql_2']['hidden'] =
					!issetArrayKey($box['tabs'][2]['fields']['show_mysql'], 'pressed')
				) ?
					'closed'
				:	'opened';
			
			$box['tabs'][2]['fields']['show_mb']['class'] = (
				$box['tabs'][2]['fields']['mb_1']['hidden'] =
					!issetArrayKey($box['tabs'][2]['fields']['show_mb'], 'pressed')
				) ?
					'closed'
				:	'opened';
			
			$box['tabs'][2]['fields']['show_gd']['class'] = (
				$box['tabs'][2]['fields']['gd_1']['hidden'] =
				$box['tabs'][2]['fields']['gd_2']['hidden'] =
				$box['tabs'][2]['fields']['gd_3']['hidden'] =
				$box['tabs'][2]['fields']['gd_4']['hidden'] =
					!issetArrayKey($box['tabs'][2]['fields']['show_gd'], 'pressed')
				) ?
					'closed'
				:	'opened';
			
			break;
		
		
		case 4:
			$strength = passwordValuesToStrengths(checkPasswordStrength(arrayKey($box['tabs'][4]['fields']['password'], '_value'), true));
			$box['tabs'][4]['fields']['password_strength']['snippet']['html'] =
				'<div class="password_'. $strength. '"><span>'. getPhrase($strength). '</span></div>';
			
			
			break;
		
		
		case 5:
			$box['tabs'][5]['fields']['path']['value'] = CMS_ROOT;
			$box['tabs'][5]['fields']['path_status']['snippet']['html'] = '&nbsp;';
			
			
			break;
		
		
		case 6:
			$box['tabs'][6]['fields']['global']['pre_field_html'] =
				'<pre>'. CMS_ROOT. 'siteconfigs/global.php'. ':</pre>';
			$box['tabs'][6]['fields']['mytribiqsite']['pre_field_html'] =
				'<pre>'. CMS_ROOT. 'siteconfigs/mytribiqsite.php'. ':</pre>';
			$box['tabs'][6]['fields']['global']['value'] = readSampleConfigFile('global', $merge);
			$box['tabs'][6]['fields']['mytribiqsite']['value'] = readSampleConfigFile('mytribiqsite', $merge);
			
			
			break;
		
		
		case 7:
			$box['tabs'][7]['errors'] = array();
			
			$merge['SALT'] = randomString(8);
			$merge['SITENAME'] = 'mytribiqsite';
			$merge['URL'] = httpOrhttps(). $_SERVER['HTTP_HOST'];
			$merge['HTTP_HOST'] = $_SERVER['HTTP_HOST'];
			$merge['SUBDIRECTORY'] = SUBDIRECTORY;
			
			//Set the latest revision number
			$merge['LATEST_REVISION_NO'] = LATEST_REVISION_NO;
			$merge['INSTALLER_REVISION_NO'] = INSTALLER_REVISION_NO;
			
			
			//Install to the database
			mysql_query('SET NAMES "UTF8"');
			mysql_query("SET collation_connection='utf8_general_ci'");
			mysql_query("SET collation_server='utf8_general_ci'");
			mysql_query("SET character_set_client='utf8'");
			mysql_query("SET character_set_connection='utf8'");
			mysql_query("SET character_set_results='utf8'");
			mysql_query("SET character_set_server='utf8'");
			
			if (!defined('DB_NAME_PREFIX')) {
				define('DB_NAME_PREFIX', $merge['DB_NAME_PREFIX']);
			}
			
			if (!defined('DB_NAME_PREFIX_GLOBAL')) {
				define('DB_NAME_PREFIX_GLOBAL', $merge['DB_NAME_PREFIX_GLOBAL']);
			}
			
			
			//Try to restore the backup
			if (issetArrayKey($box, 'tabs', 1, 'fields', 'restore', 'pressed')) {
				$files = array('global-CREATE.sql', 'global-INSERT.sql');
				
				$error = false;
				foreach ($files as $file) {
					if (!runSQL('admin/db_install/', $file, $error, $merge)) {
						$box['tabs'][7]['errors'][] = $error;
						break;
					}
				}
				
				if (empty($box['tabs'][7]['errors'])) {
					$failures = array();
					if (!restoreDatabaseFromBackup(
							arrayKey($box['tabs'][5]['fields']['path'], '_value'),
							//Attempt to check whether gzip compression has not been used
							strtolower(substr(arrayKey($box['tabs'][5]['fields']['path'], '_value'), -3)) != '.gz',
							DB_NAME_PREFIX, $failures
					)) {
						foreach ($failures as $text) {
							$box['tabs'][7]['errors'][] = $text;
						}
					}
				}

			//Try to do the installation!
			} else {
				$files = array('local-CREATE.sql', 'local-admin-CREATE.sql',
								'local-INSERT.sql', 'local-admin-INSERT.sql',
								'local-BUILD.sql',
								'global-CREATE.sql', 'global-INSERT.sql');
				
				$error = false;
				foreach ($files as $file) {
					if (!runSQL('admin/db_install/', $file, $error, $merge)) {
						$box['tabs'][7]['errors'][] = $error;
						break;
					}
				}
				
				//Was the install successful?
				if (empty($box['tabs'][7]['errors'])) {
					$freshInstall = true;
					
					//Prepare email to the installing person
					$message = $tags['email_templates']['installed_cms']['body'];
					
					$to = $merge['EMAIL_ADDRESS_GLOBAL_SUPPORT'];
					$from = 'noreply@'. $_SERVER['HTTP_HOST'];
					$subject = $tags['email_templates']['installed_cms']['subject'];
					$headers = 'From: '. $tags['email_templates']['installed_cms']['from'].  '<'. $from. '>';
					
					foreach ($merge as $pattern => $replacement) {
						$message = str_replace('[['. $pattern. ']]', $replacement, $message);
					}
					
					@mail($to, $subject, $message, $headers);
				}
			}
			
			if (!empty($box['tabs'][7]['errors'])) {
				//Did something go wrong? Remove any tables that were created.
				runSQL('admin/db_install/', 'local-DROP.sql', $error, $merge);
				runSQL('admin/db_install/', 'local-admin-DROP.sql', $error, $merge);
				runSQL('admin/db_install/', 'global-DROP.sql', $error, $merge);
			} else {
				return true;
			}
			
			break;
	}
	
	return false;
}





function loginAJAX(&$tags, &$box, $getRequest) {
	$passwordReset = false;
	
	if ($box['tab'] == 'login' && issetArrayKey($box['tabs']['login']['fields']['previous'], 'pressed')) {
		$box['_location'] = redirectAdmin($getRequest);
		return false;
	
	//Go between the login and the forgot password screens
	} elseif ($box['tab'] == 'login' && issetArrayKey($box['tabs']['login']['fields']['forgot'], 'pressed')) {
		$box['tab'] = 'forgot';
	
	} elseif ($box['tab'] == 'forgot' && issetArrayKey($box['tabs']['forgot']['fields']['previous'], 'pressed')) {
		$box['tab'] = 'login';
	
	//Check a login attempt
	} elseif ($box['tab'] == 'login' && issetArrayKey($box['tabs']['login']['fields']['login'], 'pressed')) {
		$box['tabs']['login']['errors'] = array();
		
		if (!$box['tabs']['login']['fields']['username']['_value']) {
			$box['tabs']['login']['errors'][] = getPhrase('Please enter your Username.');
		}
		
		if (!$box['tabs']['login']['fields']['password']['_value']) {
			$box['tabs']['login']['errors'][] = getPhrase('Please enter your Password.');
		}
		
		if (empty($box['tabs']['login']['errors'])) {
			$details = array();
			if (checkPasswordAdmin($box['tabs']['login']['fields']['username']['_value'], $details, $box['tabs']['login']['fields']['password']['_value'], $checkViaEmail = false)) {
				
				if ($box['tabs']['login']['fields']['remember_me']['_value']) {
					setcookie('COOKIE_LAST_ADMIN_USER', $box['tabs']['login']['fields']['username']['_value'], time()+8640000, '/');
				} else {
					setcookie('COOKIE_LAST_ADMIN_USER', '', time()-3600, '/');
				}
				
				if ($details['type'] == 'local') {
					setAdminSession($details['id'], 'local');
					
					//Keep track of currently logged in administrators
					$sql = "
						REPLACE INTO ". DB_NAME_PREFIX. "admin_session SET
							id = ". (int) $details['id']. ",
							phpsessid = '". mysql_real_escape_string(arrayKey($_COOKIE, 'PHPSESSID')). "'";
					my_mysql_query($sql);
			
					//Note the time this admin last logged in
					$sql = "
						UPDATE ". DB_NAME_PREFIX. "admins SET
							last_login = NOW()
						WHERE id = ". (int) $details['id'];
					my_mysql_query($sql);
				
				} elseif ($details['type'] == 'global') {
					setAdminSession($details['id'], 'super');
					
					//Note the time this admin last logged in
					connectGlobalDB();
						$sql = "
							UPDATE ". DB_NAME_PREFIX_GLOBAL. "superadmins SET
								last_login = NOW()
							WHERE id = ". (int) $details['id'];
						my_mysql_query($sql);
					connectLocalDB();		
				}
				
				//Don't offically mark the admin as "logged in" until they've passed all of the
				//checks in the admin login screen
				$_SESSION['admin_logged_in'] = false;
				
				return true;
			} else {
				$box['tabs']['login']['errors']['details_wrong'] = getPhrase('Your username and password combination was not recognised. Please check and try again.');
			}
		}
	
	//Reset a password
	} elseif ($box['tab'] == 'forgot' && issetArrayKey($box['tabs']['forgot']['fields']['reset'], 'pressed')) {
		$admin = array();
		$box['tabs']['login']['errors'] = array();
		
		if (!$box['tabs']['forgot']['fields']['email']['_value']) {
			$box['tabs']['forgot']['errors'][] = getPhrase('Please enter your Email.');
		
		} elseif (!validateEmailAddress($box['tabs']['forgot']['fields']['email']['_value'])) {
			$box['tabs']['forgot']['errors'][] = getPhrase('Please enter a valid Email Address.');
		
		//Check for the user by email address, without specifying their password.
		//This will return 0 if the user wasn't found, otherwise it should return
		//false (for user found but password didn't match)
		} elseif (checkPasswordAdminByEmail($box['tabs']['forgot']['fields']['email']['_value'], $admin) === 0) {
			$box['tabs']['forgot']['errors'][] = getPhrase("Sorry, we could not find that email address associated with an active Administrator on this site's database.");
		
		} else {
			
			//Prepare email to the mail with the reset password
			$merge = array();
			$merge['NAME'] = ifNull(trim($admin['full_name']), $admin['username']);
			$merge['USERNAME'] = $admin['username'];
			$merge['PASSWORD'] = resetPasswordAdmin($admin['id'], $admin['type']);
			$merge['URL'] = httpOrhttps(). $_SERVER['HTTP_HOST'];
			$merge['SUBDIRECTORY'] = SUBDIRECTORY;
			
			$message = $tags['email_templates']['reset_password']['body'];
			
			$to = $admin['full_name']. '<'. $admin['email']. '>';
			$from = 'noreply@'. $_SERVER['HTTP_HOST'];
			$subject = $tags['email_templates']['reset_password']['subject'];
			$headers = 'From: '. $tags['email_templates']['reset_password']['from'].  '<'. $from. '>';
			
			foreach ($merge as $pattern => $replacement) {
				$message = str_replace('[['. $pattern. ']]', $replacement, $message);
			}
			
			@mail($to, $subject, $message, $headers);
			$passwordReset = true;
			$box['tab'] = 'login';
		}
	}
	
	
	//Format the login screen
	if ($box['tab'] == 'login') {
		if (issetArrayKey($_COOKIE, 'COOKIE_LAST_ADMIN_USER')) {
			$box['tabs']['login']['fields']['username']['value'] = $_COOKIE['COOKIE_LAST_ADMIN_USER'];
			$box['tabs']['login']['fields']['remember_me']['value'] = true;
		}
		
		if (arrayKey($box['tabs']['login']['fields']['username'], '_value')
		 && arrayKey($box['tabs']['login']['fields']['password'], '_value')
		 && !issetArrayKey($box['tabs']['login'], 'errors', 'details_wrong')) {
			$box['tabs']['login']['fields']['login']['style'] = 'float: right;';
			$box['tabs']['login']['fields']['forgot']['style'] = 'float: right; display: none;';
		} else {
			$box['tabs']['login']['fields']['login']['style'] = 'float: right; display: none;';
			$box['tabs']['login']['fields']['forgot']['style'] = 'float: right;';
		}
		
		if ($passwordReset) {
			$box['tabs']['login']['fields']['reset']['hidden'] = false;
			$box['tabs']['login']['fields']['description']['hidden'] = true;
		} else {
			$box['tabs']['login']['fields']['reset']['hidden'] = true;
			$box['tabs']['login']['fields']['description']['hidden'] = false;
		}
	}
	
	return false;
}





function updateAJAX(&$tags, &$box, &$task) {
	if (!checkPriv('_PRIV_UPDATE_DATABASE', true)) {
		$box['tab'] = 0;
		
		if (issetArrayKey($box['tabs'][0]['fields']['previous'], 'pressed')) {
			unsetAdminSession();
			$box['_location'] = 'admin/welcome.php';
		}
	} else {
		
		$box['tab'] = 1;
		
		//Apply updates without prompting when installing, restoring backups or doing site resets
		if ($task == 'install' || $task == 'restore' || $task == 'site_reset' || issetArrayKey($box['tabs'][1]['fields']['apply'], 'pressed')) {
			
			//Update to the latest version
			$dbUpToDate = checkIfDBUpdatesAreNeeded($andDoUpdates = true);
			
			if ($dbUpToDate) {
				return true;
			}
		}
		
		
		$box['tabs'][1]['fields']['description']['snippet']['html'] =
			getPhrase('We need to update your database (<code>[[database]]</code>) to match.', array('database' => htmlspecialchars(DBNAME)));
		
		
		if ($box['tab'] == 1 && issetArrayKey($box['tabs'][1]['fields']['why'], 'pressed')) {
			$plugins = array();
			$revisions = array();
			$currentRevisionNumber = false;
			$latestRevisionNumber = getLatestRevisionNumber();
		
			//Get details on any database updates needed
			if ($updates = checkIfDBUpdatesAreNeeded(false, false, false)) {
				list($currentRevisionNumber, $plugins) = $updates;
			}
				
			if ($currentRevisionNumber === false) {
				$currentRevisionNumber = $latestRevisionNumber;
			}
			
			if ($currentRevisionNumber != $latestRevisionNumber) {
				$revisions['core'] = array('name' => getPhrase('Core Features'), 'from' => $currentRevisionNumber, 'to' => $latestRevisionNumber);
			}
			
			if (!empty($plugins)) {
				foreach($plugins as $plugin => $pluginRevisions) {
					$sql = "
						SELECT display_name
						FROM ". DB_NAME_PREFIX. "plugins
						WHERE name = '". mysql_real_escape_string($plugin). "'";
					
					if (($result = @mysql_query($sql)) && ($row = mysql_fetch_assoc($result))) {
						$pluginName = ifNull($row['display_name'], $plugin);
					} else {
						$pluginName = $plugin;
					}
					
					$revisions[$plugin] = array('name' => $pluginName, 'from' => $pluginRevisions[0] + 1, 'to' => $pluginRevisions[1]);
				}
			}
			
			sqlArraySort($revisions);
			
			$html = '<table class="revisions"><tr><th>'. getPhrase('Plugin'). '</th><th>'. getPhrase('Revision Numbers to Apply'). '</th></tr>';
			
			$oddOrEven = 'even';
			foreach ($revisions as $revision) {
				$oddOrEven = $oddOrEven == 'even'? 'odd' : 'even';
				$html .= '<tr class="'. $oddOrEven. '"><td>'. htmlspecialchars($revision['name']). '</td><td>';
				
				if ($revision['from'] == $revision['to']) {
					$html .= getPhrase('Revision #[[to]]', $revision);
				} else {
					$html .= getPhrase('Revision #[[from]] through to Revision #[[to]]', $revision);
				}
				
				$html .= '</tr>';
			}
			
			$html .= '</table>';
			
			$box['tabs'][1]['fields']['updates']['snippet']['html'] = $html;
			$box['tabs'][1]['fields']['updates']['hidden'] = false;
			$box['tabs'][1]['fields']['why']['class'] = 'opened';
		} else {
			$box['tabs'][1]['fields']['updates']['hidden'] = true;
			$box['tabs'][1]['fields']['why']['class'] = 'closed';
		}
	}
	
	return false;
}





function changePasswordAJAX(&$tags, &$box, &$task) {
	
	//Skip this screen if the Admin presses the "Skip" button
	if (issetArrayKey($box['tabs']['change_password']['fields']['skip'], 'pressed')) {
		cancelPasswordChange(session('admin_userid'), session('admin_authtype'));
		
		$_SESSION['password_needs_changing'] = 0;
		if ($task == 'change_password') {
			$task = 'password_changed';
		}
		
		return true;
	
	//Change the password if the Admin presses the change password
	} elseif (issetArrayKey($box['tabs']['change_password']['fields']['change_password'], 'pressed')) {
		$details = array();
		$box['tabs']['change_password']['errors'] = array();
		$currentPassword = $box['tabs']['change_password']['fields']['current_password']['_value']; 
		$newPassword = $box['tabs']['change_password']['fields']['password']['_value']; 
		$newPasswordConfirm = $box['tabs']['change_password']['fields']['re_password']['_value']; 
		
		if (!$currentPassword) {
			$box['tabs']['change_password']['errors'][] = getPhrase('Please enter your Current Password.');
		
		} elseif (!checkPasswordAdmin(session('admin_username'), $details, $currentPassword)) {
			$box['tabs']['change_password']['errors'][] = getPhrase('_MSG_PASS_WRONG');
		}
		
		if (!$newPassword) {
			$box['tabs']['change_password']['errors'][] = getPhrase('Please enter your New Password.');
		
		} elseif ($newPassword == $currentPassword) {
			$box['tabs']['change_password']['errors'][] = getPhrase('_MSG_PASS_NOT_CHANGED');
		
		} elseif (!checkPasswordStrength($newPassword)) {
			$box['tabs']['change_password']['errors'][] = getPhrase('_MSG_PASS_STRENGTH');
		
		} elseif (!$newPasswordConfirm) {
			$box['tabs']['change_password']['errors'][] = getPhrase('Please repeat your New Password.');
		
		} elseif ($newPassword != $newPasswordConfirm) {
			$box['tabs']['change_password']['errors'][] = getPhrase('_MSG_PASS_2');
		}
		
		//If no errors with validation, then save new password
		if (empty($box['tabs']['change_password']['errors'])) {
			setPasswordAdmin(session('admin_userid'), session('admin_authtype'), $newPassword);
			
			$_SESSION['password_needs_changing'] = 0;
			if ($task == 'change_password') {
				$task = 'password_changed';
			}
			
			return true;
		}
	}
	
	
	//Show the password strength box
	$strength = passwordValuesToStrengths(checkPasswordStrength(arrayKey($box['tabs']['change_password']['fields']['password'], '_value'), true));
	$box['tabs']['change_password']['fields']['password_strength']['snippet']['html'] =
		'<div class="password_'. $strength. '"><span>'. getPhrase($strength). '</span></div>';
	
	
	return false;
}



function diagnosticsAJAX(&$tags, &$box) {

	if (issetArrayKey($box['tabs'][0]['fields']['ignore'], 'pressed')) {
		return true;
	}
	
	
	connectGlobalDB();
		$box['tabs'][0]['fields']['backup_dir']['value']	= (string) getRow('global_settings', 'value', array('name' => 'backup_dir'), false, DB_NAME_PREFIX_GLOBAL);
		$box['tabs'][0]['fields']['docstore_dir']['value']	= (string) getRow('global_settings', 'value', array('name' => 'docstore_dir'), false, DB_NAME_PREFIX_GLOBAL);
		$box['tabs'][0]['fields']['dropbox_dir']['value']	= (string) getRow('global_settings', 'value', array('name' => 'dropbox_dir'), false, DB_NAME_PREFIX_GLOBAL);
		$box['tabs'][0]['fields']['template_dir']['value']	= CMS_ROOT. 'templates/'. SITENAME;
		$box['tabs'][0]['fields']['tmp_downloads_dir']['value']	= CMS_ROOT. 'tmp_downloads';
	connectLocalDB();
	
	if (!issetArrayKey($box, 'tabs', 0, 'fields', 'backup_dir', '_value')) {
		$box['tabs'][0]['fields']['backup_dir']['_value'] =
			$box['tabs'][0]['fields']['backup_dir']['value']?
				$box['tabs'][0]['fields']['backup_dir']['value']
			:	suggestDir('backup');
	}
	
	if (!issetArrayKey($box, 'tabs', 0, 'fields', 'docstore_dir', '_value')) {
		$box['tabs'][0]['fields']['docstore_dir']['_value'] =
			$box['tabs'][0]['fields']['docstore_dir']['value']?
				$box['tabs'][0]['fields']['docstore_dir']['value']
			:	suggestDir('docstore');
	}
	
	if (!issetArrayKey($box, 'tabs', 0, 'fields', 'dropbox_dir', '_value')) {
		$box['tabs'][0]['fields']['dropbox_dir']['_value'] =
			$box['tabs'][0]['fields']['dropbox_dir']['value']?
				$box['tabs'][0]['fields']['dropbox_dir']['value']
			:	suggestDir('dropbox');
	}
	
	
	$box['tabs'][0]['fields']['dirs']['row_class'] = 'section_valid';
	
	if (!arrayKey($box['tabs'][0]['fields']['backup_dir'], '_value')) {
		$box['tabs'][0]['fields']['backup_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['backup_dir_status']['snippet']['html'] = getPhrase('Please enter a directory.');
	
	} elseif (!is_dir(arrayKey($box['tabs'][0]['fields']['backup_dir'], '_value'))) {
		$box['tabs'][0]['fields']['backup_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['backup_dir_status']['snippet']['html'] = getPhrase('This directory does not exist.');
	
	} elseif (!is_writable(arrayKey($box['tabs'][0]['fields']['backup_dir'], '_value'))) {
		$box['tabs'][0]['fields']['backup_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['backup_dir_status']['snippet']['html'] = getPhrase('This directory is not writable.');
	
	} elseif (realpath(arrayKey($box['tabs'][0]['fields']['backup_dir'], '_value')) == realpath(CMS_ROOT)) {
		$box['tabs'][0]['fields']['backup_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['backup_dir_status']['snippet']['html'] = getPhrase('The CMS is installed in this directory. Please choose a different directory.');
	
	} elseif (SITENAME != 'mytribiqsite' && !is_dir($dirPath = arrayKey($box['tabs'][0]['fields']['backup_dir'], '_value'). '/'. SITENAME)) {
		$box['tabs'][0]['fields']['backup_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['backup_dir_status']['snippet']['html'] = getPhrase('The directory [[dir]] does not exist.', array('dir' => $dirPath));
	
	} else {
		$box['tabs'][0]['fields']['dir_1']['row_class'] = 'sub_section_valid';
		$box['tabs'][0]['fields']['backup_dir_status']['row_class'] = 'sub_valid';
		$box['tabs'][0]['fields']['backup_dir_status']['snippet']['html'] = getPhrase('This directory exists and is writable.');
	}
	
	
	if (!arrayKey($box['tabs'][0]['fields']['docstore_dir'], '_value')) {
		$box['tabs'][0]['fields']['docstore_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['docstore_dir_status']['snippet']['html'] = getPhrase('Please enter a directory.');
	
	} elseif (!is_dir(arrayKey($box['tabs'][0]['fields']['docstore_dir'], '_value'))) {
		$box['tabs'][0]['fields']['docstore_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['docstore_dir_status']['snippet']['html'] = getPhrase('This directory does not exist.');
	
	} elseif (!is_writable(arrayKey($box['tabs'][0]['fields']['docstore_dir'], '_value'))) {
		$box['tabs'][0]['fields']['docstore_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['docstore_dir_status']['snippet']['html'] = getPhrase('This directory is not writable.');
	
	} elseif (realpath(arrayKey($box['tabs'][0]['fields']['docstore_dir'], '_value')) == realpath(CMS_ROOT)) {
		$box['tabs'][0]['fields']['docstore_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['docstore_dir_status']['snippet']['html'] = getPhrase('The CMS is installed in this directory. Please choose a different directory.');
	
	} elseif (SITENAME != 'mytribiqsite' && !is_dir($dirPath = arrayKey($box['tabs'][0]['fields']['docstore_dir'], '_value'). '/'. SITENAME)) {
		$box['tabs'][0]['fields']['docstore_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['docstore_dir_status']['snippet']['html'] = getPhrase('The directory [[dir]] does not exist.', array('dir' => $dirPath));
	
	} else {
		$box['tabs'][0]['fields']['dir_2']['row_class'] = 'sub_section_valid';
		$box['tabs'][0]['fields']['docstore_dir_status']['row_class'] = 'sub_valid';
		$box['tabs'][0]['fields']['docstore_dir_status']['snippet']['html'] = getPhrase('This directory exists and is writable.');
	}
	
	
	if (!arrayKey($box['tabs'][0]['fields']['dropbox_dir'], '_value')) {
		$box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['snippet']['html'] = getPhrase('Please enter a directory.');
	
	} elseif (!is_dir(arrayKey($box['tabs'][0]['fields']['dropbox_dir'], '_value'))) {
		$box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['snippet']['html'] = getPhrase('This directory does not exist.');
	
	} elseif (!is_writable(arrayKey($box['tabs'][0]['fields']['dropbox_dir'], '_value'))) {
		$box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['snippet']['html'] = getPhrase('This directory is not writable.');
	
	} elseif (realpath(arrayKey($box['tabs'][0]['fields']['dropbox_dir'], '_value')) == realpath(CMS_ROOT)) {
		$box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['snippet']['html'] = getPhrase('The CMS is installed in this directory. Please choose a different directory.');
	
	} elseif (SITENAME != 'mytribiqsite' && !is_dir($dirPath = arrayKey($box['tabs'][0]['fields']['dropbox_dir'], '_value'). '/'. SITENAME)) {
		$box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['snippet']['html'] = getPhrase('The directory [[dir]] does not exist.', array('dir' => $dirPath));
	
	} else {
		$box['tabs'][0]['fields']['dir_3']['row_class'] = 'sub_section_valid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] = 'sub_valid';
		$box['tabs'][0]['fields']['dropbox_dir_status']['snippet']['html'] = getPhrase('This directory exists and is writable.');
	}
	
	
	if (!is_dir(CMS_ROOT. 'templates/'. SITENAME)) {
		$box['tabs'][0]['fields']['template_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['template_dir_status']['snippet']['html'] = getPhrase('"[[dir]]" does not exist.', array('dir' => CMS_ROOT. 'templates/'. SITENAME));
	} else {
		$box['tabs'][0]['fields']['dir_4']['row_class'] = 'sub_section_valid';
		$box['tabs'][0]['fields']['template_dir_status']['row_class'] = 'sub_valid';
		$box['tabs'][0]['fields']['template_dir_status']['snippet']['html'] = getPhrase('"[[dir]]" exists.', array('dir' => CMS_ROOT. 'templates/'. SITENAME));
	}
	
	
	if (!is_dir(CMS_ROOT. 'tmp_downloads')) {
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['snippet']['html'] = getPhrase('"[[dir]]" does not exist.', array('dir' => CMS_ROOT. 'tmp_downloads'));
	
	} elseif (!is_writable(CMS_ROOT. 'tmp_downloads')) {
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['row_class'] = 'sub_invalid';
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['snippet']['html'] = getPhrase('"[[dir]]" is not writable.', array('dir' => CMS_ROOT. 'tmp_downloads'));
	
	} else {
		$box['tabs'][0]['fields']['dir_5']['row_class'] = 'sub_section_valid';
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['row_class'] = 'sub_valid';
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['snippet']['html'] = getPhrase('"[[dir]]" exists and is writable.', array('dir' => CMS_ROOT. 'tmp_downloads'));
	}
	
	
	if ($box['tabs'][0]['fields']['backup_dir_status']['row_class'] == 'sub_invalid') {
		$box['tabs'][0]['fields']['show_dirs']['pressed'] =
		$box['tabs'][0]['fields']['show_dir_1']['pressed'] = true;
		$box['tabs'][0]['fields']['dirs']['row_class'] = 'section_invalid';
		$box['tabs'][0]['fields']['dir_1']['row_class'] = 'sub_section_invalid';
	}
	
	if ($box['tabs'][0]['fields']['docstore_dir_status']['row_class'] == 'sub_invalid') {
		$box['tabs'][0]['fields']['show_dirs']['pressed'] =
		$box['tabs'][0]['fields']['show_dir_2']['pressed'] = true;
		$box['tabs'][0]['fields']['dirs']['row_class'] = 'section_invalid';
		$box['tabs'][0]['fields']['dir_2']['row_class'] = 'sub_section_invalid';
	}
	
	if ($box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] == 'sub_invalid') {
		$box['tabs'][0]['fields']['show_dirs']['pressed'] =
		$box['tabs'][0]['fields']['show_dir_3']['pressed'] = true;
		$box['tabs'][0]['fields']['dirs']['row_class'] = 'section_invalid';
		$box['tabs'][0]['fields']['dir_3']['row_class'] = 'sub_section_invalid';
	}
	
	if ($box['tabs'][0]['fields']['template_dir_status']['row_class'] == 'sub_invalid') {
		$box['tabs'][0]['fields']['show_dirs']['pressed'] =
		$box['tabs'][0]['fields']['show_dir_4']['pressed'] = true;
		$box['tabs'][0]['fields']['dirs']['row_class'] = 'section_invalid';
		$box['tabs'][0]['fields']['dir_4']['row_class'] = 'sub_section_invalid';
	}
	
	if ($box['tabs'][0]['fields']['tmp_downloads_dir_status']['row_class'] == 'sub_invalid') {
		$box['tabs'][0]['fields']['show_dirs']['pressed'] =
		$box['tabs'][0]['fields']['show_dir_5']['pressed'] = true;
		$box['tabs'][0]['fields']['dirs']['row_class'] = 'section_invalid';
		$box['tabs'][0]['fields']['dir_5']['row_class'] = 'sub_section_invalid';
	}
	
	
	if ((function_exists('curl_init'))
	 && ($curl = @curl_init())
	 && (@curl_setopt($curl, CURLOPT_URL, 'http://tribiq.com/'))
	 && (@curl_setopt($curl, CURLOPT_HEADER, 0))
	 && (@curl_setopt($curl, CURLOPT_RETURNTRANSFER, 1))) {
		
		$box['tabs'][0]['fields']['curl_1']['row_class'] = 'valid';
		$box['tabs'][0]['fields']['curl_1']['snippet']['html'] = getPhrase('CURL Support is enabled in PHP.');
		$box['tabs'][0]['fields']['curl_2']['hidden'] = false;
		
		if ((@curl_exec($curl) !== false)) {
			$box['tabs'][0]['fields']['curl']['row_class'] = 'section_valid';
			$box['tabs'][0]['fields']['curl_2']['row_class'] = 'valid';
			$box['tabs'][0]['fields']['curl_2']['snippet']['html'] = getPhrase('Test CURL statement to http://tribiq.com was successful.');
		} else {
			$box['tabs'][0]['fields']['show_curl']['pressed'] = true;
			$box['tabs'][0]['fields']['curl']['row_class'] = 'section_invalid';
			$box['tabs'][0]['fields']['curl_2']['row_class'] = 'invalid';
			$box['tabs'][0]['fields']['curl_2']['snippet']['html'] = getPhrase('Test CURL statement to http://tribiq.com was not successful.');
		}
	} else {
		$box['tabs'][0]['fields']['show_curl']['pressed'] = true;
		$box['tabs'][0]['fields']['curl']['row_class'] = 'section_invalid';
		$box['tabs'][0]['fields']['curl_1']['row_class'] = 'invalid';
		$box['tabs'][0]['fields']['curl_1']['snippet']['html'] = getPhrase('CURL Support not enabled in PHP. To access the Tribiq Store you should ask your system administrator to enable/recompile PHP with CURL support.');
		$box['tabs'][0]['fields']['curl_2']['hidden'] = true;
	}
	
	
	$box['tabs'][0]['fields']['show_dirs']['class'] = (
		$box['tabs'][0]['fields']['dir_1']['hidden'] =
		$box['tabs'][0]['fields']['show_dir_1']['hidden'] =
		$box['tabs'][0]['fields']['dir_2']['hidden'] =
		$box['tabs'][0]['fields']['show_dir_2']['hidden'] =
		$box['tabs'][0]['fields']['dir_3']['hidden'] =
		$box['tabs'][0]['fields']['show_dir_3']['hidden'] =
		$box['tabs'][0]['fields']['dir_4']['hidden'] =
		$box['tabs'][0]['fields']['show_dir_4']['hidden'] =
		$box['tabs'][0]['fields']['dir_5']['hidden'] =
		$box['tabs'][0]['fields']['show_dir_5']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_dirs'], 'pressed')
		) ?
			'closed'
		:	'opened';
	
	$box['tabs'][0]['fields']['show_dir_1']['class'] = (
		$box['tabs'][0]['fields']['dir_1_blurb']['hidden'] =
		$box['tabs'][0]['fields']['backup_dir']['hidden'] =
		$box['tabs'][0]['fields']['backup_dir_status']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_dir_1'], 'pressed') || !issetArrayKey($box['tabs'][0]['fields']['show_dirs'], 'pressed')
		) ?
			'closed'
		:	'opened';
	
	$box['tabs'][0]['fields']['show_dir_2']['class'] = (
		$box['tabs'][0]['fields']['dir_2_blurb']['hidden'] =
		$box['tabs'][0]['fields']['docstore_dir']['hidden'] =
		$box['tabs'][0]['fields']['docstore_dir_status']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_dir_2'], 'pressed') || !issetArrayKey($box['tabs'][0]['fields']['show_dirs'], 'pressed')
		) ?
			'closed'
		:	'opened';
	
	$box['tabs'][0]['fields']['show_dir_3']['class'] = (
		$box['tabs'][0]['fields']['dir_3_blurb']['hidden'] =
		$box['tabs'][0]['fields']['dropbox_dir']['hidden'] =
		$box['tabs'][0]['fields']['dropbox_dir_status']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_dir_3'], 'pressed') || !issetArrayKey($box['tabs'][0]['fields']['show_dirs'], 'pressed')
		) ?
			'closed'
		:	'opened';
	
	$box['tabs'][0]['fields']['show_dir_4']['class'] = (
		$box['tabs'][0]['fields']['dir_4_blurb']['hidden'] =
		$box['tabs'][0]['fields']['template_dir']['hidden'] =
		$box['tabs'][0]['fields']['template_dir_status']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_dir_4'], 'pressed') || !issetArrayKey($box['tabs'][0]['fields']['show_dirs'], 'pressed')
		) ?
			'closed'
		:	'opened';
	
	$box['tabs'][0]['fields']['show_dir_5']['class'] = (
		$box['tabs'][0]['fields']['dir_5_blurb']['hidden'] =
		$box['tabs'][0]['fields']['tmp_downloads_dir']['hidden'] =
		$box['tabs'][0]['fields']['tmp_downloads_dir_status']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_dir_5'], 'pressed') || !issetArrayKey($box['tabs'][0]['fields']['show_dirs'], 'pressed')
		) ?
			'closed'
		:	'opened';
	
	$box['tabs'][0]['fields']['show_curl']['class'] = (
		$box['tabs'][0]['fields']['curl_1']['hidden'] =
		$box['tabs'][0]['fields']['curl_2']['hidden'] =
			!issetArrayKey($box['tabs'][0]['fields']['show_curl'], 'pressed')
		) ?
			'closed'
		:	'opened';

	
	$dirsToCreate = array();
	connectGlobalDB();
		if ($box['tabs'][0]['fields']['backup_dir_status']['row_class'] == 'sub_valid') {
			setRow('global_settings', array('value' => arrayKey($box['tabs'][0]['fields']['backup_dir'], '_value')), array('name' => 'backup_dir'), DB_NAME_PREFIX_GLOBAL);
			$dirsToCreate[] = 'backup_dir';
		}
		
		if ($box['tabs'][0]['fields']['docstore_dir_status']['row_class'] == 'sub_valid') {
			setRow('global_settings', array('value' => arrayKey($box['tabs'][0]['fields']['docstore_dir'], '_value')), array('name' => 'docstore_dir'), DB_NAME_PREFIX_GLOBAL);
			$dirsToCreate[] = 'docstore_dir';
		}
		
		if ($box['tabs'][0]['fields']['dropbox_dir_status']['row_class'] == 'sub_valid') {
			setRow('global_settings', array('value' => arrayKey($box['tabs'][0]['fields']['dropbox_dir'], '_value')), array('name' => 'dropbox_dir'), DB_NAME_PREFIX_GLOBAL);
			$dirsToCreate[] = 'dropbox_dir';
		}
	connectLocalDB();
	
	//For single-sites, automatically create the mytribiqsite subdirectory
	$box['tabs'][0]['errors'] = array();
	if (SITENAME == 'mytribiqsite') {
		foreach ($dirsToCreate as $dir) {
			if (!is_dir($dirPath = $box['tabs'][0]['fields'][$dir]['_value']. '/'. SITENAME)) {
				if (!@mkdir($dirPath)) {
					$box['tabs'][0]['errors'][] = getPhrase('Could not create the directory "[[dir]]".', array('dir' => $dirPath));
				}
			}
		}
	}

	if ($box['tabs'][0]['fields']['dirs']['row_class'] == 'section_valid' && $box['tabs'][0]['fields']['curl']['row_class'] == 'section_valid' && empty($box['tabs'][0]['errors'])) {
		return true;
	} else {
		return false;
	}
}

function congratulationsAJAX(&$tags, &$box) {
	$box['tabs'][0]['fields']['link']['snippet']['html'] =
		'<a href="'. httpOrhttps(). $_SERVER['HTTP_HOST']. SUBDIRECTORY. '">'. httpOrhttps(). $_SERVER['HTTP_HOST']. SUBDIRECTORY. '</a>';
}




function redirectAdmin($getRequest) {
	
	if (issetArrayKey($getRequest, 'sk') && checkPriv()) {
		return 'admin/storekeeper.php#'. $getRequest['sk'];
		
	} elseif (issetArrayKey($getRequest, 'desturl') && checkPriv()) {
		return httpOrhttps(). DEFAULT_URL. $getRequest['desturl'];
		
	} else {
		$cID = (int) arrayKey($getRequest, 'cID');
		$cType = ifNull(preg_replace('/\W/', '', arrayKey($getRequest, 'cType')), 'html');
	
		if ($cID && checkPerm($cID, $cType)) {
			return linkToItem($cID, $cType, false, true, '', true, false);
		} else {
			return '';
		}
	}
}





?>