<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

require '../cmsincludes/cache.inc.php';

//Ensure that the site name and subdirectory are part of the ETag, as Skins can have different ids on different servers
$ETag = 'tribiq-skin-'. $_SERVER['HTTP_HOST']. '-'. CSS_JS_VERSION. '-'. (int) $_GET['id'];

if (isset($_GET['editor'])) {
	$ETag .= '-editor';

} elseif (isset($_GET['print'])) {
	$ETag .= '-print';
}

//Cache this combination of running Plugin JavaScript
useCache($ETag);
useGZIP($_GET['gz']);

require 'visitorheader.inc.php';
header('Content-Type: text/css; charset=UTF-8');


//Define a function to include a CSS file
function includeSkinFile($file, $skinPath, $skinPathURL) {
	//Check that this looks like a valid CSS file
	if (substr($file, 0, 1) != '.'
	 && substr($file, -4) == '.css'
	 && is_file(CMS_ROOT. $skinPath. $file)) {
		
		//Include the contents of the file, being careful to correct for the fact that the relative path for images will be wrong
		echo preg_replace('/url\(([\'\"]?)([^:]+?)\)/i', 'url($1../'. $skinPathURL. '$2)', file_get_contents(CMS_ROOT. $skinPath. $file));
		
		echo "\n/**/\n";
		
		return true;
	} else {
		return false;
	}
}


//Look up the skin from the database
if (!$skin = getSkinFromId($_GET['id'])) {
	exit;
}

$templateFamily = $skin['family_name'];
$skins = array($skinName = $skin['name']);

$limit = 10;
do {
	$desc = false;
	$addedSkin = false;
	
	if (validateSkinFromXMLDescriptionByName($templateFamily, $skinName, $desc)) {
		if ($skinName = (string) $desc->extension_of_skin) {
			if (!in_array($skinName, $skins)) {
				array_unshift($skins, $skinName);
				$addedSkin = true;
			}
		}
	}
} while (--$limit && $addedSkin);


foreach ($skins as $skinName) {
	$skinPath = 'templates/'. SITENAME. '/'. $templateFamily. '/skins/'. $skinName. '/';
	$skinPathURL = 'templates/'. SITENAME. '/'. $templateFamily. '/skins/'. rawurlencode($skinName). '/';
	
	if (!is_dir(CMS_ROOT. $skinPath)) {
		echo "\n\n". getPhrase('Not Found: '). $templateFamily. '/skins/'. $skinName;
		exit;
	}
	
	if (get('editor')) {
		if (includeSkinFile('tinymce.css', $skinPath, $skinPathURL)) {
		} elseif (includeSkinFile('punymce.css', $skinPath, $skinPathURL)) {
		} elseif (includeSkinFile('ckeditor.css', $skinPath, $skinPathURL)) {
		} elseif (includeSkinFile('fckeditor.css', $skinPath, $skinPathURL)) {
		}
	
	} elseif (get('print')) {
		includeSkinFile('stylesheet_print.css', $skinPath, $skinPathURL);
	
	} else {
		//Include all of the CSS needed for a skin
		
		//Include the reset file if it is there
		includeSkinFile('reset.css', $skinPath, $skinPathURL);
		//Include the main stylesheet first, if it is there
		includeSkinFile('stylesheet.css', $skinPath, $skinPathURL);
		
		//Include every other non-special CSS file in the directory
		$specialFiles = array(
			'reset.css' => true,
			'stylesheet.css' => true,
			'ckeditor.css' => true,
			'fckeditor.css' => true,
			'tinymce.css' => true,
			'punymce.css' => true,
			'style_ie.css' => true,
			'style_ie6.css' => true,
			'style_ie7.css' => true,
			'style_ie8.css' => true,
			'stylesheet_print.css' => true);
		
		foreach(scandir(CMS_ROOT. $skinPath) as $file) {
			if (!$specialFiles[$file]) {
				includeSkinFile($file, $skinPath, $skinPathURL);
			}
		}
		
		//Include a list of fixes for IE
		if (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE') !== false) {
			includeSkinFile('style_ie.css', $skinPath, $skinPathURL);
			if (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 9') !== false) {
				includeSkinFile('style_ie9.css', $skinPath, $skinPathURL);
			} elseif (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 8') !== false) {
				includeSkinFile('style_ie8.css', $skinPath, $skinPathURL);
			} elseif (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 7') !== false) {
				includeSkinFile('style_ie7.css', $skinPath, $skinPathURL);
			} elseif (strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 6') !== false) {
				includeSkinFile('style_ie6.css', $skinPath, $skinPathURL);
			}
		}
	}
}