
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


/**
  * This section lists all of the JavaScript functions and variables available to Plugins on the client side.
 */
tribiq.enc = function(id, name, className, pluginClassNameForPhrases) {{
  if (typeof window[className] != 'function') {
	
	window[className] = function() {};
	
	
	/*
		Variables
		(See environment_variables.inc.php for their descriptions)
	*/
	
	window[className].cID = tribiq.cID;
	window[className].cType = tribiq.cType;
	window[className].cVersion = tribiq.cVersion;
	window[className].languageId = tribiq.languageId;
	window[className].pluginId = id;
	window[className].pluginName = name;
	window[className].pluginClassName = className;
	window[className].pluginClassNameForPhrases = pluginClassNameForPhrases;
	
	
	/*
		Utility Functions
	*/
	
	//Launch an AJAX request to your Plugin's handleAJAX placeholder method
	window[className].AJAX = function(requests, post) {
		return tribiq.pluginAJAX(this.pluginName, requests, post, false);
	};
	 
	//Launch a JSON request to your Plugin's handleAJAX placeholder method
	window[className].JSON = function(requests, post) {
		return tribiq.pluginAJAX(this.pluginName, requests, post, true);
	};
	
 /**
  * The floatingBoxAnchor() method allows Plugin Developers to create a hyperlink which calls a
  * Tribiq-themed alert box to appear when an Admin clicks on it.
  * 
  * If a Plugin Developer wishes to have a confirmation box and not an alert box, then the $buttons
  * input can be passed the HTML needed for the buttons they wish to appear.
  * 
  * Note that this will only work in Admin mode.
  */
	window[className].floatingMessage = function(html, buttonsHTML, warning) {
		if (typeof tribiqA != 'undefined') {
			tribiqA.floatingBox(html, buttonsHTML, warning);
		}
	};
	
	//Returns the instance id of this Plugin.
	//Warning: If you have more than one instance on a page then it's not defined which one will be returned
	window[className].instanceId = function() {
		for (i in this.slots) {
			return this.slots[i].instanceId;
		}
		return false;
	};
	
	//Fetch one of your visitor phrases via AJAX.
	window[className].phrase = function(code, request) {
		return tribiq.getPluginPhrase(this.pluginClassNameForPhrases, code, request);
	};
	 
	//Go to a content item
	window[className].goToItem = function(cID, cType, request) {
		tribiq.goToURL(tribiq.linkToItem(cID, cType, request));
	};
	 
	//Go to a URL
	window[className].goToURL = function(url) {
		tribiq.goToURL(url);
	};
	
	//Reload the contents of a slot.
	window[className].refreshPluginSlot = function(slotNameOrContainedElement, requests, scrollToTopOfSlot) {
		var slotName = tribiq.getSlotnameFromEl(slotNameOrContainedElement);
		
		if (slotName === false) {
			return;
		}
		
		if (scrollToTopOfSlot === undefined) {
			scrollToTopOfSlot = true;
		}
		
		tribiq.refreshPluginSlot(slotName, 'lookup', requests, true, scrollToTopOfSlot);
	};
	
	//Scroll to the top of a slot
	window[className].scrollToSlotTop = function(slotNameOrContainedElement, neverScrollDown) {
		var slotName = tribiq.getSlotnameFromEl(slotNameOrContainedElement);
		tribiq.scrollToSlotTop(slotName, neverScrollDown);
	};
	
	//Returns the slot name that this Plugin is in
	//Warning: If this Plugin is in more than one slot on a page then it's not defined which one be returned
	window[className].slotName = function() {
		for (i in this.slots) {
			return this.slots[i].slotName;
		}
		return false;
	};
	
	
	
	
	
	/*
		This bit of code stores a record of which Plugins are on the page.
		You can reference these objects using some of the API functions above.
	*/
	
	window[className].slots = new Object();
	
	tribiq.plugins[id] = function() {};
	tribiq.plugins[id].pluginName = name;
	tribiq.plugins[id].pluginClassName = className;
	tribiq.plugins[id].pluginClassNameForPhrases = pluginClassNameForPhrases;

  }
}}