<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */


class tribiq_avls extends plugin_base_class {
	
	
	public function lineStorekeeper($path, &$panel, $refinerName, $refinerId, $selectMode, $isXML) {
		switch ($path) {
			case 'tribiq__content/content':
				if ($refinerName == 'tribiq_avls__chained_in_link') {
					unset($panel['columns']['tribiq_avls__links']);
				} else {
					unset($panel['columns']['tribiq_avls__language_id']);
				}
				
				break;
		}
	}
	
	public function fillStorekeeper($path, &$panel, $refinerName, $refinerId, $selectMode, $isXML) {
		switch ($path) {
			case 'tribiq__content/content':
				$mrg = array(
					'type' => getContentTypeName($cType = $this->get('refiner__content_type')),
					'lang' => getLanguageName($langId = $this->get('refiner__language')));
		
				$numLanguages = count($langs = getLanguages());
				if (!isset($this->getVariables['refiner__trash']) && $numLanguages > 1) {
					
					$panel['item_buttons']['tribiq_avls__declare']['combine_items']['path'] .= $this->get('refiner__content_type'). '//';
					$panel['collection_buttons']['tribiq_avls__link_to_chain']['pick_items']['path'] .= $this->get('refiner__content_type'). '//';
					
					if ($refinerName == 'tribiq_avls__chained_in_link') {
						foreach ($panel['items'] as &$item) {
							if (!is_array($item['traits'])) {
								$item['traits'] = array();
							}
							$item['traits']['tribiq_avls__linked'] = true;
						}
						
						unset($panel['item_buttons']['tribiq_avls__declare']);
						unset($panel['item_buttons']['tribiq_avls__view']);
						unset($panel['collection_buttons']['create']);
						if (count($panel['items']) >= $numLanguages) {
							unset($panel['collection_buttons']['tribiq_avls__link_to_chain']);
						}
						
						$panel['label_format'] = "[[tag]] \n [[tribiq_avls__language_id]]";
						$panel['title'] = getPhrase('Equivalence Chain: Reference Language "[[lang]]"', $mrg);
						
					} else {
						foreach($langs as $lang) {
							if ($lang['id'] != $langId) {
								$panel['columns']['lang_'. $lang['id']] = array('title' => $lang['id']);
							}
						}
						
						foreach ($panel['items'] as $cID => &$item) {
							if (!is_array($item['traits'])) {
								$item['traits'] = array();
							}
							
							$item['tribiq_avls__links'] = 0;
							if ($equivId = tribiq_avls::getEquivalenceKey($cID, $cType)) {
								$equivs = tribiq_avls::equivalences($cID, $cType, $includeCurrent = false, $equivId);
								
								foreach($langs as $lang) {
									if (issetArrayKey($equivs, $lang['id'])) {
										++$item['tribiq_avls__links'];
										$status = getLatestVersionStatus($equivs[$lang['id']]['cID'], $cType, $useRevisionStatus = true);
										$item['lang_'. $lang['id']. '__css_class'] = 'tribiq_avls_colicon tribiq_avls_colicon__'. $status;
											
										switch($status) {
											case 'published': 
											case 'revision': 
											case 'private_draft': 
											case 'hidden': 
											case 'archived': 
											default:
										}
									}
								}
							}
							
							if ($item['tribiq_avls__links'] < $numLanguages) {
								$item['traits']['tribiq_avls__can_link'] = true;
							}
							if ($item['tribiq_avls__links'] > 0) {
								--$item['tribiq_avls__links'];
								$item['traits']['tribiq_avls__linked'] = true;
							}
						}
					}
				} else {
					unset($panel['columns']['tribiq_avls__links']);
				}
				
				break;
			
			
			case 'tribiq__languages':
				if (!$isXML) {
					$atLeastOneLanguageEnabled = false;
					foreach ($panel['items'] as $id => &$item) {
						if ($item['enabled']) {
							$atLeastOneLanguageEnabled = true;
						}
					}
					
					if (!$atLeastOneLanguageEnabled) {
						unset($panel['item_buttons']['tribiq_avls__add_another_language']);
					}
				}
				
				break;
		}
	}


	public function storekeeperAJAX($path, $ids, $ids2, $refinerName, $refinerId) {
	
		if ($this->post('remove')) {
			tribiq_avls::removeEquivalence($this->post('id'), $this->post('refiner__content_type'));
		
		} elseif ($this->post('declare')) {
			
			$cID1 = $this->post('id');
			$cID2 = ifNull($this->post('id2'), $this->post('refiner__tribiq_avls__chained_in_link'));
			$cType = $this->post('refiner__content_type');
			
			if (($lang = getContentLang($cID1, $cType)) == getContentLang($cID2, $cType)) {
				echo getPhrase('Please ensure that both Content Items are in a different Language.');
			
			} elseif (!$this->checkMergeAllowed($cID1, $cID2, $cType)) {
				echo getPhrase('An Equivalence Chain cannot contain more than one Content Item of the same Language. Please select a different Item.');
			
			} else {
				$this->checkMergeAllowed($cID1, $cID2, $cType, $doMerge = true);
			}
		}
	}
	
	
	
	
	
	






	static function equivalentItem(&$cID, &$cType, $langId = false) {
		
		if (!$cID || !$cType) {
			return false;
		}
		
		if ($langId === false) {
			$langId = $_SESSION['user_lang'];
		}
		
		$id = tribiq_avls::getEquivalenceKey($cID, $cType);
		
		$sql = "
			SELECT content_id, content_type	
			FROM ".DB_NAME_PREFIX."lang_equiv_content
			WHERE language_id = '". mysql_real_escape_string($langId). "'
				AND id = ". (int) $id;
		$result = my_mysql_query($sql);
		
		if ($row = mysql_fetch_assoc($result)) {
			$cID = $row['content_id'];
			$cType = $row['content_type'];
			return true;
		} else {
			return false;
		}
	}
	
	protected function checkMergeAllowed($focusCID, $targetCID, $cType, $doMerge = false) {
		
		$focusEquivId = tribiq_avls::getEquivalenceKey($focusCID, $cType);
		$focusLanguageId = getContentLang($focusCID, $cType);
		
		$targetLanguageId = getContentLang($targetCID, $cType);
		
		//Work out a list of items in the target language that it would be legitimate to
		//declare a content equivalence with.
		//This is a bit complex, as we can include content items that are already in an
		//equivalence, if there are no overlaps between the two equivalences!
		$sql = "
			SELECT DISTINCT
				c.id AS content_id,
				target_equiv.id AS target_equiv_id
			FROM ". DB_NAME_PREFIX. "content_". mysql_real_escape_string($cType). " AS c
			LEFT JOIN ". DB_NAME_PREFIX. "lang_equiv_content as target_equiv
			   ON target_equiv.content_id = c.id
			  AND target_equiv.content_type = '". mysql_real_escape_string($cType). "'
			WHERE c.language_id = '". mysql_real_escape_string($targetLanguageId). "'
			  AND c.`status` IN ('published', 'private_draft', 'hidden')
			  AND ((target_equiv.id IS NULL
				". ($focusEquivId? "
					AND c.language_id NOT IN (
						SELECT language_id
						FROM ". DB_NAME_PREFIX. "lang_equiv_content
						WHERE id = ". (int) $focusEquivId. "
					)
				" : ""). ")
			   OR target_equiv.id NOT IN (
				SELECT DISTINCT id
				FROM ". DB_NAME_PREFIX. "lang_equiv_content
				". ($focusEquivId? "
					WHERE language_id IN (
						SELECT language_id
						FROM ". DB_NAME_PREFIX. "lang_equiv_content
						WHERE id = ". (int) $focusEquivId. "
					)
				" : "
					WHERE language_id = '". mysql_real_escape_string($focusLanguageId). "'
				"). "
			))
			  AND c.id = ". (int) $targetCID;
		
		
		$result = my_mysql_query($sql);
		if ($row = mysql_fetch_assoc($result)) {
			if ($doMerge) {
				$targetEquivId = $row['target_equiv_id'];
				
				//There will be four cases to deal with, depending on which - if any - of the two equivalence ids are set
				//However we can narrow this down to three cases as if one is set and one is not, then it does not matter which
				//way around they are
				if (!$focusEquivId && $targetEquivId) {
					$focusEquivId = $targetEquivId;
					$targetEquivId = null;
					
					$t = $focusCID;
					$focusCID = $targetCID;
					$targetCID = $t;
					
					$t = $focusLanguageId;
					$focusLanguageId = $targetLanguageId;
					$targetLanguageId = $t;
				}
				
				
				if (!$targetEquivId) {
					//Deal with the two cases where one or more content items do not have an equivalence yet
					if (!$focusEquivId) {
						//If neither have one, then we need to create a new equivalence id and then set them both
						$focusEquivId = tribiq_avls::getNewEquivId();
						
						$sql = "
							INSERT INTO ". DB_NAME_PREFIX. "lang_equiv_content SET
								id = ". (int) $focusEquivId. ",
								language_id = '". mysql_real_escape_string($focusLanguageId). "',
								content_id = ". (int) $focusCID. ",
								content_type = '". mysql_real_escape_string($cType). "'";
						my_mysql_query($sql);
					}
					
					//If one has one and the other has not, set the one that does not
					$sql = "
						INSERT INTO ". DB_NAME_PREFIX. "lang_equiv_content SET
							id = ". (int) $focusEquivId. ",
							language_id = '". mysql_real_escape_string($targetLanguageId). "',
							content_id = ". (int) $targetCID. ",
							content_type = '". mysql_real_escape_string($cType). "'";
					
					my_mysql_query($sql);
				
				} else {
					//Merge two equivalences into one
					//Make sure we keep the lowest equivalence id and chuck the largest number
					$sql = "
						UPDATE ". DB_NAME_PREFIX. "lang_equiv_content SET
							id = ". (int) ($focusEquivId < $targetEquivId? $focusEquivId : $targetEquivId) . "
						WHERE id = ". (int) ($focusEquivId < $targetEquivId? $targetEquivId : $focusEquivId);
					
					my_mysql_query($sql);
				}
			
			}
			
			return true;
		} else {
			return false;
		}
	}

	static function equivalences($cID, $cType, $includeCurrent = true, $id = false) {
		$equivs = array();
		
		if ($id === false) {
			$id = tribiq_avls::getEquivalenceKey($cID, $cType);
		}
		
		$sql = "
			SELECT language_id, content_id AS cID, content_type	AS cType
			FROM ".DB_NAME_PREFIX."lang_equiv_content
			WHERE id = ". (int) $id."
			ORDER BY language_id";
		$result = my_mysql_query($sql);
		
		while($equiv = mysql_fetch_assoc($result)) {
			if ($includeCurrent || $equiv['content_id'] != $cID) {
				$equivs[$equiv['language_id']] = $equiv;
			}
		}
		
		return $equivs;
	}



	static function getEquivalenceKey($cID, $cType, $langId = false) {
		$sql = "
			SELECT id
			FROM ".DB_NAME_PREFIX."lang_equiv_content
			WHERE content_id = ". (int) $cID. "
				AND content_type = '". mysql_real_escape_string($cType). "'";
		if ($langId) {
			$sql .= "
				AND language_id = '". mysql_real_escape_string($langId). "'";
		}
		
		$result = my_mysql_query($sql);
		if (mysql_num_rows($result)>0) {
			$row = mysql_fetch_array($result);
			return $row['id'];
		} else {
			return false;
		}
	}
	
	


	static function getNewEquivId() {
		$sql = "
			SELECT max(id)
			FROM ". DB_NAME_PREFIX. "lang_equiv_content";
		list($currentEquivId) = mysql_fetch_array(my_mysql_query($sql));
		
		//If lang_equiv_content is empty, the SQL above will return NULL. Fix that here
		if (!$currentEquivId) {
			$currentEquivId = 1;
		} else {
			$currentEquivId++;
		}
		
		return $currentEquivId;
	}
	
	



	//Given a menu id, try to find its equivalent in another language
	static function menuEquivalence($menuId, $langId) {
	
		$sql = "
			SELECT m2.id
			FROM ". DB_NAME_PREFIX. "menuitems AS m1
			INNER JOIN ". DB_NAME_PREFIX. "lang_equiv_content e1
			   ON m1.content_id = e1.content_id
			  AND m1.content_type = e1.content_type
			INNER JOIN ". DB_NAME_PREFIX. "lang_equiv_content e2
			   ON e1.id = e2.id
			INNER JOIN ". DB_NAME_PREFIX. "menuitems m2
			   ON m2.content_id = e2.content_id
			  AND m2.content_type = e2.content_type
			WHERE m1.id = ". (int) $menuId. "
			  AND e2.language_id = '". htmlspecialchars($langId). "'
			ORDER BY m1.redundancy = 'primary' DESC,  m2.redundancy = 'primary' DESC
			LIMIT 1";
		
		$result = my_mysql_query($sql);
		
		if (mysql_num_rows($result)>0) {
			$row = mysql_fetch_assoc($result);
			return $row['id'];
		} else {
			return false;
		}
	}

	//If the user duplicates content into another language, add a record of this into the
	//equivalences table if apropriate.
	static function recordEquivalenceOfDuplicatedLanguage($cType, $oldCID, $newCID) {
		
		$oldLang = getContentLang($oldCID, $cType);
		$newLang = getContentLang($newCID, $cType);
		
		if ($oldLang == $newLang) {
			return;
		}
	
		$equivId = tribiq_avls::getEquivalenceKey($oldCID, $cType);
		
		//Check if an equivalence already existed
		if ($equivId == 0) {
		
			//If not, we'll need to add a new one
			$equivId = tribiq_avls::getNewEquivId();
			
			//Add the current item into the equivalence table
			$sql = "
				INSERT INTO ". DB_NAME_PREFIX. "lang_equiv_content(id, language_id, content_id, content_type)
				values(". (int) $equivId. ", '". mysql_real_escape_string($oldLang). "', ". (int) $oldCID. ", '". mysql_real_escape_string($cType). "')";
			my_mysql_query($sql);
			
			$insertEquiv = true;
		} else {
			//Check an equivalence doesn't already exist in the new language!
			$sql = "
				SELECT COUNT(*)
				FROM ". DB_NAME_PREFIX. "lang_equiv_content
				WHERE language_id = '". mysql_real_escape_string($newLang). "'
				  AND id = ". $equivId;
			
			$result = my_mysql_query($sql);
			list($exists) = mysql_fetch_row($result);
			
			$insertEquiv = $exists == 0;
		}
		
		//If we can, insert a new equivalence in for the new content
		if ($insertEquiv) {
			$sql = "
				INSERT INTO ". DB_NAME_PREFIX. "lang_equiv_content(id, language_id, content_id, content_type)
				values(". $equivId. ", '". mysql_real_escape_string($newLang). "', ". $newCID. ", '". $cType. "')";
			my_mysql_query($sql);
		}
	
	}


	//Remove a content equivalence link from the database
	static function removeEquivalence($cID, $cType, $equivId = false) {
		
		if ($equivId === false) {
			$equivId = tribiq_avls::getEquivalenceKey($cID, $cType);
		}
	
		//Check the number of equivalence entries for this equivalence
		$sql = "
			SELECT id
			FROM ". DB_NAME_PREFIX. "lang_equiv_content
			WHERE id = ". (int) $equivId;
		
		$result = my_mysql_query($sql);
		
		if (mysql_num_rows($result) == 2) {
			//A content item cannot be an equivalent on its own!
			//So if there are only two left, and we are removing one, we must actually
			//remove them both
			$sql = "
				DELETE FROM ". DB_NAME_PREFIX. "lang_equiv_content
				WHERE id = ". (int) $equivId;
		} else {
			//Otherwise just remove the requested equivalence entry from the database
			$sql = "
				DELETE FROM ". DB_NAME_PREFIX. "lang_equiv_content 
				WHERE content_id = ". (int) $cID."
				  AND content_type = '". mysql_real_escape_string($cType). "'
				LIMIT 1";
		}
		
		my_mysql_query($sql);
	}

}
?>