<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */
if (!defined('NOT_ACCESSED_DIRECTLY') || !checkPriv()) exit;

switch ($path) {
	case 'tribiq__menu/menu_items':
		if ($this->get('quick_create') && (
				($this->get('refiner__children') && checkPriv('_PRIV_ADD_MENU_ITEM'))
			 || (!$this->get('refiner__children') && checkPriv('_PRIV_ADD_TOP_LEVEL_MENU_ITEM'))
		)) {
			$defaultName = '';
			if ($this->get('parent__id') && $this->get('parent__refiner__content_type')) {
				$defaultName = getItemTitle($this->get('parent__id'), $this->get('parent__refiner__content_type'));
			} elseif ($this->get('parent__cID') && $this->get('parent__cType')) {
				$defaultName = getItemTitle($this->get('parent__cID'), $this->get('parent__cType'));
			}
			
			echo
				'<p>', 
					getPhrase('Create a new Menu Item.'), 
				'</p><p>',
					getPhrase('Name:'),
					' <input type="text" id="quick_create_name" name="quick_create_name" value="', htmlspecialchars($defaultName), '"/>',
				'</p>';
		
		} elseif ($this->post('quick_create') && (
					($this->post('refiner__children') && checkPriv('_PRIV_ADD_MENU_ITEM'))
				 || (!$this->post('refiner__children') && checkPriv('_PRIV_ADD_TOP_LEVEL_MENU_ITEM'))
		)) {
			
			if (!$this->post('quick_create_name')) {
				echo getPhrase('Please enter a name for your Menu Item.');
				return false;
			} else {
				$menuId = false;
				$submission = array(
					'name' => $this->post('quick_create_name'),
					'section_id' => $this->post('refiner__section'),
					'language_id' => $this->post('refiner__language'),
					'content_id' => 0,
					'target_loc' => 'none',
					'content_type' => '',
					'parent_id' => (int) $this->post('refiner__children'));
				
				saveMenuDetails($submission, $menuId);
				return $menuId;
			}
			
		
		} elseif ($this->post('change_parent') && ((checkPriv('_PRIV_EDIT_MENU_ITEM') && (int) $ids2) || (checkPriv('_PRIV_EDIT_TOP_LEVEL_MENU_ITEM') && !((int) $ids2)))) {
			foreach (explode(',', $ids) as $id) {
				$details = array(
					'mID' => (int) $id,
					'parent_id' => (int) $ids2,
					'parentMenuID' => (int) $this->post('refiner__children'),
					'section_id' => $this->post('refiner__section'));
				
				$validation = validateMenuDetails($details, $id);
				
				if (!$validation['valid']) {
					foreach ($validation['errArray'] as $error) {
						echo $error;
					}
					exit;
				}
			}
			
			foreach (explode(',', $ids) as $id) {
				$submission = array(
					'mID' => (int) $id,
					'parent_id' => (int) $ids2,
					'parentMenuID' => (int) $this->post('refiner__children'),
					'section_id' => $this->post('refiner__section'));
				
				saveMenuDetails($submission, $id);
			}
			
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_DELETE_MENU_ITEM')) {
			foreach (explode(',', $ids) as $id) {
				deleteMenuItem_recursive($id);
			}
		
		} elseif ($this->post('reorder') && checkPriv('_PRIV_SHUFFLE_MENU_ITEM')) {
			foreach (explode(',', $ids) as $id) {
				if ($this->post('item__'. $id)) {
					$sql = "
						UPDATE ". DB_NAME_PREFIX. "menuitems SET
							ordinal = ". (int) $this->post('item__'. $id). "
						WHERE id = ". (int) $id;
					my_mysql_query($sql);
				}
			}
		}
		
		break;
	
	
	case 'tribiq__content/content':
		if ($this->request('refiner__content_type')) {
			includeContentType(preg_replace('/\W/', '', $this->request('refiner__content_type')));
		}
		
		if ($this->post('add_to_menu') && checkPriv('_PRIV_ADD_MENU_ITEM')) {
			if ($menuDetails = getMenuItemFromMenuID($ids2)) {
				if ($menuDetails['target_loc'] == 'none'
				 || ($menuDetails['target_loc'] == 'int' && !$menuDetails['content_id'])
				 || ($menuDetails['target_loc'] == 'ext' && !$menuDetails['ext_url'])
				) {
					$submission = array(
						'content_id' => $ids,
						'target_loc' => 'int',
						'content_type' => $this->post('refiner__content_type'));
					saveMenuDetails($submission, $ids2);
				} else {
					echo getPhrase('Please select an unlinked Menu Item.');
				}
			}
		
		} elseif ($this->post('hide') && checkPriv('_PRIV_HIDE_CONTENT')) {
			foreach (explode(',', $ids) as $id) {
				hideContent($id, $this->request('refiner__content_type'));
			}
		
		} elseif ($this->post('trash') && checkPriv('_PRIV_TRASH_CONTENT')) {
			foreach (explode(',', $ids) as $id) {
				if (!checkIfAliasIsSystemAlias($id, $this->request('refiner__content_type'))) {
					trashContent($id, $this->request('refiner__content_type'));
				}
			}
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_DELETE_DRAFT')) {
			foreach (explode(',', $ids) as $id) {
				if (getLatestVersion($id, $this->request('refiner__content_type')) > 1 || !checkIfAliasIsSystemAlias($id, $this->request('refiner__content_type'))) {
					$version = deleteContent($id, $this->request('refiner__content_type'));
				}
				
				if ($this->get('frontend')) {
					if ($version) {
						echo json_encode(array("version" => $version));
					} else {
						$validParentMenuItemFound = false;
					
						if ($parentMenuId = getMenuParent($_GET['mID'])) {
							if ($contentItem = getContentFromMenu($parentMenuId)) {
								$validParentMenuItemFound = true;
							}
						}
						
						if ($validParentMenuItemFound) {
							echo json_encode($contentItem);
						} else {
							echo json_encode(array("home_alias" => getLanguageDefaultHomepage($_SESSION['user_lang'])));
						}
					}
				}
			}
		
		} elseif ($this->post('create_multiple') && checkPriv('_PRIV_CREATE_FIRST_DRAFT')) {
			if (includeContentType($cType = $GLOBALS['cType'] = $this->post('refiner__content_type'))) {
				$newIds = array();
				foreach (explode(',', $ids) as $filename) {
					$filename = decodeItemIdForStorekeeper($filename);
					$details = array(
						'current_file' => $filename,
						'selected_file' => $filename,
						'cID' => 0,
						'cType' => $cType,
						'cVersion' => 1);
					
					$valid = validateDropbox($details);
					if (!empty($valid['errArray'])) {
						echo getPhrase('There was a problem with the [[file]] file.', array('file' => $filename)), '<br/>';
						foreach($valid['errArray'] as $error) {
							echo $error, '<br/>';
						}
						echo '<br/>';
						continue;
					}
					
					$newDetails = setItemMeta($details['cID'], $details['cVersion'], $filename, $this->post('refiner__language'));
					$details['cID'] = $newIds[] = $newDetails['cID'];
					$details['cVersion'] = $newDetails['cVersion'];
					
					importContent($details);
				}
				
				return $newIds;
			}
		
		} elseif ($this->post('createDraft') && checkPriv('_PRIV_CREATE_REVISION_DRAFT')) {
			foreach (explode(',', $ids) as $id) {
				createPrivateDraft($id,getLatestVersion($id,$this->request('refiner__content_type')),getLatestVersion($id,$this->request('refiner__content_type')),$this->request('refiner__content_type'));
			}
		}
		
		break;
	
	
	case 'tribiq__content/categories':
		if ($this->post('delete') && checkPriv('_PRIV_DELETE_CATEGORY')) {
			foreach (explode(',', $ids) as $id) {
				$this->deleteCategory($id);
			}
		}
		
		break;
	
	
	case 'tribiq__content/media/images':
		if ($this->post('upload') && (
			($this->post('refiner__content_id') && checkPriv('_PRIV_ADD_NEW_IMAGE')) ||
			(!$this->post('refiner__content_id') && checkPriv('_PRIV_ADD_IMAGE_TO_ITEM'))
		)) {
			if ($this->post('refiner__content_id') && ($version = getLatestVersion($this->post('refiner__content_id'), $this->post('refiner__content_type')))) {
				$result = handleMediaUpload(
					'Filedata', getSiteConfig('max_content_image_filesize'),
					'storekeeper_populate', false, false,
					$this->post('refiner__content_id'), $this->post('refiner__content_type'), $version);
			} else {
				$result = handleMediaUpload(
					'Filedata', getSiteConfig('max_content_image_filesize'),
					'storekeeper_populate', false, false);
			}
			
			if (is_array($result)) {
				foreach ($result as $error) {
					echo $error. "\n";
				}
				return false;
			
			} else {
				echo 1;
				return $result;
			}
		
		} elseif ($this->post('add') && checkPriv('_PRIV_ADD_IMAGE_TO_ITEM') && $this->post('refiner__content_id') && $this->post('refiner__content_type')) {
			foreach (explode(',', $ids) as $checksum) {
				addMediaToContentItemIfNeeded('image', $checksum, $this->post('refiner__content_id'), $this->post('refiner__content_type'));
			}
			return $ids;
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_DELETE_IMAGE')) {
			foreach (explode(',', $ids) as $checksum) {
				deleteImageByCheckSum($checksum, $this->post('refiner__content_id'), $this->post('refiner__content_type'));
			}
		
		} elseif ($this->post('make_sticky') && checkPriv('_PRIV_SET_ITEM_STICKY_IMAGE')) {
			$version = getLatestVersion($this->post('refiner__content_id'), $this->post('refiner__content_type'));
			
			$sql = "UPDATE " . DB_NAME_PREFIX . "images
					SET sticky_flag = 0
					WHERE content_item_id = " . (int) $this->post('refiner__content_id') . "
						AND content_item_version = " . (int) $version . "
						AND content_item_type = '" . mysql_real_escape_string($this->post('refiner__content_type')) . "'";
			my_mysql_query($sql);
			
			$sql = "UPDATE " . DB_NAME_PREFIX . "images
					SET sticky_flag = 1
					WHERE content_item_id = " . (int) $this->post('refiner__content_id') . "
						AND content_item_version = " . (int) $version . "
						AND content_item_type = '" . mysql_real_escape_string($this->post('refiner__content_type')) . "'
						AND checksum_inc_name = '" . mysql_real_escape_string($ids) . "'";
			my_mysql_query($sql);
		}
		
		break;
	
	
	case 'tribiq__content/media/movies':
		if ($this->post('upload') && (
			($this->post('refiner__content_id') && checkPriv('_PRIV_ADD_NEW_MOVIE')) ||
			(!$this->post('refiner__content_id') && checkPriv('_PRIV_ADD_MOVIE_TO_ITEM'))
		)) {
			
			if ($this->post('refiner__content_id') && ($version = getLatestVersion($this->post('refiner__content_id'), $this->post('refiner__content_type')))) {
				$result = handleMediaUpload(
					'Filedata', getSiteConfig('max_content_movie_filesize'),
					'storekeeper_populate', false, true,
					$this->post('refiner__content_id'), $this->post('refiner__content_type'), $version);
			} else {
				$result = handleMediaUpload(
					'Filedata', getSiteConfig('max_content_movie_filesize'),
					'storekeeper_populate', false, true);
			}
			
			if (is_array($result)) {
				foreach ($result as $error) {
					echo $error. "\n";
				}
				return false;
			
			} else {
				echo 1;
				return $result;
			}
		
		} elseif ($this->post('add') && checkPriv('_PRIV_ADD_MOVIE_TO_ITEM') && $this->post('refiner__content_id') && $this->post('refiner__content_type')) {
			foreach (explode(',', $ids) as $checksum) {
				addMediaToContentItemIfNeeded('movie', $checksum, $this->post('refiner__content_id'), $this->post('refiner__content_type'));
			}
			return $ids;
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_DELETE_MOVIE')) {
			foreach (explode(',', $ids) as $checksum) {
				deleteMovieByCheckSum($checksum, $this->post('refiner__content_id'), $this->post('refiner__content_type'));
			}
		}
		
		break;
		
		
	case 'tribiq__templates_and_skins/templates/template_files/templates':
		if ($this->post("mode")=="delete_template") {
			deleteTemplate($_POST['id'],$_POST['family_name']);
		}
		
		break;	
		
		
	case 'tribiq__templates_and_skins/templates/skin_files':
		switch ($this->post("action")){
			case 'register_skin':
				if (validateSkinFromXMLDescriptionByName(decodeItemIdForStorekeeper($_POST['refiner__template_family']),decodeItemIdForStorekeeper($_POST['id']),$desc)) {
					if (isset($desc->extension_of_skin)
					 && !getRow('skins', array('id'), array('family_name'=>decodeItemIdForStorekeeper($_POST['refiner__template_family']),'name'=>$desc->extension_of_skin))) {
						echo
							'<!--Message_Type:Error-->',
						getPhrase('Cannot register the Skin "[[skin_1]]" as its dependent Skin "[[skin_2]]"  is not registered.',
									array(	'skin_1'=>decodeItemIdForStorekeeper($_POST['id']),
											'skin_2'=>$desc->extension_of_skin )
								 );
					} else {
						insertSkin(decodeItemIdForStorekeeper($_POST['refiner__template_family']), decodeItemIdForStorekeeper($_POST['id']), $desc);	
					}
				}
				
				break;
			
			
			case 'unregister_skin':
				$skinDependencies = array();
				if ($registeredSkins = getSkins(decodeItemIdForStorekeeper($_POST["refiner__template_family"]))){
					foreach($registeredSkins as $registeredSkin){
						if (validateSkinFromXMLDescriptionByName(decodeItemIdForStorekeeper($registeredSkin['family_name']),$registeredSkin['name'],$desc)){
							if ($desc->extension_of_skin!=''){
								$skinDependencies[(string)$desc->extension_of_skin][] = $registeredSkin['name'];
							}
						}
					}
				}

				if (issetArrayKey($skinDependencies,decodeItemIdForStorekeeper($_POST['id']))){
							echo
								'<!--Message_Type:Error-->',
							getPhrase('Cannot de-register the Skin "[[skin_1]]" as its dependent Skin(s) "[[skin_2]]" is/are registered.',
										array(	'skin_1'=>decodeItemIdForStorekeeper($_POST['id']),
												'skin_2'=>implode(", ",arrayKey($skinDependencies,decodeItemIdForStorekeeper($_POST['id']))) 
											 )
									 );
				} else {
					deleteSkin(decodeItemIdForStorekeeper($_POST['refiner__template_family']),decodeItemIdForStorekeeper($_POST['id']),$desc);
				}
				break;
		}
		break;
	case 'tribiq__plugins':
		
		if (!$plugin = getPluginDetails($ids)) {
			echo getPhrase('Plugin not found!');
			exit;
		}
		
		$reload = false;
		if ($this->post('initialize') && checkPriv('_PRIV_INITIALISE_PLUGIN') && $plugin['status'] == '_ENUM_INSTALLED') {
			if (($initialise_status = initialisePlugin($ids)) != 'success') {
				echo $initialise_status;
			} else {
				$plugin = getPluginDetails($ids);
				$langs = getLanguages();
				if ($plugin['vlp_class'] && $plugin['vlp_class'] == $plugin['class_name'] && !empty($langs)) {
					$phrasePacks = scanPluginPhraseDir($plugin['plugin_name']);
					if (!empty($phrasePacks)) {
						echo
							'<!--Message_Type:Question-->',
							'<!--Button_HTML:',
								'<input class="submit_selected" value="', getPhrase("Allow"), '" onclick="tribiqSK.itemButtonClick(\'import_phrases\');" type="button"> ',
								'<input value="', getPhrase("Don't Allow"), '" type="button">',
							'-->',
							getPhrase(
								'The Plugin "[[name]]" wishes to add its Phrases to your Language Pack(s). We recommend you allow this.',
								array('name' => $plugin['display_name']));
					}
				}
			}
		
		} elseif ($this->post('run_first_time') && checkPriv('_PRIV_RUN_PLUGIN_FIRST_TIME') && $plugin['status'] == '_ENUM_INITIALISED') {
			$reload = true;
			runPlugin($ids);
		
		} elseif ($this->post('run') && checkPriv('_PRIV_RUN_PLUGIN') && $plugin['status'] == '_ENUM_SUSPENDED') {
			$reload = true;
			runPlugin($ids);
		
		} elseif ($this->get('upgrade') && checkPriv('_PRIV_MANAGE_PLUGIN_INSTANCE') && $plugin['status'] == '_ENUM_RUNNING') {
			if (!count($instances = succeedPlugin($ids, $preview = true))) {
				echo
					'<p>', getPhrase(
						'There are no Instances availible for the Plugin &quot;[[pluginName]]&quot; to Upgrade.',
						array('pluginName' => htmlspecialchars($plugin['display_name']))
					), '</p>';
			
			} else {
				echo
					'<p>', getPhrase(
						'The following Plugin Instances will be Upgraded to the &quot;[[pluginName]]&quot; Plugin:',
						array('pluginName' => htmlspecialchars($plugin['display_name']))
					), '</p>';
				
				echo '<ul>';
				foreach($instances as $instance) {
					echo '<li>', htmlspecialchars($instance), '</li>';
				}
				echo '</ul>';
			}
		
		} elseif ($this->post('upgrade') && checkPriv('_PRIV_MANAGE_PLUGIN_INSTANCE') && $plugin['status'] == '_ENUM_RUNNING') {
			succeedPlugin($ids);
		
		} elseif ($this->post('suspend') && checkPriv('_PRIV_SUSPEND_PLUGIN') && $plugin['status'] == '_ENUM_RUNNING') {
			$reload = true;
			suspendPlugin($ids);
		
		} elseif ($this->post('uninstall') && checkPriv("_PRIV_UNINSTALL_PLUGIN") && ($plugin['status'] == '_ENUM_INITIALISED' || $plugin['status'] == '_ENUM_SUSPENDED')) {
			uninstallPlugin($ids);
		}
		
		//Send a command to reload Storekeeper, if the XML map may have changed.
		if ($reload && is_dir($plugin['path']. 'storekeeper')) {
			echo '<!--Reload_Storekeeper-->';
		}
		
		break;

	
	case 'tribiq__plugins/instances':
		if ($this->post('delete') && checkPriv('_PRIV_MANAGE_PLUGIN_INSTANCE')) {
			foreach (explode(',', $ids) as $id) {
				$usage = checkInstancesUsage($id);
				if (!$usage['count']) {
					deletePluginInstance($id);
				}
			}
		}
		
		break;

	
	case 'tribiq__languages':
		if ($this->get('export') && checkPriv('_PRIV_EXPORT_VLP')) {
			
			$phrases = array();
			$sql = "
				SELECT COUNT(*)
				FROM (
					SELECT DISTINCT code, plugin_class
					FROM ". DB_NAME_PREFIX. "visitor_phrases
				) AS c";
			$result = my_mysql_query($sql);
			list($phrases['total']) = mysql_fetch_row($result);
			
			$sql = "
				SELECT COUNT(*)
				FROM (
					SELECT DISTINCT code, plugin_class
					FROM ". DB_NAME_PREFIX. "visitor_phrases
					WHERE language_id = '". mysql_real_escape_string($ids). "'
				) AS c";
			$result = my_mysql_query($sql);
			list($phrases['present']) = mysql_fetch_row($result);
			$phrases['missing'] = $phrases['total'] - $phrases['present'];
			
			echo '
				<p>',
					getPhrase(
						"This will export all of the Phrases in the Language Pack &quot;[[lang]]&quot;.",
						array('lang' => htmlspecialchars(getLanguageName($ids)))
					),
				'</p>
				<table>';
			
			if ($ids != DEFAULT_LANGUAGE) {
				echo '
				<tr><td class="label">
					<label for="te_reference_text">', getPhrase('Reference text:'), '</label>
				</td><td>
					<input id="te_reference_text" name="reference_text" type="checkbox" value="1"/>
					<label for="te_reference_text">',
						getPhrase(
							'Include Phrases from &quot;[[lang]]&quot; as a reference',
							array('lang' => htmlspecialchars(getLanguageName(DEFAULT_LANGUAGE)))
						),
					'</label>
				</td></tr>';
			}
			
			echo '
				<tr><td class="label">
					<label class="label" for="te_phrases">', getPhrase('Phrases to export:'), '</label>
				</td><td>
					<select id="te_phrases" name="phrases">
						<option value="present">', getPhrase('Only existing Phrases ([[present]])', $phrases), '</option>
						<option value="missing">', getPhrase('Only missing Phrases ([[missing]])', $phrases), '</option>
						<option value="all">', getPhrase('All possible Phrases ([[total]])', $phrases), '</option>
					</select>
				</td></tr></table>';
		
		} elseif ($this->post('import') && checkPriv('_PRIV_IMPORT_VLP')) {
			$languageId = $ids;
			$numberOf = importVisitorLanguagePack($_FILES['Filedata']['tmp_name'], $languageId, false);
			$this->languageImportResults($numberOf);
			return $languageId;
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_DELETE_VLP')) {
			$sql = "
				DELETE
				FROM " . DB_NAME_PREFIX . "visitor_phrases
				WHERE language_id = '" . mysql_real_escape_string($_POST['id']) . "'
				  AND '" . mysql_real_escape_string($_POST['id']) . "' NOT IN (
					SELECT id
					FROM " . DB_NAME_PREFIX . "languages
				)";
			my_mysql_query($sql);
		}
		
		break;

	
	case 'tribiq__languages/vlp':
		if ($this->post('delete_phrase') && checkPriv('_PRIV_DELETE_VLP')) {
			foreach (explode(',', $ids) as $id) {
				deleteRow('visitor_phrases', array('id' => $id));
			}
		}
		
		break;

	
	case 'tribiq__languages/remote_store':
		
		if ($this->post('download') && checkPriv('_PRIV_IMPORT_VLP')) {
			$dummy = array();
			$this->CURLLanguageFetch($dummy, 3, $refinerId, $ids);
		}
		
		break;

	
	case 'tribiq__users':
		if ($this->post('delete_user') && checkPriv('_PRIV_DELETE_USER')) {
			foreach (explode(',', $ids) as $id) {
				deleteUser($id);
			}
		
		} elseif ($this->post('remove_users_from_group') && checkPriv('_PRIV_MANAGE_GROUP_MEMBERSHIP')) {
			foreach (explode(',', $ids) as $id) {
				addUserToGroup($id, $refinerId, $remove = true);
			}
		
		} elseif ($this->post('add_users_to_group') && checkPriv('_PRIV_MANAGE_GROUP_MEMBERSHIP')) {
			foreach (explode(',', $ids) as $userId) {
				foreach (explode(',', $ids2) as $groupId) {
					addUserToGroup($userId, $groupId);
				}
			}
		
		} elseif ($this->post('add_user_to_group') && $this->post('refiner__group_members') && checkPriv('_PRIV_MANAGE_GROUP_MEMBERSHIP')) {
			foreach (explode(',', $ids) as $userId) {
				addUserToGroup($userId, $this->post('refiner__group_members'));
			}
		}
		
		break;
	
	
	case 'tribiq__users/groups':
		if ($this->post('delete_group') && checkPriv('_PRIV_DELETE_GROUP')) {
			foreach (explode(',', $ids) as $id) {
				deleteGroup($id);
			}
		
		} elseif ($this->post('remove_users_from_group') && checkPriv('_PRIV_MANAGE_GROUP_MEMBERSHIP')) {
			foreach (explode(',', $ids) as $id) {
				addUserToGroup($refinerId, $id, $remove = true);
			}
		
		} elseif ($this->post('add_users_to_group') && checkPriv('_PRIV_MANAGE_GROUP_MEMBERSHIP')) {
			foreach (explode(',', $ids) as $groupId) {
				foreach (explode(',', $ids2) as $userId) {
					addUserToGroup($userId, $groupId);
				}
			}
		
		} elseif ($this->post('add_user_to_group') && $this->post('refiner__users_groups') && checkPriv('_PRIV_MANAGE_GROUP_MEMBERSHIP')) {
			foreach (explode(',', $ids) as $groupId) {
				addUserToGroup($this->post('refiner__users_groups'), $groupId);
			}
		}
		
		break;
		
		
	case 'tribiq__administration':
		if ($this->post('site_reset') && checkPriv('_PRIV_RESET_SITE')) {
			resetSite($error);
			echo '<!--Reload_Storekeeper-->';
		}
		
		break;
		
		
	case 'tribiq__administration/admins/local_admins':
		if ($this->post('trash') && checkPriv('_PRIV_DELETE_ADMIN')) {
			foreach (explode(',', $ids) as $id) {
				deleteAdmin($id, 'local');
			}
		
		} elseif ($this->post('restore') && checkPriv('_PRIV_DELETE_ADMIN')) {
			foreach (explode(',', $ids) as $id) {
				deleteAdmin($id, 'local', true);
			}
		}
		
		break;
		
		
	case 'tribiq__administration/spare_urls':
		if ($this->post('delete') && checkPriv('_PRIV_MANAGE_SPARE_DOMAIN_NAME')) {
			connectGlobalDB();
				$sql = "
					DELETE FROM ". DB_NAME_PREFIX_GLOBAL . "spare_urls
					WHERE sitename = '" . mysql_real_escape_string(SITENAME) . "'
					  AND requested_url = '". mysql_real_escape_string(decodeItemIdForStorekeeper($ids)). "'";
				my_mysql_query($sql);
			connectLocalDB();
		}
		
		break;


	case 'tribiq__administration/backups':
		//Check to see if we can proceed
		if ($errors = initialiseBackupFunctions(false)) {
			exit;
		}
		
		if ($ids) {
			$filename = CMS_BACKUP_DIR. '/'. SITENAME . '/'. decodeItemIdForStorekeeper($ids);
		}
		
		if ($this->post('create') && checkPriv('_PRIV_BACKUP_SITE')) {
			//Create a new file in the backup directory, and write the backup into it
			$backupPath = CMS_BACKUP_DIR. '/'. SITENAME. '/' . generateFilenameForBackups();
			
			$g = gzopen($backupPath, 'wb');
			createDatabaseBackupScript($g);
			gzclose($g);
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_RESTORE_SITE')) {
			unlink($filename);
		
		} elseif ($this->post('upload_and_restore') && checkPriv('_PRIV_RESTORE_SITE')) {
			//Call restoreDatabaseFromBackup on the temp file
			if (restoreDatabaseFromBackup(
							$_FILES['Filedata']['tmp_name'],
							//Attempt to check whether gzip compression has not been used
							strtolower(substr($_FILES['Filedata']['name'], -3)) != '.gz',
							DB_NAME_PREFIX, $failures
			)) {
				echo '<!--Reload_Storekeeper-->';
			} else {
				foreach ($failures as $text) {
					echo $text;
				}
			}
		
		} elseif ($this->post('restore') && checkPriv('_PRIV_RESTORE_SITE')) {
			//Restore a backup from the file system
			if (restoreDatabaseFromBackup(
					$filename,
					//Attempt to check whether gzip compression has been used, or if this is a plain sql file
					strtolower(substr($filename, -3)) != '.gz',
					DB_NAME_PREFIX, $failures
			)) {
				echo '<!--Reload_Storekeeper-->';
			} else {
				foreach ($failures as $text) {
					echo $text;
				}
			}
		}
		
		
		break;


	case 'tribiq__content/dropbox':
		
		if ($this->post('delete') && checkPriv('_PRIV_DELETE_FROM_DROPBOX')) {
			foreach (explode(',', $ids) as $id) {
				unlink(CMS_DROPBOX_DIR. '/'. SITENAME . '/'. decodeItemIdForStorekeeper($id));
			}
		
		} elseif ($this->post('upload') && checkPriv('_PRIV_UPLOAD_ITEM_FILE')) {
			require_once CMS_ROOT. 'cmsincludes/common_contenttype.inc.php';
			
			$fileExtension = preg_replace("#^.+\\.([^.]+)$#", "\\1", $_FILES['Filedata']['name']);
			if (!documentTypeIsAllowed($fileExtension)) {
				echo getPhrase('_FILE_UPLOAD_ERROR_INVALID_TYPE');
				exit;
			}
			
			$error = arrayKey($_FILES, 'Filedata', 'error');
			switch ($error) {
				case UPLOAD_ERR_INI_SIZE:
				case UPLOAD_ERR_FORM_SIZE:
					echo getPhrase('Your file was too large to be uploaded.');
					exit;
				case UPLOAD_ERR_PARTIAL:
				case UPLOAD_ERR_NO_FILE:
					echo getPhrase('There was a problem whilst uploading your file.');
					exit;
				case UPLOAD_ERR_NO_TMP_DIR:
					echo 'UPLOAD_ERR_NO_TMP_DIR';
					exit;
				case UPLOAD_ERR_CANT_WRITE:
					echo 'UPLOAD_ERR_CANT_WRITE';
					exit;
				case UPLOAD_ERR_EXTENSION:
					echo 'UPLOAD_ERR_EXTENSION';
					exit;
			}
			
			move_uploaded_file($_FILES['Filedata']['tmp_name'], CMS_DROPBOX_DIR . "/" . SITENAME . "/" . ($filename = tidyFilename($_FILES['Filedata']['name'])));
			echo 1;
			return encodeItemIdForStorekeeper($filename);
		}
		
		break;
	
	
	case 'tribiq__administration/favicon':
		if ($this->post('upload') && checkPriv('_PRIV_EDIT_SITE_SETTING')) {
			$result = handleMediaUpload('Filedata', getSiteConfig('max_content_image_filesize'), 'favicon', false, false);
			
			if (is_array($result)) {
				foreach ($result as $error) {
					echo $error. "\n";
				}
				return false;
			
			} else {
				echo 1;
				return $result;
			}
		
		} elseif ($this->post('delete') && checkPriv('_PRIV_EDIT_SITE_SETTING')) {
			$sql = "
				DELETE FROM ". DB_NAME_PREFIX. "favicons
				WHERE `usage` = '". mysql_real_escape_string($ids). "'";
			my_mysql_query($sql);
		}
		
		break;
		
		
	case 'tribiq__administration/urls':
		if ($this->post('delete') && checkPriv('_PRIV_DELETE_URL')) {
			deleteSiteRegistrySitePublicURL(SITENAME, decodeItemIdForStorekeeper($ids));
		}
		
		break;
		
		
	default:
		print_r($this->getVariables);
		print_r($this->postVariables);
}

return false;
?>