<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

class tribiq_content_list extends plugin_base_class {
	protected $contentType;
	protected $dataField;
	protected $year;
	protected $years;
	protected $pages;
	protected $totalPages;
	protected $rows = false;
	protected $items = false;
	protected $sql;
	
	
	function init() {
		
		if ($this->setting('data_field') == 'description') {
			$this->dataField = 'description';
		} elseif ($this->setting('data_field') == 'content_bodytop') {
			$this->dataField = 'content_bodytop';
		} elseif ($this->setting('data_field') == 'content_bodymain') {
			$this->dataField = 'content_bodymain';
		} elseif ($this->setting('data_field') == 'content_bodyfoot') {
			$this->dataField = 'content_bodyfoot';
		} elseif ($this->setting('data_field') == 'content_summary') {
			$this->dataField = 'content_summary';
		} else {
			$this->dataField = false;
		}
		
		$this->registerGetRequest('page', 1);
		$this->registerGetRequest('year');
		
		//If we are filtering by year, pick a year to display
		if ($this->setting('filter_by_year') && !($this->year = $this->get('year'))) {
			 list($this->year) = mysql_fetch_row(my_mysql_query("SELECT YEAR(NOW())"));
		}
		
		//Pick a page to display
		$this->page = is_numeric($this->get('page'))? (int) $this->get('page') : 1;
		$this->start = ($this->page-1) * $this->setting('page_size');
		
		
		//Loop through each item to display, and add its details to an array of merge fields
		$this->items = array();
		if ($result = $this->lookForContent()) {
			while($row = mysql_fetch_assoc($result)) {
				
				if (!($row['link'] = $row['alias'])) {
					$row['link'] = $row['id'];
				}
				
				$item = array();
				
				if ($this->dataField == 'description') {
					$item['Excerpt_Text'] = $row['content_table_data'];
					if (arrayKey($_REQUEST, 'method_call') == 'showRSS') {
						$item['Excerpt_Text'] = XMLEscape($item['Excerpt_Text']);
					} else {
						$item['Excerpt_Text'] = htmlspecialchars($item['Excerpt_Text']);
					}
				
				} elseif ($this->dataField == 'content_summary' && arrayKey($_REQUEST, 'method_call') != 'showRSS') {
					$item['Excerpt_Text'] = $row['content_table_data'];
				
				} elseif ($this->dataField) {
					$item['Excerpt_Text'] = $row['content_table_data'];
					
					if ($this->setting('excerpt_length') == 'preserve' && arrayKey($_REQUEST, 'method_call') != 'showRSS') {
						if ($this->setting('strip_titles_from_excerpts')) {
							$item['Excerpt_Text'] = preg_replace('@<h\d>.*?</h\d>@is', '', $item['Excerpt_Text']);
						}
					} else {
						$excerptLength = $this->setting('excerpt_length') == 'preserve'? 100000 : $this->setting('excerpt_length');
						$this->displayHTMLAsPlainText($item['Excerpt_Text'], $excerptLength, $this->setting('strip_titles_from_excerpts'));
					}
					
					//Attempt to tidy the excerpt a little for the RSS feed
					if (arrayKey($_REQUEST, 'method_call') == 'showRSS') {
						$item['Excerpt_Text'] = XMLEscape(html_entity_decode(strip_tags($item['Excerpt_Text']), ENT_QUOTES, 'UTF-8'));
					}
				}
				
				$item['No'] = $this->start + 1;
				$item['Id'] = $row['id']. '_'. $row['content_type'];
				$item['Link'] = $this->linkToItemAnchor($row['link'], $row['content_type']);
				$item['Full_Link'] = htmlspecialchars($this->linkToItem($row['link'], $row['content_type'], true));
				$item['Content_Type'] = $row['content_type'];
				$item['Title'] = htmlspecialchars($row['title']);
				$item['Keywords'] = htmlspecialchars($row['keywords']);
				$item['Description'] = htmlspecialchars($row['description']);
				
				if ($this->setting('show_dates')) {
					$item['Date'] = formatDateNicely(
						$row['content_table_date'],
						$this->setting('date_format'),
						(bool) $this->setting('show_times'),
						arrayKey($_REQUEST, 'method_call') == 'showRSS');
				}
				
				if ($this->setting('show_sticky_images') && ($image = $this->itemStickyImage($row['id'], $row['content_type'], $row['version']))) {
					$item['Sticky_Image'] = '<img class="sticky_image" alt="'. $item['Title']. '" src="'. htmlspecialchars($image). '"';
					
					$setWidth = $this->setting('set_width') && (int) $this->setting('width');
					$setHeight = $this->setting('set_height') && (int) $this->setting('height');
					
					if ($setWidth || $setHeight) {
						$item['Sticky_Image'] .= 'style="';
						if ($setWidth) {
							$item['Sticky_Image'] .= 'width: '. (int) $this->setting('width'). 'px;';
						}
						if ($setHeight) {
							$item['Sticky_Image'] .= ' height: '. (int) $this->setting('height'). 'px;';
						}
						$item['Sticky_Image'] .= '"';
					}
					
					$item['Sticky_Image'] .= '/>';
				}
				
				$this->addExtraMergeFields($row, $item);
				
				$this->items[] = $item;
			}
		}
		
		return true;
	}
	
	function addExtraMergeFields(&$row, &$item) {
	
	}
	
	
	//Returns a list of fields needed by the Plugin
	//Intended to be easily overwritten
	function lookForContentSelect() {
		return "
			SELECT
				c.id,
				c.title,
				c.keywords,
				c.description,
				". ($this->contentType == 'html'? "c.alias" : "'' AS alias"). ",
				'" . mysql_real_escape_string($this->contentType) . "' AS `content_type`,
				c.version,
				c.publisher_id,
				c.publisher_authtype,
				IFNULL(IFNULL(c.publication_date, c.published_datetime), c.created_datetime) AS `content_table_date`,
				". ifNull($this->dataField, "''"). " AS `content_table_data`";
	}
	
	//Returns the table to select from
	//Intended to be easily overwritten
	function lookForContentFrom() {
		
		//Allow any content type to be listed
		$this->contentType = $this->setting('content_type');
		
		$sql = "
			FROM ". DB_NAME_PREFIX. "content_". mysql_real_escape_string($this->contentType). " c ";
		
		
		//refiner by a category if requested
		if ($this->setting('category') && checkRowExists('categories', array('id' => (int) $this->setting('category')))) {
			$sql .= "
			WHERE (c.id, c.version) IN (
				SELECT item_id, version
				FROM ". DB_NAME_PREFIX. "category_item_link
				WHERE category_id = ". (int) $this->setting('category'). "
				  AND content_type = '". mysql_real_escape_string($this->contentType). "'
			  )";
		} else {
			$sql .= "
			WHERE 1=1";
		}
		
		$sql .= "
			  AND c.language_id = '". mysql_real_escape_string($_SESSION['user_lang']). "'";
		
		return $sql;
	}	
	//Returns a SQL statement that should identify each content item in the list
	//Intended to be easily overwritten
	function lookForContentSQL() {
		
		$sql =
			$this->lookForContentFrom().
			$this->sqlToSearchContentTable($this->contentType, $this->setting('hide_private_items'), $this->setting('only_show'));
		
		
		if (!$this->checkContentTypeExists($this->contentType)) {
			return false;
		}
		
		//Exclude this page itself
		if ($this->cType == $this->contentType) {
			$sql .= "
			  AND c.id != ". (int) $this->cID;
		}
		
		return $sql;
	}
	
	//Runs the SQL statement that will return a list of content items
	function lookForContent() {
		
		$sql = $this->lookForContentSQL();
		
		if ($sql === false) {
			return false;
		}
		
		//Add options to group by year and filter by the current year if asked to
		if ($this->setting('filter_by_year')) {
			$this->populateYearSelectList($sql);
		}
		
		//Get a count of how many items we have to display
		$result = my_mysql_query('SELECT COUNT(*) '. $sql);
		list($this->rows) = mysql_fetch_row($result);
		
		$this->totalPages = (int) ceil($this->rows / $this->setting('page_size'));
		
		//Loop through each page to display, and add its details to an array of merge fields
		$this->pages = array();
		for ($i = 1; $i <= $this->setting('page_limit') && $i <= $this->totalPages; ++$i) {
			$this->pages[$i] = '&page='. $i. ($this->year? '&year='. $this->year :null);
		}
		
		$sql =
			$this->lookForContentSelect().
			$sql.
			$this->orderContentBy(). "
			LIMIT ". (int) $this->start. ", ". (int) $this->setting('page_size');
		
		return my_mysql_query($sql);
	}
	
	function orderContentBy() {
		
		if ($this->setting('order') == 'Alphabetically') {
			return "
			ORDER BY c.title";
		
		} elseif ($this->setting('order') == 'Oldest first') {
			return "
			ORDER BY `content_table_date`, c.id";
		
		} else {
			return "
			ORDER BY `content_table_date` DESC, c.id DESC";
		}
	
	}
	
	
	function populateYearSelectList(&$sql) {
		//Given the conditions above, get a list of dates availible for display
		$sql2 = "
			SELECT DISTINCT YEAR(IFNULL(IFNULL(c.publication_date, c.published_datetime), c.created_datetime))
			". $sql;
		
		if ($this->setting('order') == 'Most recent first') {
			$sql2 .= "
				ORDER BY 1 DESC";
		} else {
			$sql2 .= "
				ORDER BY 1";
		}
		
		$result = my_mysql_query($sql2);
		
		//Loop through each year to display, and add its details to an array of merge fields
		$this->years = array();
		while($row = mysql_fetch_row($result)) {
			if ($row[0]) {
				$this->years[$row[0]] = '&year='. $row[0];
			}
		}
		
		//If there is nothing for the selected year, default to the first year in the select lsit
		if (!isset($this->years[$this->year])) {
			foreach ($this->years as $year => $dummy) {
				$this->year = $year;
				break;
			}
		}
		
		$sql .= "
		  AND YEAR(IFNULL(IFNULL(c.publication_date, c.published_datetime), c.created_datetime)) = ". (int) $this->year;
	}



	protected final function checkContentTypeExists() {
		return checkRowExists('contenttypes', array('content_type_id' => $this->contentType));
	}

	
	
	function addToPageHead() {
		if ($this->setting('enable_rss')) {
			echo '
				<link
					rel="alternate"
					type="application/rss+xml"
					href="'. htmlspecialchars($this->showRSSLink()). '"
					title="'. htmlspecialchars(getItemTitle($this->cID, $this->cType, $this->cVersion)). '" />';

		}
	}
	
	
	function showSlot() {
		
		$moreLink = false;
		if ($this->setting('show_more_link') && $this->getCIDAndCTypeFromSetting($cID, $cType, 'more_hyperlink_target')) {
			if ($this->inc('tribiq_avls')) {
				tribiq_avls::equivalentItem($cID, $cType);
			}
			$moreLink = $this->linkToItemAnchor($cID, $cType);
		}
		
		if ($this->setting('show_pagination') && count($this->pages) > 1) {
			$this->pagination('pagination_style', $this->page, $this->pages, $pagination);
		} else {
			$pagination = false;
		}
		
		if ($this->setting('filter_by_year') && count($this->years)) {
			$this->pagination('year_selector_style', $this->year, $this->years, $yearPagination);
		} else {
			$yearPagination = false;
		}

		$this->framework(
			'Slot', 
			array(
				'More_Link' => $moreLink,
				'More_Link_Title' => $moreLink? $this->phrase($this->setting('more_link_text')): null,
				'Pagination' => $pagination,
				'Results' => $this->rows,
				'Year' => $this->year,
				'RSS_Link' => $this->setting('enable_rss')? htmlspecialchars($this->showRSSLink()) : null,
				'Title_With_Content' => $this->setting('show_headings')? $this->phrase($this->setting('title_with_content')): null,
				'Title_With_No_Content' => $this->setting('show_headings')? $this->phrase($this->setting('title_without_content')): null,
				'Year' => $yearPagination
			),
			array(
				'More' => (bool) $moreLink,
				'No_Rows' => !count($this->items),
				'Rows' => count($this->items),
				'Row' => $this->items,
				'Show_Date' => $this->setting('show_dates'),
				'Show_Excerpt' => (bool) $this->dataField,
				'Show_Item_Title' => $this->setting('show_titles'),
				'Show_Sticky_Image' => (bool) $this->setting('show_sticky_images'),
				'Show_RSS_Link' => (bool) $this->setting('enable_rss'),
				'Show_Title' => $this->setting('show_headings') && $this->setting('title_with_content'),
				'Show_No_Title' => $this->setting('show_headings') && $this->setting('title_without_content'),
				'Years' => (bool) $yearPagination
			)
		);
		
	}
	
	
	function showRSS() {
		
		$this->framework(
			'RSS', 
			array(
				'Description' => XMLEscape(getItemDescription($this->cID, $this->cType, $this->cVersion)),
				'Link' => XMLEscape($this->linkToItem($this->cID, $this->cType, true)),
				'RSS_Link' => XMLEscape($this->showRSSLink()),
				'Title' => XMLEscape(getItemTitle($this->cID, $this->cType, $this->cVersion)),
				'Results' => $this->rows,
				'Year' => $this->year
			),
			array(
				'Rows' => true,
				'RSS_Item' => $this->items
			)
		);
	
	}
	
	
}
?>
