<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

class tribiq_content extends plugin_base_class {
	
	var $content;
	var $status;
	var $mergeFields = array();
	var $sections = array();
	
	function handleAJAX() {
		
		//Handle submitting revisions by AJAX
		if ($this->post('cID') && $this->post('cType') && $this->post('cVersion') && $this->post('content__section')) {
			//Get the current status of the content item
			$this->status = getContentStatus($this->post('cID'), $this->post('cType'), $this->post('cVersion'));
			
			//Include admin mode fuctionality if the user is in admin mode
			//Special case for all content plugins
			//Updates to the content areas of private drafts
			if (checkPriv('_PRIV_EDIT_DRAFT')) {
				if ($this->status == 'private_draft') {
					saveItemContent($this->post('cID'), $this->post('cVersion'), $this->post('content__section'), $this->post('cType'), $this->postVariables['content__content'], $_SESSION['admin_userid'], $_SESSION['admin_authtype']);
				} else {
					echo getPhrase('Your changes were not saved, as this version has already been Published.');
				}
			} else {
				echo getPhrase('Your changes were not saved, as you have been logged out.');
			}
			
			exit;
		}
		
		if (checkPriv()) {
			//Create a private draft
			if ($this->post('create_draft') && checkPriv('_PRIV_EDIT_DRAFT')) {
				if (($this->latestVersion = getLatestVersion($this->post('cID'), $this->post('cType')))
				 && ($this->latestVersionStatus = getContentStatus($this->post('cID'), $this->post('cType')))) {
					if (($this->latestVersionStatus == 'published') || ($this->latestVersionStatus == 'hidden') || ($this->latestVersionStatus == 'archived')) {
				 		if (includeContentType($this->post('cType')) && createPrivateDraft($this->post('cID'), $this->post('cVersion'), $this->latestVersion)) {
				 			$_SESSION['tribiq_content__edit_section'] = $this->post('cID'). '_'. $this->post('cType'). '_'. $this->post('section');
				 		} else {
				 			echo getPhrase('Sorry, this content could not be edited');
				 		}
					} else {
						echo getPhrase('_SORRY_CONTENT_VERSION_NOT_EDITABLE_PLEASE_SELECT_OTHER_VERSION');
					}
				}
			}
		}
	}
	
	//When the plugin is set up, also get the content item's status and the content section to display
	function init() {
		
		if ($this->setting('section') == 'content_bodytop') {
			$this->section = 'content_bodytop';
		} elseif ($this->setting('section') == 'content_bodyfoot') {
			$this->section = 'content_bodyfoot';
		} else {
			$this->section = 'content_bodymain';
		}
		
		//Get the current status of the content item in admin mode, to check for actions the admin can take
		if (checkPriv()) {
			$this->status = getContentStatus($this->cID, $this->cType, $this->cVersion);
			$this->latestVersion = getLatestVersion($this->cID, $this->cType);
			$this->latestVersionStatus = getContentStatus($this->cID, $this->cType, $this->latestVersion);
		}
		
		//Get the current contents of the section
		$sql = "
			SELECT
				title,
				". mysql_real_escape_string($this->section). " AS `content`,
				IFNULL(IFNULL(publication_date, published_datetime), created_datetime) AS `date`
			FROM ". DB_NAME_PREFIX. "content_". mysql_real_escape_string($this->cType). "
			WHERE id = ". (int) $this->cID. "
			  AND version = ". (int) $this->cVersion;
		
		$result = my_mysql_query($sql);
		list($this->mergeFields['Title'], $this->content, $this->mergeFields['Date']) = mysql_fetch_row($result);
		
		$this->mergeFields['Title'] = htmlspecialchars($this->mergeFields['Title']);
		$this->mergeFields['Date'] = formatDateNicely($this->mergeFields['Date'], $this->setting('date_format'));
		
		if ($this->status == 'private_draft'?
				checkPriv('_PRIV_EDIT_DRAFT')
			:
				checkPriv('_PRIV_CREATE_REVISION_DRAFT')
		) {
			if (session('tribiq_content__edit_section') == $this->cID. '_'. $this->cType. '_'. $this->section) {
				$this->callScript('tribiq_content', 'edit', $this->slotName, $this->section);
				unset($_SESSION['tribiq_content__edit_section']);
			}
			
			//Open the editor if it has been requested
			if ($this->request('content__edit_slot') == $this->slotName) {
				$this->markSlotAsBeingEdited(true);
				$this->callScript('tribiq_content', 'open', $this->setting('editor_size') == '_WIDE'? 1 : 0, $this->containerId);
			} else {
				$this->markSlotAsBeingEdited(false);
			}
		}
		
		//If there is no content then don't display the plugin in visitor mode
		return !($this->content === '' || $this->content === NULL);
	}
	
	
	function showSlot() {
		$this->frameworkHead('Outer', 'Content', $this->mergeFields, $this->sections);
			$this->showContent();
		$this->frameworkFoot('Outer', 'Content', $this->mergeFields, $this->sections);
	}
	
	
	function showContent() {
		
		//In visitor mode? Admin doesn't have permissions to edit? Just display the content and then exit.
		if (
			$this->status == 'private_draft'?
				!checkPriv('_PRIV_EDIT_DRAFT')
			:
				!checkPriv('_PRIV_CREATE_REVISION_DRAFT')
		) {
			echo $this->content;
			return;
		}
		
		
		//Is this a historic version? Is there already a draft created? Don't allow any editing if so!
		if ($this->cVersion != $this->latestVersion && $this->latestVersionStatus == 'private_draft') {
			echo '
				<div class="tribiq_hide_from_print">
					<img src="admin/images/icons/icon_edit_disabled.gif" border="0" alt="', getPhrase('_EDIT_DISABLED'), '"
						title="', getPhrase('_AUTHORING'), '|', getPhrase('_SORRY_CONTENT_VERSION_NOT_EDITABLE_PLEASE_SELECT_OTHER_VERSION'), '" />
				</div>'; 	
			
			echo $this->content;
			return;
		
		} elseif (!$this->beingEdited()) {
			echo '
				<div class="hide_from_print">';
			
			//Allow private drafts to be edited inline; otherwise require the user to reload the page
			if ($this->status == 'private_draft') {
				echo '
					<a', $this->refreshPluginSlotAnchor('&content__edit_slot='. $this->slotName. '&content__section='. $this->section), '>';
			} else {
				echo '
					<a href="#" onclick="',
						floatingBoxJS(
							getPhrase('<p>You are about to edit a Published Content Item.</p><p>When you do this, the CMS makes an identical Draft version for you to work on privately.</p><p>You can make changes to the Draft, but visitors will only see your changes when you Publish the new version.</p><p>Are you sure you want to make Draft now?</p>'),
							'<input type="button" class="submit_selected" value="'. getPhrase('Make a new Draft'). '" onclick="tribiq_content.createPrivateDraft(\''. $this->slotName. '\', \''. $this->section. '\');" />',
							true,
							true),
					' return false;">';
			}
			
			if ($this->section == 'content_bodytop') {
				$editPhrase = getPhrase('_EDIT_CONTENT_BODYTOP_INLINE');
			} elseif ($this->section == 'content_bodymain') {
				$editPhrase = getPhrase('_EDIT_CONTENT_BODYMAIN_INLINE');
			} elseif ($this->section == 'content_bodyfoot') {
				$editPhrase = getPhrase('_EDIT_CONTENT_BODYFOOT_INLINE');
			}
			
			echo '
						<img src="admin/images/slots/edit_icon.gif" class="pluginAdminButton" border="0" alt="', $editPhrase, '"
							title="', getPhrase('_AUTHORING'), '|', $editPhrase, '" />
					</a>';
					
			
			echo '
				</div>';
			
			echo $this->content;
		
		} else {
			
			//Display the content area in an editor, so the admin can change its contents
			//Note that the actual function that saves the content is still part of the CORE CMS
			echo
				'<form>
					<input type="hidden" id="', $this->containerId, '_section" name="content__section" value="', $this->section, '" />
					<textarea name="content__content" style="width: 100%; height: 450px; visibility: hidden;" class="tinymce_content">',
						htmlspecialchars($this->content),
					'</textarea>
					<div id="', $this->containerId, '__buttons" style="text-align: right; height: 1px; overflow: visible; width: 100%;">
						<div style="text-align: right; position: relative; right: 17px; bottom: ',
								(strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 6') !== false ||
								 strpos($_SERVER['HTTP_USER_AGENT'], 'MSIE 7') !== false ||
								 strpos($_SERVER['HTTP_USER_AGENT'], 'Firefox') !== false)? 22 : 20, 'px;">
							<input type="button" onclick="tribiq_content.saveViaAJAX(\'', $this->slotName, '\', true);"
								style="font-size: 11px; margin: 0px; padding: 1px 9px; border:1px solid #C5C5C5; color: #FFF; background-color: #E51700;"
								value="', getPhrase('Save and preview Draft'), '" title="', getPhrase('Save and preview Draft'), '|', getPhrase('Save changes and close the editor.'), '"/>
							<input type="button" onclick="tribiq_content.close(\'', $this->slotName, '\');"
								style="font-size: 11px; margin: 0px; padding: 1px 9px; border:1px solid #C5C5C5; color: #FFF; background-color: #2E778A;"
								value="', getPhrase('Abandon Changes'), '" title="', getPhrase('Abandon Changes'), '|', getPhrase('Abandon unsaved changes and close the editor.'), '"/>
						</div>
					</div>
				</form>';
		}
	}
	
}



?>