<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */




class tribiq_ctype_document extends plugin_base_class {
	
	function init() {
		if ($this->cType != 'document') {
			return false;
		}
		
		$this->forcePageReload();
		includeContentType('document');
		
		
		$this->mergeFields = array();
		$this->allowedChildSections = array();
		$document = getDocument( $this->cID, $this->cVersion );
		
		$this->mergeFields['Document_Download_Title'] = $this->phrase( '_DOCUMENT_DOWNLOAD_TITLE:' );
		$this->mergeFields['Title'] = htmlspecialchars( $document['title'] );
		
		if ( $this->mergeFields['Description'] = htmlspecialchars( $document['description'] ) ) {
			$this->mergeFields['Description_Label'] = $this->phrase( '_DESCRIPTION:' );
			$this->allowedChildSections['Summary_Section'] = true;
			$this->allowedChildSections['Description_Section'] = true;
		}

		$this->mergeFields['Type'] = htmlspecialchars( $document['type'] );
		$this->mergeFields['Type_Label'] = $this->phrase( '_TYPE:' );
		
		if ($this->setting('use_sticky_image') && ($image = $this->itemStickyImage($this->cID, $this->cType, $this->cVersion))){
			$this->mergeFields['Sticky_image'] = "background: url('" .  htmlspecialchars($image) . "') no-repeat scroll 0 0;";
		}elseif ( in_array( $this->mergeFields['Type'], array( 'avi', 'doc', 'jpg', 'gz', 'pdf', 'ppt', 'rtf', 'txt', 'xls', 'zip' ) ) ) {
			$this->mergeFields['Icon'] = $this->mergeFields['Type'] . '_icon.jpg';
			$this->mergeFields['Icon_Class'] = $this->mergeFields['Type'] . '_icon';
		} else {
			$this->mergeFields['Icon'] = 'unknown_icon.jpg';
			$this->mergeFields['Icon_Class'] = 'unknown_icon';
		}
			
		$this->mergeFields['Size'] = formatFilesizeNicely( $document['size'],0,false,"tribiq_ctype_document" );
		$this->mergeFields['Size_Label'] = $this->phrase( '_FILE_SIZE:' );


		
		
		
		if ( $this->mergeFields['Document_Text_Count'] = strlen( trim( $document['document_text'] ) ) ) {
			$this->mergeFields['Document_Text_Summary'] = htmlspecialchars( substr( $document['document_text'], 0, 255 ) );
			$this->mergeFields['PlainText_Label'] = $this->phrase( '_PLAINTEXT_VERSION_EXTRACT:' );
			$this->allowedChildSections['Summary_Section'] = true;
			$this->allowedChildSections['Document_Text_Section'] = true;
		}
		
		if ( $this->mergeFields['Published'] = formatDateNicely( getPublicationDate( $this->cID, $this->cVersion ) ) ) {
			$this->mergeFields['Published_Label'] = $this->phrase( '_PUBLISHED:' );
			$this->allowedChildSections['Published_Section'] = true;
		}

		if (!$this->mergeFields['Link'] = buildDownload( $this->cID, $this->cVersion ) ) {
			$this->mergeFields['Download_Unavailable'] = $this->phrase( '_DOCUMENT_UNAVAILABLE_DOWNLOAD' );
			$this->allowedChildSections['No_Document_Link_Section'] = true;
			$this->allowedChildSections['No_Document_Image_Link_Section'] = true;
		} else {
			$this->mergeFields['Download_Now'] = $this->phrase( '_DOWNLOAD_NOW' );
			$this->mergeFields['Download_File'] = $this->phrase( '_DOWNLOAD_DOCUMENT_FILE' );
			$this->allowedChildSections['Document_Image_Link_Section'] = true;
		}
		
		
		if ($this->cType == 'document' && $this->mergeFields['Link'] && $this->get('download_now')) {
			$this->headerRedirect($this->mergeFields['Link']);
		}
		
		
		return true;
	}

	function showSlot() {
		if ($this->cType != 'document') {
			echo getPhrase( '_PLUGIN_NEEDS_CONTENT_TYPE', array( 'cType' => 'document' ) );
			return;
		}
		
		$this->framework( 'Outer', $this->mergeFields, $this->allowedChildSections );
	}
}
?>