<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */




class tribiq_ctype_picture extends plugin_base_class {
	
	function init() {
		$this->forcePageReload();
		return $this->cType == 'picture';
	}

	function showSlot() {
		if ($this->cType != 'picture') {
			echo getPhrase('_PLUGIN_NEEDS_CONTENT_TYPE', array('cType' => 'picture'));
			return;
		}
		
		includeContentType('picture');
		
		$mergeFields = array();
		$subSections = array();
		
		
		$constraints = array();
		
		if ($this->setting('resize_image_for_download')) {
			$constraints[] = array(
				"width" => $this->setting('image_width'),
				"height" => $this->setting('image_height'));
		}
		
		if ($this->setting('offer_alternate_size_for_download')) {
			$constraints[] = array(
				"width" => $this->setting('alternate_image_width'),
				"height" => $this->setting('alternate_image_height'));
		}
		
		if ($this->setting('offer_original_image_size_for_download')) {
			$constraints[] = array();
		}
		

		if ($picture = getPicture($this->cID, $this->cVersion)) {
			
			$subSections['Picture'] = true;
			
			$mergeFields['File_Size_Label'] = $this->phrase( '_FILE_SIZE:' );
			$mergeFields['Size'] = formatFilesizeNicely($picture['size'],0,false,"tribiq_ctype_picture");
			
			$mergeFields['Document_Type_Label'] = $this->phrase( '_DOCUMENT_TYPE:' );
			$mergeFields['Type'] = formatFileTypeNicely($picture['mime_type'],"tribiq_ctype_picture");
			
			$mergeFields['Image_Name_Label'] = $this->phrase( '_IMAGE_NAME:' );
			$mergeFields['Filename'] = htmlspecialchars($picture['filename']);
			
			$mergeFields['Src'] = 'cmsincludes/image.php?cID='. $this->cID.
												'&amp;cType='. $this->cType.
												'&amp;cVersion='. $this->cVersion.
												'&amp;filename='. htmlspecialchars($picture['filename']);
			
			$mergeFields['Date_Created_Label'] = $this->phrase( '_DATE_CREATED:' );
			$mergeFields['Date_Created'] = formatDateNicely(getDateCreated($this->cID));
			$mergeFields['Date_Published'] = formatDateNicely($picture['published_datetime']);
			
			
			
			if ($picture['published_datetime'] && $mergeFields['Date_Published'] != $mergeFields['Date_Created']) {
				$subSections['Published'] = true;
				$mergeFields['Date_Modified_Label'] = $this->phrase( '_DATE_MODIFIED:' );
			}
			
			if($picture['width'] <> "" AND $picture['height'] <> "") {
				$subSections['Dimensions'] = true;				
				$mergeFields['Dimensions_Label'] = $this->phrase( '_DIMENSIONS:' );
				$mergeFields['Width'] = (int) $picture['width'];
				$mergeFields['Height'] = (int) $picture['height'];
				
			}
			
			$mergeFields['Download_Label'] = $this->phrase( '_DOWNLOAD_ORIGINAL:' );

			if ($links = buildDownloads($this->cID, $this->cVersion, $constraints, $this->setting('upscale_image_if_needed'))) {
				foreach( $links as $key => $value ) {
					$links[$key]['Download_Version'] = $this->phrase( '_DOWNLOAD_VERSION', array( 'xDimension' => $links[$key]['Download_Width'], 'yDimension' => $links[$key]['Download_Height'] ));
				}
				$subSections['Download_Links'] = $links;
			} else {
				$subSections['No_Download_Links'] = $links;
				$mergeFields['Picture_Unavailable_Download'] = $this->phrase( '_PICTURE_UNAVAILABLE_DOWNLOAD' );
			}
			$this->framework('Outer', $mergeFields, $subSections);
		}
		
	}
	
	
	


}
?>
