<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */





class tribiq_extranet_registration extends tribiq_extranet {
	
	public function init() {
		$this->lang = getLanguage();
		$this->manageCookies();
		
		$this->mode = 'modeRegistration';
		
		if ($this->useScreenName) {
			$this->subSections['Choose_Screen_Name'] = true;
		}
		
		
		if (issetArrayKey($_SESSION, 'extranetUserID')) {
			$this->mode = 'modeLoggedIn';
			
		} elseif ($this->setting('after_registration_email_template') && $this->post('extranet_resend')) {
			if ($this->validateFormFields('Resend_Form') && $this->sendRegistrationEmail()) {
				$this->mode = 'modeResent';
			} else {
				$this->mode = 'modeResend';
			}
			
		} elseif ($this->setting('after_registration_email_template') && $this->get('extranet_resend')) {
			$this->mode = 'modeResend';
			
		} elseif ($this->post('extranet_register')) {
			if ($this->post('screen_name')) {
				$this->postVariables['screen_name'] = trim($this->postVariables['screen_name']);
			}
			
			if ($this->registerUser()) {
				if ($this->sendRegistrationEmail()) {
					$this->mode = 'modeRegisteredEmail';			
				} else {
					$this->mode = 'modeRegisteredNoEmail';			
				}
				$this->sendRegistrationEmailToAdmin();
			}
		
		} elseif ($this->get('confirm_email')) {
			$user = $this->confirmEmail($this->get('hash'));
			$this->logUserIn($user['id'], $user);
			$this->mode = 'modeLoggedIn';
			
			$this->redirectToPage();
		}
		
		
		return true;
	}
	
	
	protected function modeRegistration() {
		$this->addLoginLinks();
		
		//Overwrite the Resend_Link from the addLoginLinks() function
		if ($this->setting('after_registration_email_template')) {
			$this->subSections['Resend_Link_Section'] = true;
			$this->objects['Resend_Link'] = $this->refreshPluginSlotAnchor('&extranet_resend=1');
		} else {
			$this->subSections['Resend_Link_Section'] = false;
			unset($this->objects['Resend_Link']);
		}
		
		$this->frameworkHead('Outer', 'Registration_Form', $this->objects, $this->subSections);
			echo $this->openForm();
					$this->framework('Registration_Form', $this->objects, $this->subSections);
			echo $this->closeForm();
		$this->frameworkFoot('Outer', 'Registration_Form', $this->objects, $this->subSections);
	}

	protected function modeResend() {
		$this->addLoginLinks();
		
		$this->objects['Registration_Link'] = $this->refreshPluginSlotAnchor();
		
		$this->frameworkHead('Outer', 'Resend_Form', $this->objects, $this->subSections);
			echo $this->openForm(),
					$this->remember('extranet_resend');
					$this->framework('Resend_Form', $this->objects, $this->subSections);
			echo $this->closeForm();
		$this->frameworkFoot('Outer', 'Resend_Form', $this->objects, $this->subSections);
	}

	protected function modeRegistered() {
		$this->subSections['Registered'] = true;
		$this->framework('Outer', $this->objects, $this->subSections);
	}

	protected function modeRegisteredEmail() {
		$this->subSections['Registered_Email_Sent'] = true;
		$this->framework('Outer', $this->objects, $this->subSections);
	}

	protected function modeResent() {
		$this->subSections['Registered_Email_Resent'] = true;
		$this->framework('Outer', $this->objects, $this->subSections);
	}

	protected function modeRegisteredNoEmail() {
		$this->subSections['Registered_No_Email_Sent'] = true;
		$this->framework('Outer', $this->objects, $this->subSections);
	}
	



	
	
	protected function registerUser() {
		if (!$fields = $this->validateFormFields('Registration_Form')) {
			return false;
		}

		if ($this->useScreenName) {
			$fields['username'] = $this->post('screen_name');
		} else {
			$fields['username'] = $this->post('email');
		}
		
		$fields['status'] = 'pending';
		$fields['created_date'] = date("Y-m-d H:i:s");
		$fields['password'] = createPassword();
		$fields['hash'] = md5($fields['username']. SITENAME);
		
		$sql = "";
		foreach(getFields(DB_NAME_PREFIX, 'users') as $field => $details) {
			if (isset($fields[$field])) {
				addFieldToSQL($sql, DB_NAME_PREFIX. 'users', $field, $fields, false, $details);
			}
		}
		
		my_mysql_query($sql);
		$userId = mysql_insert_id();
		
		$this->sendSignalFromForm('eventUserRegistered', $userId);
		
		return $userId;
	}
	
	protected function sendRegistrationEmail(){
		if ($this->setting('after_registration_email_template')) {
			if ((count($rcpts=$this->registrationEmailRcpts($this->post('email')))) ){
				if ($this->inc('tribiq_email_template_manager')){
					$userDetails = $this->getDetailsFromEmail($this->post('email'));
					
					$userDetails['IP_Address'] = $_SERVER['REMOTE_ADDR'];
					$userDetails['CMS_URL'] = $this->absCMSDirURL();
					$userDetails['Email_Confirmation_Link'] = $this->linkToItem($this->cID, $this->cType, $fullPath = true, $request = '&confirm_email=1&hash='. $userDetails['Hash']);
					
					if (!tribiq_email_template_manager::sendEmailsUsingTemplate(implode(",",$rcpts) ,$this->setting('after_registration_email_template'),$userDetails,array())){
						$this->message = $this->phrase('_COULD_NOT_SEND_REGISTRATION_EMAIL');
						return false;
					}
				} else {
					$this->message = $this->phrase('_COULD_NOT_SEND_REGISTRATION_EMAIL');
					return false;
				}
			} else {
				return false;
			}
		} else {
			return false;
		}
		return true;
	}
	
	protected function sendRegistrationEmailToAdmin(){
		if ($this->setting('registration_notification')
		 && $this->setting('registration_notification_email_address')
		 && $this->setting('registration_notification_email_template')) {
			if ($this->inc('tribiq_email_template_manager')) {
				$userDetails = $this->getDetailsFromEmail($this->post('email'));
				
				$userDetails['IP_Address'] = $_SERVER['REMOTE_ADDR'];
				$userDetails['CMS_URL'] = $this->absCMSDirURL();
				$userDetails['Email_Confirmation_Link'] = $this->linkToItem($this->cID, $this->cType, $fullPath = true, $request = '&confirm_email=1&hash='. $userDetails['Hash']);
				$userDetails['Storekeeper_Link'] = httpOrhttps(). DEFAULT_URL. SUBDIRECTORY. 'admin/storekeeper.php#tribiq__users//'. $userDetails['id'];
				
				tribiq_email_template_manager::sendEmailsUsingTemplate(
					$this->setting('registration_notification_email_address'),
					$this->setting('registration_notification_email_template'),
					$userDetails,
					array());
			}
		}
	}
	
	protected function registrationEmailRcpts($userId){
		if ($userDetails = $this->getDetailsFromEmail($this->post('email'))){
			$rv[]=$userDetails['Email'];
		}					
		return $rv;
	}
	
	function confirmEmail ($hash) {
		$sql = "UPDATE " . DB_NAME_PREFIX . "users
				SET email_verified = 1,
					status = 'active'
				WHERE hash = '" . mysql_real_escape_string($hash) . "'";
				
		$result = my_mysql_query($sql);
		
		$sql = "SELECT id,
					first_name,
					screen_name,
					username,
					email,
					password
				FROM " . DB_NAME_PREFIX . "users
				WHERE hash = '" . mysql_real_escape_string($hash) . "'";
		
		$result = my_mysql_query($sql);
		
		if ($row = mysql_fetch_array($result)) {
		
			if ($this->setting('add_new_users_to_a_group') && (int) $this->setting('add_new_users_to_group')) {
				addUserToGroup($row['id'], (int) $this->setting('add_new_users_to_group'));
			}
			
			return $row;
		} else {
			return false;
		}
	}
	
	
	//If a page is created with this Plugin as the Featured Plugin,
	//on a Language without a Registration Page set, set this as the Registration Page.
	public static function eventQuickAddContentItem($cID, $instance) {
		if ($instance['class_name'] == 'tribiq_extranet_registration') {
			$sql = "
				UPDATE ". DB_NAME_PREFIX. "languages SET
					extranet_registration_page_id = ". (int) $cID. "
				WHERE id = '". mysql_real_escape_string(getContentLang($cID, 'html')). "'
				  AND extranet_registration_page_id = 0";
			my_mysql_query($sql);
		}
	}
}
?>