<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */





class tribiq_extranet extends plugin_base_class {
	
	var $mode;
	var $reloadPage = false;
	var $errors = array();
	var $message = false;
	var $lang = array();
	var $useScreenName = true;
		
	var $objects = array();
	var $subSections = array();
	
	
	public function init() {
		$this->lang = getLanguage();
		$this->mode = 'modeLogin';
		
		
		if (issetArrayKey($_SESSION, 'extranetUserID')) {
			$this->mode = 'modeLoggedIn';
		} else {
			if ($this->checkFrameworkSectionExists('Login_Form')) {
				if ($this->setting('enable_remember_me')) {
					$this->subSections['Remember_Me_Section'] = true;
				}
				if ($this->setting('enable_log_me_in')) {
					$this->subSections['Log_Me_In_Section'] = true;
				}
				
				if (!$this->useScreenName || $this->setting('login_with') == 'Email') {
					$this->subSections['Login_with_email'] = true;
					$this->objects['extranet_email'] = arrayKey($_COOKIE, 'COOKIE_LAST_EXTRANET_EMAIL');
				} else {
					$this->subSections['Login_with_screen_name'] = true;
					$this->objects['extranet_screen_name'] = arrayKey($_COOKIE, 'COOKIE_LAST_EXTRANET_SCREEN_NAME');
				}
			}
			
			if ($this->post('extranet_login')) {
				//Check if the login was successful and redirect the user if so
				if ($this->checkLogin()) {
					$this->redirectToPage();
				}
			}
		}
		
		$this->manageCookies();
		return true;
	}
	
	
	public function showSlot() {
		
		//A message if given
		if ($this->message) {
			$this->subSections['Message_Display'] = true;
			$this->objects['Message'] = $this->message;
		}
		
		//Display any errors we encountered
		if (!empty($this->errors)) {
			$this->subSections['Error_Display'] = $this->errors;
		}
		
		$mode = $this->mode;
		$this->$mode();
	}
	
	
	//Display a login form
	protected function modeLogin() {
		$this->addLoginLinks();
		
		if ($this->checkFrameworkSectionExists('Login_Form')) {
			$this->subSections['Login_title_section'] = true;
			
			$this->frameworkHead('Outer', 'Login_Form', $this->objects, $this->subSections);
				
				if ($this->cType == 'html' && $this->cID == $this->lang['extranet_login_page_id']) {
					echo $this->openForm();
				} else {
					echo '<form action="'. htmlspecialchars($this->linkToItem($this->lang['extranet_login_page_id'])). '" method="post">';
				}
				
						$this->framework('Login_Form', $this->objects, $this->subSections);
				echo $this->closeForm();
			$this->frameworkFoot('Outer', 'Login_Form', $this->objects, $this->subSections);
		
		} else {
			$this->subSections['Login'] = true;
			$this->framework('Outer', $this->objects, $this->subSections);
		}
	}
	
	protected function addLoginLinks() {
		
		if ($this->lang['extranet_login_page_id'] && $this->checkFrameworkSectionExists('Login_Link_Section')) {
			$this->subSections['Login_Link_Section'] = true;
			$this->objects['Login_Link'] = $this->linkToItemAnchor($this->lang['extranet_login_page_id']);
		}
		if ($this->lang['extranet_password_reminder_page_id'] && $this->checkFrameworkSectionExists('Forget_Password_Link_Section')) {
			$this->subSections['Forget_Password_Link_Section'] = true;
			$this->objects['Forget_Password_Link'] = $this->linkToItemAnchor($this->lang['extranet_password_reminder_page_id']);
		}
		if ($this->lang['extranet_registration_page_id'] && $this->checkFrameworkSectionExists('Registration_Link_Section')) {
			$this->subSections['Registration_Link_Section'] = true;
			$this->objects['Registration_Link'] = $this->linkToItemAnchor($this->lang['extranet_registration_page_id']);
		}
		if ($this->lang['extranet_registration_page_id'] && $this->checkFrameworkSectionExists('Resend_Link_Section')) {
			$this->subSections['Resend_Link_Section'] = true;
			$this->objects['Resend_Link'] = $this->linkToItemAnchor($this->lang['extranet_registration_page_id'], 'html', false, '&extranet_resend=1');
		}
	}
	
	
	//Display a welcome message when the user is logged in
	protected function modeLoggedIn() {
		$this->addLoggedInLinks();
		
		$this->subSections['Logged_In'] = true;
		$this->framework('Outer', $this->objects, $this->subSections);
	}
	
	protected function addLoggedInLinks() {
		if ($this->checkFrameworkSectionExists('Welcome_Message_Section')) {
			$this->subSections['Welcome_Message_Section'] = true;
			$this->objects['Welcome_Message'] = $this->phrase('_WELCOME', array('user' => htmlspecialchars(arrayKey($_SESSION, 'extranetUser_firstname'))));
		}
		
		if ($this->lang['extranet_change_password_page_id'] && $this->checkFrameworkSectionExists('Change_Password_Link_Section')) {
			$this->subSections['Change_Password_Link_Section'] = true;
			$this->objects['Change_Password_Link'] = $this->linkToItemAnchor($this->lang['extranet_change_password_page_id']);
		}
		if ($this->lang['extranet_logout_page_id'] && $this->checkFrameworkSectionExists('Logout_Link_Section')) {
			$this->subSections['Logout_Link_Section'] = true;
			$this->objects['Logout_Link'] = $this->linkToItemAnchor($this->lang['extranet_logout_page_id']);
		}
		
		if (issetArrayKey($_SESSION, 'destURL') && $this->cID != $_SESSION['destCID'] && $this->cType != $_SESSION['destCType']) {
			$this->subSections['Destination_url_section'] = true;
			$this->objects['destURL_Title'] = htmlspecialchars($_SESSION['destTitle']);
			$this->objects['destURL_Link'] = htmlspecialchars($_SESSION['destURL']);
		}
	}
	













	
	
	
	protected function manageCookies() {
		$this->useScreenName = (bool) getSiteConfig('user_use_screen_name');
		
		//Set the User's email/screenname cookie on their local machine if requested
		if (isset($_SESSION['SET_EXTRANET_LOGIN_COOKIE'])) {
			if (!$this->useScreenName || $this->setting('login_with') == 'Email') {
				setcookie('COOKIE_LAST_EXTRANET_EMAIL', $_SESSION['SET_EXTRANET_LOGIN_COOKIE'], time()+8640000, '/');
			} else {
				setcookie('COOKIE_LAST_EXTRANET_SCREEN_NAME', $_SESSION['SET_EXTRANET_LOGIN_COOKIE'], time()+8640000, '/');
			}
			unset($_SESSION['SET_EXTRANET_LOGIN_COOKIE']);
		
		//Remove the User's email/screenname cookie on their local machine if requested
		} elseif (isset($_SESSION['FORGET_EXTRANET_LOGIN_COOKIE'])) {
			if (!$this->useScreenName || $this->setting('login_with') == 'Email') {
				setcookie('COOKIE_LAST_EXTRANET_EMAIL', '', time()-3600, '/');
			} else {
				setcookie('COOKIE_LAST_EXTRANET_SCREEN_NAME', '', time()-3600, '/');
			}
			unset($_SESSION['FORGET_EXTRANET_LOGIN_COOKIE']);
		}
		
		//Set a hash of the User's details in a cookie on their local machine if requested, so they can be logged in automatically
		if (isset($_SESSION['SET_EXTRANET_LOG_ME_IN_COOKIE'])) {
			setcookie('LOG_ME_IN_COOKIE', $_SESSION['SET_EXTRANET_LOG_ME_IN_COOKIE'], time()+8640000, '/');
			unset($_SESSION['SET_EXTRANET_LOG_ME_IN_COOKIE']);
		
		//Remove the hash of the User's details if requested
		} elseif (isset($_SESSION['FORGET_EXTRANET_LOG_ME_IN_COOKIE'])) {
			setcookie('LOG_ME_IN_COOKIE', '', time()-3600, '/');
			unset($_SESSION['FORGET_EXTRANET_LOG_ME_IN_COOKIE']);
		}
	}
	
	
	protected function checkPermsOnDestURL() {
		return issetArrayKey($_SESSION, 'destCID') && issetArrayKey($_SESSION, 'destCType') && checkPerm($_SESSION['destCID'], $_SESSION['destCType']);
	}
	
	protected function redirectToPage($showWelcomePage = true, $redirectBackIfPossible = true, $redirectRegardlessOfPerms = true) {
		
		//Redirect the user to the welcome page, if one is set and enabled in the Plugin Settings
		$cID = $cType = false;
		
		//Check to see if there is a valid destination URL
		$validDestURL = false;
		if (session('destURL')) {
			$validDestURL = true;
			
			//Check to see if the destination URL is not an extranet page/the home page
			if (session('destCID') && session('destCType') == 'html') {
				$lang = getLanguage();
				if (session('destCID') == $lang['homepage_id']
				 || session('destCID') == $lang['extranet_login_page_id']
				 || session('destCID') == $lang['extranet_registration_page_id']
				 || session('destCID') == $lang['extranet_password_reminder_page_id']
				 || session('destCID') == $lang['extranet_change_password_page_id']
				 || session('destCID') == $lang['extranet_logout_page_id']) {
					$validDestURL = false;
				}
			}
		}
		
		if ($showWelcomePage
			&& ( $this->getCIDAndCTypeFromSetting($cID, $cType, 'welcome_page'))
			&& ( $this->setting('show_welcome_page') == '_ALWAYS'
			||  ($this->setting('show_welcome_page') == '_IF_NO_PREVIOUS_PAGE' && !$validDestURL))
		) {
			if ($this->inc('tribiq_avls')) {
				tribiq_avls::equivalentItem($cID, $cType);
			}
			$this->headerRedirect($this->linkToItem($cID, $cType, true));
		
		//Otherwise attempt to redirect the user back where they came from
		} elseif ($redirectBackIfPossible && $validDestURL && ($redirectRegardlessOfPerms || $this->checkPermsOnDestURL())) {
			$this->headerRedirect($_SESSION['destURL']);
		
		//Otherwise stay on the current page
		} else {
			$this->headerRedirect($this->linkToItem($this->cID, $this->cType, true));
		}
	}
	
	protected function validateFormFields($section) {
		//Validate a form submission
		$fields = array();
		foreach($this->frameworkFields($section, $this->subSections) as $name => $field) {
			if (engToBoolean($field['required']) && !$this->post($name)) {
				$this->errors[] = array('Error' => $this->phrase('_ERROR_'. strtoupper($name)));
			
			} elseif (($field['pattern'] == 'email' || $field['pattern'] == 'new_email'
					|| $field['pattern'] == 'existing_email' || $field['pattern'] == 'unverified_email') && $this->post($name)) {
				
				if (!validateEmailAddress($this->post($name))) {
					$this->errors[] = array('Error' => $this->phrase('_ERROR_INVALID_'. strtoupper($name)));
				
				} elseif ($field['pattern'] == 'new_email') {
					if (checkRowExists('users', array('email' => $this->post($name)))
					 || checkRowExists('users', array('username' => $this->post($name)))) {
						$this->errors[] = array('Error' => $this->phrase('_ERROR_EMAIL_IN_USE'));
					}
				
				} elseif ($field['pattern'] == 'existing_email' || $field['pattern'] == 'unverified_email') {
					if (!checkRowExists('users', array('email' => $this->post($name)))) {
						$this->errors[] = array('Error' => $this->phrase('_ERROR_EMAIL_NOT_IN_USE'));
					
					} elseif ($field['pattern'] == 'unverified_email' && getRow('users', 'email_verified', array('email' => $this->post($name)))) {
						$this->errors[] = array('Error' => $this->phrase('_ERROR_EMAIL_VERIFIED'));
					}
				}
			
			} elseif (($field['pattern'] == 'screen_name' || $field['pattern'] == 'new_screen_name') && $this->post($name)) {
				if (!validateScreenName($this->post($name))) {
					$this->errors[] = array('Error' => $this->phrase('_ERROR_INVALID_'. strtoupper($name)));
				
				} elseif ($field['pattern'] == 'new_screen_name') {
					if (checkRowExists('users', array('username' => $this->post($name))) ||
						checkRowExists('users', array('screen_name' => $this->post($name)))) {
						$this->errors[] = array('Error' => $this->phrase('_ERROR_SCREEN_NAME_IN_USE'));
					}
				}
			}
			
			$fields[$name] = $this->post($name);
		}
		
		return empty($this->errors)? $fields : false;
	}
	
	
	//Log a user in after successful validation
	function logUserIn($userId, $user) {
		
		if ($this->setting('enable_remember_me')) {
			if ($this->post('extranet_remember_me')) {
				if (!$this->useScreenName || $this->setting('login_with') == 'Email') {
					$_SESSION['SET_EXTRANET_LOGIN_COOKIE'] = $user['email'];
				} else {
					$_SESSION['SET_EXTRANET_LOGIN_COOKIE'] = $user['screen_name'];
				}
			} elseif ($this->setting('enable_remember_me')) {
				$_SESSION['FORGET_EXTRANET_LOGIN_COOKIE'] = true;
			}
		}
			
		if ($this->setting('enable_log_me_in')) {
			if ($this->post('extranet_log_me_in')) {
				$_SESSION['SET_EXTRANET_LOG_ME_IN_COOKIE'] = $user['id']. '_'. md5(SITENAME. $user['id']. $user['username']. $user['email']. $user['password']);
			}
		}
		
		logUserIn($userId, $user);
	}
	
	//Handle actions
	protected function checkLogin() {
		
		if ($this->validateFormFields('Login_Form')) {
			//Check if this user exists, their password is correct, and they are active. Only log them in if so.
			if (!$this->useScreenName || $this->setting('login_with') == 'Email') {
				$user = checkLogin($this->post('extranet_email'), $this->post('extranet_password'), 'email');
			} else {
				$user = checkLogin($this->post('extranet_screen_name'), $this->post('extranet_password'), 'screen_name');
			}

			if ($user) {
				$this->logUserIn($user['id'], $user);
				return true;
				
			} else {
				$this->errors[] = array('Error' => $this->phrase('_ERROR_USER_OR_PASS_NOT_VERIFIED'));
			}
		}
		
		return false;
	}
	
	
	protected function logout() {
		logUserOut();
	}
	
	
	protected function sendSignalFromForm($signalName, $userId) {
		
		$fields = array();
		foreach ($this->postVariables as $name => &$value) {
			switch ($name) {
				case 'cID':
				case 'cType':
				case 'extranet_edit_profile':
				case 'extranet_register':
				case 'extranet_update_profile':
				case 'instanceId':
				case 'slotName':
				case 'tab':
					continue;
				default:
					$fields[$name] = $value;
			}
		}
		
		return sendSignal($signalName, array('userId' => $userId, 'instanceId' => $this->instanceId, 'fields' => $fields));
	}
	
	
	
	
	

	protected function getPassword($id) {
		$sql = "
			SELECT password
			FROM ". DB_NAME_PREFIX. "users
			WHERE id = ". (int) $id;
		
		$result = my_mysql_query($sql);
		if ($row = mysql_fetch_array($result)) {
			return $row['password'];
		} else {
			return false;
		}
	}
	
	protected function getDetailsFromEmail($email) {
		$sql = "
			SELECT
				id AS id,
				first_name AS First_Name,
				last_name AS Last_Name,
				username AS Username,
				screen_name AS Screen_Name,
				password AS Password,
				email AS Email,
				hash AS Hash
			FROM ". DB_NAME_PREFIX. "users
			WHERE email = '". mysql_real_escape_string($email). "'";
		
		$result = my_mysql_query($sql);
		return mysql_fetch_array($result);
	}
	
	
	//If a page is created with this Plugin as the Featured Plugin,
	//on a Language without a Login Page set, set this as the Login Page.
	public static function eventQuickAddContentItem($cID, $instance) {
		if ($instance['class_name'] == 'tribiq_extranet') {
			$sql = "
				UPDATE ". DB_NAME_PREFIX. "languages SET
					extranet_login_page_id = ". (int) $cID. "
				WHERE id = '". mysql_real_escape_string(getContentLang($cID, 'html')). "'
				  AND extranet_login_page_id = 0";
			my_mysql_query($sql);
		}
	}
}
?>
