<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */




class tribiq_feed_reader extends plugin_base_class {

	protected $xmlParser;
	protected $content;
	protected $newsDates;
	protected $itemCount;
	protected $insideItem = false;
	protected $tag = '';
	protected $title = '';
	protected $description = '';
	protected $date = '';
	protected $link = '';
	protected $attributes = '';
	protected $linkTarget = '';

	
	function init() {
		if ( 'New window' == $this->setting( 'target' ) ) {
			$this->linkTarget = ' target="_blank"';
		}
		return true;
	}


	public function __construct() {
		$this->xmlParser = xml_parser_create();
		xml_parser_set_option( $this->xmlParser, XML_OPTION_CASE_FOLDING, 1 );
		xml_parser_set_option( $this->xmlParser, XML_OPTION_SKIP_WHITE, 1 );
		xml_set_object( $this->xmlParser, $this );
		xml_set_element_handler( $this->xmlParser, 'startElementHandler', 'endElementHandler' );
		xml_set_character_data_handler( $this->xmlParser, 'characterDataHandler' );
	}
	
	function startElementHandler( $xmlParser, $tagName, $attributes ) {
		if ( $this->insideItem )  {
			$this->tag = $tagName;
			$this->attributes = $attributes;
		} elseif  ( 'ITEM' == $tagName || 'ENTRY' == $tagName )  {
			$this->insideItem = true;
		}
	}

	function endElementHandler( $xmlParser, $tagName ) {
		if ( $this->insideItem && ( 'ITEM' == $tagName || 'ENTRY' == $tagName ) ) {
			$this->link = trim( $this->link );
			if (  empty( $this->link ) ) {	
				array_push( $this->content, array( 
					'title' => htmlspecialchars( trim( $this->title ) ),
					'description' => ( trim( $this->description) ),
					'date' =>  trim( $this->date ) ) ); 
			} else {
				array_push( $this->content, array( 
					'title' => '<a href="' . trim( $this->link ) . '"' . $this->linkTarget . '>'. htmlspecialchars( trim( $this->title ) ) . '</a>',
					'description' => ( trim( $this->description) ),
					'date' =>  trim( $this->date ) ) ); 
			}
			
			if ($this->setting('rss_date_format')=='backslashed_american'){
				$dateFeedContent = trim($this->date);
			}elseif($this->setting('rss_date_format')=='backslashed_european'){
				$dateFeedContent = explode('/',trim($this->date));
				if (count($dateFeedContent)==3){
					$dateFeedContent=$dateFeedContent[1] . '/' . $dateFeedContent[0] . '/' . $dateFeedContent[2] ;
				} else{
					$dateFeedContent=$feedContent['date'];
				}
			}elseif($this->setting('rss_date_format')=='autodetect'){
				$dateFeedContent = trim($this->date);
			} else {
				$dateFeedContent = trim($this->date);
			}
			$this->newsDates[] = strtotime($dateFeedContent);
			
			$this->title = '';
			$this->description = '';
			$this->link = '';
			$this->date = '';
			$this->attributes = array();
			$this->insideItem = false;
		}
	}

	function characterDataHandler( $xmlParser, $data ) {
		if ( $this->insideItem ) {
			switch ( $this->tag ) {
				case 'TITLE':
					$this->title .= $data;
				break;
				case 'DESCRIPTION':
				case 'SUMMARY':
				case 'CONTENT':
					$this->description .= $data;
				break;
				case 'ADMIN_MESSAGE':
					if ( checkPriv() ) {
						$this->description .= ' ' . $data;
					}
				break;
				case 'LINK':
					if ( array_key_exists( 'HREF', $this->attributes ) ) {
						$this->link .= $this->attributes['HREF'];
					}
					$this->link .= $data;
				break;
				case 'PUBDATE':
				case 'UPDATED':
					$this->date .= $data;
				break;
			}
		}
	}
	
	function showSlot() {
		$this->itemCount = $this->setting( 'number_feeds_to_show' );
		
		$pageMergeFields = array(
			'Title' => $this->setting( 'feed_title' ),
			'Source' => '<p>Feed source: <a href="' . $this->setting( 'feed_source' ) . '">' . $this->setting( 'feed_source' ) . '</a></p>'
		);

		$subSections = array( 'Content_Section' => true );
		$dateFormat = '';

		if ( "Don't show" == $this->setting( 'show_date_time' ) ) {
			$subSections['Date_Section'] = false; 
		} else {
			$subSections['Date_Section'] = true; 
			if ( $this->setting('date_format') == '_SHORT' ) {
				$dateFormat = getSiteConfig('vis_date_format_short');
			} elseif ( $this->setting('date_format') == '_LONG' ) {
				$dateFormat = getSiteConfig('vis_date_format_long');
			} elseif ( $this->setting('date_format') == '_MEDIUM' ) {
				$dateFormat = getSiteConfig('vis_date_format_med');
			}
		}

		$this->frameworkHead('Feed_Reader', 'Feeds', $pageMergeFields, $subSections );
			$itemsCount =0;
			foreach ( $this->getRssFeed() as $feedContent ) {
				if ($itemCount++>$this->itemCount){
					break;
				}
				if ($this->setting('rss_date_format')=='backslashed_american'){
					$dateFeedContent = $feedContent['date'];
				}elseif($this->setting('rss_date_format')=='backslashed_european'){
					$dateFeedContent = explode('/',$feedContent['date']);
					if (count($dateFeedContent)==3){
						$dateFeedContent=$dateFeedContent[1] . '/' . $dateFeedContent[0] . '/' . $dateFeedContent[2] ;
					} else{
						$dateFeedContent=$feedContent['date'];
					}
				}elseif($this->setting('rss_date_format')=='autodetect'){
					$dateFeedContent = $feedContent['date'];
				} else {
					$dateFeedContent = $feedContent['date'];
				}

				if ( "Show date only" == $this->setting( 'show_date_time' ) ) {
					$feedContent['date'] = formatDateNicely( date( 'Y-m-d', strtotime( $dateFeedContent ) ), $dateFormat );
				} elseif ( "Show date and time" == $this->setting( 'show_date_time' ) ) {
					$feedContent['date'] = formatDateTimeNicely( date( 'Y-m-d H:i:s', strtotime( $dateFeedContent ) ), $dateFormat );
				}
				
				if ( ! $this->setting( 'size' ) > 0 ) {
					$feedContent['description'] = '';
				} else {
					$feedContent['description'] = $this->truncateNicely( trim ( strip_tags( strtr( $feedContent['description'], array( "\n" => '<br> ', "\r\n" =>'<br> ' ) ) ) ) , $this->setting( 'size' ) );
				}
				$feedMergeFields = array( 
					'Feed_Title' => $feedContent['title'], 
					'Feed_Description' => $feedContent['description'], 
					'Date' => $feedContent['date'] 
				);
				$this->framework( 'Feeds', $feedMergeFields, $subSections );
			}
		$this->frameworkFoot('Feed_Reader', 'Feeds', $pageMergeFields, $subSections );
	
	}
	
	protected function getLiveFeed() {
		$feed = @file_get_contents( $this->setting( 'feed_source' ) );
		if ( ! $feed  ) { 
			$feed = '<?xml version="1.0" encoding="UTF-8"?>
			<error>
				<item>
					<title>Feed read error</title>
					<link>' . htmlentities( $this->setting( 'feed_source' ) ) . '</link>
					<description>Error reading feed data</description>
					<admin_message>from ' . htmlentities( $this->setting( 'feed_source' ) ) . '.</admin_message>
					<updated>' . date( 'Y-m-d H:i:s' ) . '</updated>
				</item>
			</error>';
		}
		return $feed;
	}

	protected function getRssFeed() {
		$this->content = array();
		$xml = $this->cache('getLiveFeed', 60 * (int) $this->setting('cache'), $this->setting('feed_source'));
		xml_parse( $this->xmlParser, $xml );
		xml_parser_free( $this->xmlParser );
		if ($this->setting('news_order')=='asc'){
			array_multisort($this->newsDates, SORT_ASC,$this->content);
		} elseif($this->setting('news_order')=='desc') {
			array_multisort($this->newsDates, SORT_DESC,$this->content);
		}
		return $this->content;
	}
	
	protected function truncateNicely( $string, $max ) {
		$string = trim( $string );
		if ( strlen( $string ) > $max ) {
			$breakpoint2 = strpos( $string, ' ', $max ); // find last ' '
			$breakpoint1 = strrpos( substr( $string, 0, $breakpoint2 ), ' ' ); // find new last ' '
    		if ( false ===  $breakpoint1 ) { 
				$string = ''; 
			} else {
				$string = substr( $string, 0, $breakpoint1 ) . $this->phrase( '_EXCERPT_CUTOFF' ); 
			}
		}
		return $string;
	}

}

?>