<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

class tribiq_search_results extends plugin_base_class {
	
	var $fields;
	var $cTypeToSearch;
	var $searchString;
	var $results;
	
	function init() {
		$defaultTab = $this->setting('search_html')? 'html' : ($this->setting('search_document')? 'document' : 'news');
		
		$this->registerGetRequest('page', 1);
		$this->registerGetRequest('searchString');
		$this->registerGetRequest('tab', $defaultTab);
		
		$this->searchString = $this->getVariables['searchString'];
		$this->page = $this->getVariables['page']? $this->getVariables['page'] : 1;
		
		$this->cTypeToSearch = ifNull($this->getVariables['tab'], $defaultTab);
		
		return true;
	}
	
	function setSearchFields() {
		
		$fields = array();
		$fields[] =	array('name' => 'title',			'weighting' => $this->setting('search_title')? 1 : 0);
		$fields[] =	array('name' => 'keywords',			'weighting' => $this->setting('search_keywords')? 1 : 0);
		$fields[] =	array('name' => 'description',		'weighting' => $this->setting('search_description')? 1 : 0);
		$fields[] =	array('name' => 'content_bodymain',	'weighting' => $this->setting('search_content_bodymain')? 1 : 0);

		$this->fields = array();
		
		if ($this->setting('search_html')) {
			$this->fields['html'] = $fields;
			$this->fields['html'][]		= array('name' => 'alias',			'weighting' => $this->setting('search_alias')? 1 : 0);
		}
		
		if ($this->setting('search_document')) {
			$this->fields['document'] = $fields;
		}
		
		if ($this->setting('search_news')) {
			$this->fields['news'] = $fields;
		}
	}
	
	
	function showSlot() {
	
		//Draw a field on the page to let tell that there is a search results tab on the page in JavaScript
		//Need to improve this...
		echo '
			<form action="">
				<div>
					<input type="hidden" id="searchResultsSlot" value="', $this->slotName, '" />
					<input type="hidden" id="searchResultsInstanceId" value="', $this->instanceId, '" />
				</div>
			</form>';
		
		$this->mergeFields = array();
		$this->sections = array();
		
		if ($this->searchString) {
			$this->sections['Search_Result_Heading'] = true;
			$this->mergeFields['Search_Results_For'] = $this->phrase( '_SEARCH_RESULTS_FOR', array('term' => htmlspecialchars('"' .$this->searchString . '"')));
		}
		
		$this->drawSearchBox();
		
		if ($this->searchString) {
			$this->doSearch( htmlspecialchars($this->searchString));
			
			$this->sections['Search_Result_Tabs'] = true;

			if ($this->results[$this->cTypeToSearch]['Record_Count']) {
				$this->sections['Search_Result_Rows'] = true;
				$this->sections['Search_Result_'. $this->cTypeToSearch. '_Row'] = $this->results[$this->cTypeToSearch]['search_results'];
				
				if ($this->results[$this->cTypeToSearch]['pagination']) {
					$this->pagination(
						'pagination_style',
						$this->page, $this->results[$this->cTypeToSearch]['pagination'],
						$this->mergeFields['Search_Pagination']);
				}
			} else {
				$this->sections['Search_No_Results_'. $this->cTypeToSearch] = true;
			}
		}
		
		if (method_exists($this, 'drawCategories')) {
			$this->drawCategories();
		}
			
		$this->framework('Outer', $this->mergeFields, $this->sections);
	}
	



	function doSearch($searchTerm) {
		$this->setSearchFields();
		$this->results['Search_String'] = htmlspecialchars($this->searchString);
		
		//Launch a search on html, document and news content types in turn
		$this->results = array();
		$this->sections['Search_Result_Tab'] = array();
		foreach($this->fields as $cType => $fields) {
			$this->results[$cType] = $this->searchContent($cType, $fields);
			$this->sections['Search_Result_Tab'][$cType] = array(
				'Tab_On' => $this->results[$cType]['Tab_On'],
				'Tab_Onclick' => $this->results[$cType]['Tab_Onclick'],
				'Type' => $this->results[$cType]['Type'],
				'Record_Count' =>$this->results[$cType]['Record_Count']
			);
		}
		
		//We'll only be displaying the details on one content type at a time
		//Sanitise the result set that we will be displaying, and prepare the url and other details
		if ($this->results[$this->cTypeToSearch]['Record_Count']) {
			foreach($this->results[$this->cTypeToSearch]['search_results'] as $i => &$result) {
				
				$result['Result_No'] = $this->results[$this->cTypeToSearch]['offset'] + $i;
				$this->displayHTMLAsPlainText($result['content_bodymain'], $this->setting('excerpt_length'), $this->setting('strip_titles_from_excerpts'), $searchTerm );
	
				$result['title']		= htmlspecialchars($result['title']);
				$result['keywords']		= htmlspecialchars($result['keywords']);
				$result['description']	= htmlspecialchars($result['description']);
				
				if (isset($result['document_text'])) {
					$result['document_text'] = htmlspecialchars($result['document_text']);
				}
				
				$result['url'] = htmlspecialchars($this->linkToItem($result['alias']? $result['alias'] : $result['id'], $this->cTypeToSearch));
			}
		}
	}
	
	
	function drawSearchBox($cID = false, $cType = false) {
		
		$this->sections['Search_Box'] = true;
		$this->mergeFields['Search_Field_ID'] = 'search_field_'. $this->containerId;
		$this->mergeFields['Search_String'] = htmlspecialchars($this->searchString);
		
		
		if ($cID && $cType) {
			$this->mergeFields['Search_Submit'] = '';
		} else {
			$cID = $this->cID;
			$cType = $this->cType;
			$this->launchAJAXSearch();
		}
		
		$this->mergeFields['Search_Target'] = indexDotPHP(true);
		$this->mergeFields['Search_Page_Alias'] = $cID;
		$this->mergeFields['Search_Page_cType'] = $cType;
	}
	
	function launchAJAXSearch() {
		$this->mergeFields['Search_Submit'] =
			$this->pluginClassName. ".refreshPluginSlot('". $this->slotName. "', '&searchString=' + encodeURIComponent(get('". $this->mergeFields['Search_Field_ID']. "').value)); return false;";
	}
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	
	/**
	 *	This function provides search functionality for the major contenttypes.
	 *	It has the following parameters:
	 *	
	 *	$cType: String.  The content type to be searched
	 *	
	 *	$fields: Array.  Contains arrays with the following keys:
	 *		'name': String.  The name of the field in the database table
	 *		'weighting': Number. Indicates if the field is to be searched, and the weight the field has in the search results
	 *		
	 *	$categories: Array.  An array of category_ids with which to limit the search
	 *	
	 */
	function searchContent($cType, $fields) {
	
		if (!is_array($fields)) {
			return false;
		}
		
		$searchterm = preg_replace("/\s+/", " ", trim(htmlspecialchars(stripslashes($this->searchString))));
		$searchterm = str_replace("'","\\'",$searchterm);
	
		$searchTerms = $this->getSearchtermParts($searchterm);
		
		$sql = "SELECT id, ";
		
		foreach ($fields as $field) {
			$sql .= $field['name'] . ",";
		}
		
		$sql_match1 = "
						(";
	
		foreach ($searchTerms as $searchTerm => $searchTermType) {
			$booleanSearch = "";
			
			if ($searchTermType=='word') {
				$searchTermTypeWeighting = 1;
			} else {
				$booleanSearch = " IN BOOLEAN MODE";
				$searchTermTypeWeighting = 2;
			}
			
			foreach ($fields as $field) {
				if ($field['weighting']) {
					if ($field['name']=="alias" || $field['name']=="type") {
						$sql_match1 .= "((" . $field['name'] . " LIKE '" . mysql_real_escape_string($searchTerm) . "%') * " . $searchTermTypeWeighting . " *" . $field['weighting'] . ") + ";
					} else {
						$sql_match1 .= "(MATCH (" . $field['name'] . ") AGAINST ('" . mysql_real_escape_string($searchTerm) . "'" . $booleanSearch . ") * " . $searchTermTypeWeighting . " *" . $field['weighting'] . ") + ";
					}
				}
			}
		}
		
		if (substr($sql_match1,-2,1)=="+") {
			$sql_match1 = substr($sql_match1,0,strlen($sql_match1)-2);
		}
		
		$sql_match2 = ") AS relevance
						FROM " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType);
		
		if (is_array($this->categories)) {
			$sql_match2 .= "
						LEFT JOIN " . DB_NAME_PREFIX . "category_item_link AS cil
							ON cil.item_id = id";
		}
		
		if (session('extranetUserID')) {		
			$sql_match2 .= "			
						LEFT JOIN " . DB_NAME_PREFIX . "group_content_link as gcl
							ON " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . ".id = gcl.content_id 
								AND " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . ".version = gcl.content_version
						LEFT JOIN " . DB_NAME_PREFIX . "user_content_link as ucl
							ON " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . ".id = ucl.content_id
								AND " . DB_NAME_PREFIX . "content_" . mysql_real_escape_string($cType) . ".version = ucl.content_version";
		}
		
		$sql_match2 .= "
						WHERE status = 'published'
							AND language_id = '".$_SESSION["user_lang"]."'";
		
		if (!$this->setting('show_private_items')) {
			if (session('extranetUserID')) {
				$sql_match2 .= "
							AND (
									(gcl.content_type = '" . mysql_real_escape_string($cType) . "'
									AND gcl.group_id IN (";
									
					if ($groups = getUserGroups($_SESSION['extranetUserID'])) {
						$sql_match2 .= implode(",",$groups);
					} else {
						$sql_match2 .= "0";
					}
									
					$sql_match2 .= ")
									AND private = 1)
								OR
									(ucl.content_type = '" . mysql_real_escape_string($cType) . "'
									AND ucl.user_id = ";
									
					if ($_SESSION['extranetUserID']) {
						$sql_match2 .= (int) $_SESSION['extranetUserID'];
					} else {
						$sql_match2 .= "0";
					}
									
					$sql_match2 .= "
									AND private = 1)
								OR (gcl.content_type IS NULL AND ucl.content_type IS NULL AND private = 1)
								OR
									private = 0
								)";
			} else {
				$sql_match2 .= "
							AND private = 0";
			}
		}
	
		if (is_array($this->categories)) {
			$sql_match2 .= "
						AND cil.content_type = '" . mysql_real_escape_string($cType) . "'
						AND cil.category_id IN (" . preg_replace('/[^0-9,]/', '', implode(",",$this->categories)) . ")";
			
		}
		
		foreach ($searchTerms as $searchTerm => $searchTermType) {
			if ($searchTermType!='whole phrase') {
				$wildcard = "";
				
				if ($searchTermType=='word') {
					$wildcard = "*";
				}
	
				$sql_match3 .= " (";
			
				foreach ($fields as $field) {
					if ($field['weighting']) {
						if ($field['name']=="alias" || $field['name']=="type") {
							$sql_match3 .= "(" . $field['name'] . " LIKE '" . mysql_real_escape_string($searchTerm) . "%') OR ";
						} else {
							$sql_match3 .= "(MATCH (" . $field['name'] . ") AGAINST ('" . mysql_real_escape_string($searchTerm) . "" . $wildcard . "' IN BOOLEAN MODE)) OR ";
						}
					}
				}
				
				if (substr($sql_match3,-3,2)=="OR") {
					$sql_match3 = substr($sql_match3,0,strlen($sql_match3)-3);
				}
				
				$sql_match3 .= ") OR ";			
			}
		}	

		if ($sql_match3) {
			$sql_match2 .= " AND (";
			
			if (substr($sql_match3,-3,2)=="OR") {
				$sql_match3 = substr($sql_match3,0,strlen($sql_match3)-3);
			}

			$sql_match2 .= $sql_match3 . ") ";
		}	
	
		$sql .= $sql_match1 . $sql_match2 ." GROUP BY id HAVING relevance > 0 ORDER by relevance DESC";
	
		$result = my_mysql_query($sql);
		
		$recordCount = mysql_num_rows($result);
		
		if ($recordCount > 0) {
			
			if ($this->setting('use_pagination')) {
				
				$pageSize = (int) $this->setting('page_size');
				
				$numberOfPages = ceil($recordCount/$pageSize);
				
				for ($i=1;$i<=$numberOfPages;$i++) {
					$pagination[$i] = '&page='. $i. '&tab='. $cType. '&searchString='. rawurlencode($this->searchString);
				}
				
				if ($this->page == 1) {
					$record_number = 1;
				} else {
					$record_number = (($this->page - 1) * $pageSize) + 1;
				}
				
				$sql .= "
						LIMIT " . ($record_number-1) . "," . $pageSize;
						
				$result = my_mysql_query($sql);
			} else {
				$pagination = false;
			}
		
			while ($row = mysql_fetch_assoc($result)) {
				$searchresults[] = $row;
			}
		}
		
			
		return array(
			"Type" => $this->phrase( '_' . $cType),
			"Record_Count" => $recordCount,
			"search_results" => $searchresults,
			"pagination" => $pagination,
			"offset" => $record_number,
			"Tab_On" => $cType == $this->cTypeToSearch? '_on' :null,
			"Tab_Onclick" => $this->refreshPluginSlotAnchor('&tab='. $cType. '&searchString='. rawurlencode($this->searchString))
		);
	
	}
	
	
	
	
	
	function getSearchtermParts($searchString) {
		$searchString = str_replace('+', '', rawurldecode($searchString));
		
		//Break the search string up into tokens.
		//Normally we break by spaces, but you can use a pattern in double quotes to override this
		preg_match_all('/"([^"]+)"|(\S+)/', $searchString, $searchStrings, PREG_SET_ORDER);
		
		$quotesUsed = false;
		$searchWordsAndPhrases = array();
		foreach($searchStrings as $string) {
			if (isset($string[2])) {
				$searchWordsAndPhrases[$string[2]] = 'word';
			} else {
				$searchWordsAndPhrases[$string[1]] = 'phrase';
				$quotesUsed = true;
			}
		}
		
		//Just in case the user doesn't know about the "using quotes to group words together" feature,
		//add the whole phrase in as a search term
		if (!$quotesUsed && count($searchStrings > 1)) {
			$searchWordsAndPhrases[$searchString] = 'whole phrase';
		}
		
		return $searchWordsAndPhrases;
	}
	
	
	//If a page is created with this Plugin as the Featured Plugin,
	//on a Language without a Search Page set, set this as the Search Page.
	public static function eventQuickAddContentItem($cID, $instance) {
		if ($instance['class_name'] == 'tribiq_search_results') {
			$sql = "
				UPDATE ". DB_NAME_PREFIX. "languages SET
					search_page_id = ". (int) $cID. "
				WHERE id = '". mysql_real_escape_string(getContentLang($cID, 'html')). "'
				  AND search_page_id = 0";
			my_mysql_query($sql);
		}
	}
	
}
?>
