<?php
if (!defined('NOT_ACCESSED_DIRECTLY')) exit;
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */




class tribiq_twitter extends plugin_base_class {

	private $p;
	private $atomEntries = array();
	private $elementStack = array();

	public function __construct() {
		$this->p = xml_parser_create();
		xml_parser_set_option($this->p, XML_OPTION_CASE_FOLDING, 0);
		xml_parser_set_option($this->p, XML_OPTION_SKIP_WHITE, 1);

		xml_set_object($this->p, $this);
		xml_set_element_handler($this->p, 'openingTagHandler', 'closingTagHandler');
		xml_set_character_data_handler($this->p, 'characterDataHandler');
	}

	function init() {
		$this->forcePageReload();
		return true;
	}

	private function openingTagHandler($p, $tag_name, $attributes) {
		if($tag_name == 'entry')
			array_push($this->atomEntries, array());

		array_push($this->elementStack, $tag_name);
	}

	private function closingTagHandler($p, $name) {
		array_pop($this->elementStack);
	}

	private function characterDataHandler($p, $data) {
		if($this->elementStack[count($this->elementStack) - 2] == 'entry') {
			$this->atomEntries[count($this->atomEntries) - 1][end($this->elementStack)] .= htmlspecialchars( trim($data) );
		}
	}

	private function getCachedTwitter($twitterUrl) {
		list($result) = mysql_fetch_row(my_mysql_query('
			SELECT result
			FROM '.DB_NAME_PREFIX.TRIBIQ_TWITTER_PREFIX.'cache
			WHERE request = "'.mysql_real_escape_string($twitterUrl).'"
			AND DATE_ADD(last_cached_timestamp, INTERVAL  '.mysql_real_escape_string($this->setting('minutes_to_cache_result')).' MINUTE) > NOW()
		'));

		return $result;
	}

	private function getLiveTwitter($twitterUrl) {
		$twitterFeed = @file_get_contents($twitterUrl);
		if ( ! $twitterFeed  ) { 
			$twitterFeed = '<?xml version="1.0" encoding="UTF-8"?>
			<error>
				<item>
					<content>Error reading Twitter data</content>
					<admin_message>from ' . htmlentities( $twitterUrl ) . '.</admin_message>
					<published>' . date( 'Y-m-d H:i:s' ) . '</published>
				</item>
			</error>';
		}
		

		my_mysql_query('
			REPLACE INTO '.DB_NAME_PREFIX.TRIBIQ_TWITTER_PREFIX.'cache
			SET
				request = "'.mysql_real_escape_string($twitterUrl).'",
				last_cached_timestamp = NOW(),
				result = "'.mysql_real_escape_string($twitterFeed).'"
		');

		return $twitterFeed;
	}

	function showSlot() {
		$twitterUrl = 'http://twitter.com/statuses/user_timeline/' . $this->setting('twitter_username') . '.atom?count=' . (int) $this->setting('number_tweets_to_show');
		$xml = $this->getCachedTwitter($twitterUrl);
		if($xml == null) 
			$xml = $this->getLiveTwitter($twitterUrl);
		xml_parse($this->p, $xml);
		xml_parser_free($this->p);
		
		$pageMergeFields = array(
			'Title' => $this->setting('feed_title'),
			'TwitterUserName' => $this->setting('twitter_username'),
			'FollowMeLink' => $this->setting('follow_link_text')
		);
		$subSections = array( 'Content_Section' => true );
		if ( ! $this->setting( 'show_date_time' ) ) {
			$subSections['Date_Section'] = false; 
			$date = '';
		} else {
			$subSections['Date_Section'] = true; 
			if ( $this->setting('date_format') == '_SHORT' ) {
				$dateFormat = getSiteConfig('vis_date_format_short');
			} elseif ( $this->setting('date_format') == '_LONG' ) {
				$dateFormat = getSiteConfig('vis_date_format_long');
			} else {
				$dateFormat = getSiteConfig('vis_date_format_med');
			}
		}
		$this->frameworkHead('TwitterFeed', 'Tweet' );
		$this->framework('Header', $pageMergeFields );
			if (sizeof($this->atomEntries)>0) {
				foreach ($this->atomEntries as $item) {
					if ( $this->setting('show_date_time') ) {
						$date = formatDateTimeNicely( date( 'Y-m-d H:i:s', strtotime( $item['published'] ) ), $dateFormat );
					}
					if ( array_key_exists( 'admin_message', $item ) && checkPriv()  ) {
						 $item['content'] .= ' ' . $item['admin_message'];
					}
					$item['content']= preg_replace("/(ht|f)tp(s){0,1}\:\/\/([-\w]+\.)+(com|org|net|gov|mil|biz|info|mobi|name|aero|jobs|museum|travel|[a-z]{2}){0,1}[\/\&\?\%\=A-Za-z0-9\;\:\,\.\_\-]+/",'<a href="$0">$0</a>',$item['content']);
	
					$tweetMergeFields = array( 'Content' => $item['content'], 'Date' => $date );
					$this->framework( 'Tweet', $tweetMergeFields , $subSections );
				}
			} else {
				$tweetMergeFields = array( 'Content' => $this->phrase("_FEED_UNAVAILABLE"), 'Date' => $date );
				$this->framework( 'Tweet', $tweetMergeFields , $subSections );
			}
		$this->framework( 'Footer', $pageMergeFields );
		$this->frameworkFoot('TwitterFeed', 'Tweet' );
	}
}
?>
