<?php
/*
 * This file is part of Tribiq CMS - a web-based Content Management System
 * and Extranet. Copyright (c) 2007-2010 Tribal Limited, registered in England.
 * For more information please visit http://tribiq.com.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License 
 * as published by the Free Software Foundation, either version 3 of the
 * Licence, or (at your option) any later version.
 *
 * Please support Free Software. If you use this program please credit
 * the authors by placing a link "Powered by Tribiq CMS" to 
 * http://tribiq.com in the foot of all your templates.
 *
 * Tribiq CMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org.licenses/>.
 */

require '../../adminheader.inc.php';
require CMS_ROOT. 'cmsincludes/thickbox.inc.php';

exitIfNotCheckPriv('_PRIV_VIEW_MENU_ITEM');
if ($_POST['mID']) {
	$hasEditPermission = $_POST['parentMenuID']? checkPriv('_PRIV_EDIT_MENU_ITEM') : checkPriv('_PRIV_EDIT_TOP_LEVEL_MENU_ITEM');
} else {
	$hasEditPermission = $_POST['parentMenuID']? checkPriv('_PRIV_ADD_MENU_ITEM') : checkPriv('_PRIV_ADD_TOP_LEVEL_MENU_ITEM');
}

$noSaving = false;
$noValidation = false;
$showEditButton = $hasEditPermission && $_POST['mID'];
$editMode = $_POST['formMode']=="edit" || !$_POST['mID'];
require 'includes/common_head.inc.php';


//Saving data
if ($_POST['save']=="true") {
	
	$menuId = $_POST['mID'];
	
	saveMenuDetails($_POST, $menuId);
	
	//Overwrite the menuId in the primary key if needed
	echo json_encode(array('mID' => $menuId));
	

//Check if a form is valid
} elseif ($_POST['validate']) {

	echo json_encode(validateMenuDetails($_POST));
	

//Call the display form function.
} else {
	
	echo getPhrase('Use this screen to set the Name for this Menu Item, and preview how it will look in the Breadcrumb Trail.');
	
	$dbDetails = array();
	$fields = $_POST;
	//Load details from the database if an id has been given
	if ($_POST['mID']) {
		$dbDetails = getMenuItemFromMenuID($_POST['mID']);
		
		foreach ($dbDetails as $field => $value) {
			if (!isset($fields[$field])) {
				$fields[$field] = $value;
			}
		}
	//Otherwise for new menu items, set the requested section_id, language_id and parent_id
	} elseif (!$fields['section_id']) {
		$fields['parent_id'] = $_POST['parentMenuID'];
		$fields['section_id'] = $_POST['sectionId'];
		$fields['language_id'] = $_POST['languageId'];
	}
	
	
	
	thickboxForm($formHTMLHead, $formHTMLFoot, $editMode, $showEditButton);
	
	echo $formHTMLHead, '
		<tr>
			<td class="title" style="vertical-align:bottom;">
				<label>', getPhrase('Section:'), '</label>
			</td><td>
				', htmlspecialchars($fields['section_id']), '
			</td>
		</tr>';


	//Get a breadcrumb trail of menu items leading to the current menu item
	$menuArray =
		cms_core::getMenuStructure($fields['section_id'], $fields['parent_id'], 0,
											 999, 999, $fields['language_id'],
											 true, true, false, true);
	
	//Display each menu item up until the current menu item
	$level = 0;
	while ($menuArray) {
		echo '
			<tr>
				<td class="title" style="vertical-align:bottom;">
					<label>', getPhrase('_LEVEL_N:', array('n' => ++$level)), '</label>
				</td><td>
					', htmlspecialchars($menuArray[0]['name']), '
				</td>
			</tr>';
		
		$menuArray = $menuArray[0]['children'];
	}
	
	//Display the current menu item in the same format, however let the admin edit the name in edit mode
	echo '
			<tr>
				<td class="title" style="vertical-align:bottom;">
					<label>', getPhrase('_LEVEL_N:', array('n' => ++$level)), '</label>
				</td><td class="', $fields['name']? 'mandatory_correct' : 'mandatory', '">';
	
	if (!$editMode) {
		echo htmlspecialchars($fields['name']);
		echo '
					<input type="hidden" id="originalName" value="', htmlspecialchars($fields['name']), '" />';
	} else {
		echo '
					<input type="text" class="form_field" id="name" name="name" value="', htmlspecialchars($fields['name']), '" onChange="if (tribiqAFB.pk) tribiqAFB.pk.currentName = this.value;" />
					<input type="hidden" id="parent_id" name="parent_id" value="', $fields['parent_id'], '" />
					<input type="hidden" id="section_id" name="section_id" value="', $fields['section_id'], '" />
					<input type="hidden" id="language_id" name="language_id" value="', $fields['language_id'], '" />
					<div class="icon_field"/>';
	}
	
	//For existing menu items, show a button to launch the storekeeper and find a new parent
	if ($editMode && $_POST['mID']) {
		
		//Build up a link to the current parent in Storekeeper
		if ($fields['parent_id']) {
			$path = getMenuItemStorekeeperDeepLink($fields['parent_id']);
		} else {
			$path = 'tribiq__menu/item//'. $fields['language_id']. '//item//'. $fields['section_id'];
		}
		
		echo '
				</td>
			</tr>
			<tr>
				<td></td>
				<td>
					<input type="button" class="launch_keeper" value="', getPhrase('_CHANGE_MENU_ITEMS_PARENT'), '" onClick="
						tribiqA.SK(
							\'tribiqTab\', \'setMenuParent\', false,
							\'', jsOnClickEscape($path), '\',
							\'tribiq__menu/menu_items\', \'tribiq__menu/menu_items\', \'tribiq__menu/menu_items\');" />';
		
		//If this menu item isn't on the top level, show controls to move the menu item there
		if ($level > 1) {
			echo '
					<input type="button" class="submit" value="', getPhrase('_MOVE_MENU_ITEM_TO_TOP_LEVEL'), '" onClick="tribiqTab.setMenuToTopLevel();" />';
		}
	}
	
	echo '
				</td>
			</tr>
		
		', $formHTMLFoot;
}
?>